/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "configstore.h"

/**
 * This is the main store object and contains all
 * settings.
 */
QHash<QString, QString> ConfigStore::store;

/** 
 * These preferences will be stored inside the .pwsrc file.
 */
QString ConfigStore::params[] = { "prefs_pws_clpbrd_timeout", 
                                  "prefs_pws_recentfiles", 
                                  "prefs_pws_maxrecentfiles", 
                                  "prefs_pws_show_grid", 
                                  "prefs_pws_confirm_delete", 
                                  "prefs_pws_confirm_clpbrd",
                                  "prefs_pws_save_backups",
                                  "prefs_pws_save_num_backups",
                                  "prefs_pws_doubleclick_action",
                                  "prefs_pws_enter_action",
                                  "prefs_pws_space_action",
                                  "prefs_pws_enable_idle_timeout",
                                  "prefs_pws_cur_dir",
                                  "prefs_pws_pos_x",
                                  "prefs_pws_pos_y",
                                  "prefs_pws_height",
                                  "prefs_pws_width",
                                  "prefs_pws_date_format",
                                  "prefs_pws_showownlyrecentname",
                                  "prefs_pws_scroll_hint",
                                  "prefs_pws_save_backups_dir" };

/**
 * Loads Preferences in file at path.
 */
void ConfigStore::loadConfig (const QString & path)
{
    defaults();

    QFile file(path);
    if (!file.open (QIODevice::ReadOnly))
    {
        std::cerr << QObject::tr("Problem reading config file (").toAscii().data() 
                  << path.toAscii().data() << ")." << std::endl;

        if (getenv("HOMESHARE") && getenv ("HOMEPATH"))
            { store["configfile"] = QString (getenv("HOMESHARE")) + QString(getenv("HOMEPATH")); }
        else if (getenv("HOMEDRIVE") && getenv ("HOMEPATH"))
            { store["configfile"] = QString (getenv("HOMEDRIVE")) + QString(getenv("HOMEPATH")); }
        else if (getenv ("HOME"))
            { store["configfile"] = QString (getenv("HOME")); }

        store["configfile"] += QString(QDir::separator())  + ".pwsrc";
        return;
    }

    QTextStream stream (&file);
    while (!stream.atEnd())
    {
        QString line = stream.readLine();
        line = line.trimmed();

        if (!line.startsWith ("="))
        {
            QStringList list = line.split("=", QString::SkipEmptyParts);
            if (list.size() == 2 && validParam (list[0].trimmed()))
                { store[list[0].trimmed()] = list[1].trimmed(); }
            else if (list.size() == 1 && validParam (list[0].trimmed()))
                { store[list[0].trimmed()] = ""; }
            else
            { 
                std::cerr << QObject::tr("Unknown configuration parameter (").toAscii().data() 
                          << path.toAscii().data() << "): " << line.toAscii().data() << std::endl;
            }
        }
    }
    store["configfile"] = path;

    file.close();
}

/**
 * Adds a recent file entry.
 */
void ConfigStore::addRecentFile (const QString & file)
{
    if (file != "")
    {
        QStringList files = store["prefs_pws_recentfiles"].split("$");
        store["prefs_pws_recentfiles"] = file;
        for (int x = 0; (x < files.size()) && (x < store["prefs_pws_maxrecentfiles"].toInt()); x++)
        {
            if (files.at(x) != file && files.at(x) != "")
                { store["prefs_pws_recentfiles"] += "$" + files.at(x); }
        }
    }
}

/**
 * Set default values.
 *
 * @param type - Whether all perferences, only application specific
 *               preferences or only pws database specific preferences
 *               should be resetted.
 */
void ConfigStore::defaults (ConfigStore::PREFS type)
{
    if (type == ALL || type == PWS)
    {
        // prefs stored inside the password safe
        store["prefs_bool_showpwdefault"]                = "0";
        store["prefs_bool_showpasswordintree"]           = "0"; /* not implemented */
        store["prefs_bool_sortascending"]                = "0"; /* not implemented */
        store["prefs_bool_usedefuser"]                   = "0";
        store["prefs_bool_saveimmediately"]              = "0";
        store["prefs_bool_pwuselowercase"]               = "1";
        store["prefs_bool_pwuseuppercase"]               = "1";
        store["prefs_bool_pwusedigits"]                  = "1";
        store["prefs_bool_pwusesysmbols"]                = "0";
        store["prefs_bool_pwusehexdigit"]                = "0";
        store["prefs_bool_pweasyvision"]                 = "0";
        store["prefs_bool_displayexpandedaddeditdialog"] = "0"; /* not implemented */
        store["prefs_bool_maintaindatetimestamps"]       = "0";
        store["prefs_bool_savepasswordhistory"]          = "0";
        store["prefs_bool_shownotesdefault"]             = "0"; /* not implemented */
        store["prefs_bool_showusernameintree"]           = "0"; /* not implemented */
        store["prefs_int_pwlendefault"]                  = "8";
        store["prefs_int_idletimeout"]                   = "5";
        store["prefs_int_treedisplaystatusatopen"]       = "0"; /* not implemented */
        store["prefs_int_numpwhistorydefault"]           = "3";
        store["prefs_str_defaultusername"]               = "";
        store["prefs_str_defaultautootypestring"]        = "\\u\\t\\p\\n";

        // pwsv3 header prefs
        store["pws_hdr_uuid"]          = "";
        store["pws_hdr_last_uid"]      = "";
        store["pws_hdr_last_modified"] = "0";
        store["pws_hdr_last_app"]      = "";
        store["pws_hdr_last_host"]     = "";
        store["pws_hdr_db_name"]       = "";
        store["pws_hdr_db_desc"]       = "";
        store["pws_hdr_version"]       = "";
    }
    
    if (type == ALL || type == APP)
    {
        // special prefs
        store["curfile"]  = "";
        store["readonly"] = "0";

        // application specific prefs
        if (store["prefs_pws_maxrecentfiles"].isEmpty()) store["prefs_pws_maxrecentfiles"] = "3";
        if (store["prefs_pws_show_grid"].isEmpty())      store["prefs_pws_show_grid"]      = "1";
        if (store["prefs_pws_confirm_delete"].isEmpty()) store["prefs_pws_confirm_delete"] = "1";
        if (store["prefs_pws_confirm_clpbrd"].isEmpty()) store["prefs_pws_confirm_clpbrd"] = "0";
        if (store["prefs_pws_save_backups"].isEmpty())   store["prefs_pws_save_backups"]   = "1";
        if (store["prefs_pws_save_num_backups"].isEmpty())    store["prefs_pws_save_num_backups"] = "";
        if (store["prefs_pws_save_backups_dir"].isEmpty())    store["prefs_pws_save_backups_dir"] = "";
        if (store["prefs_pws_showownlyrecentname"].isEmpty()) store["prefs_pws_showownlyrecentname"] = "1";
        if (store["prefs_pws_clpbrd_timeout"].isEmpty())      store["prefs_pws_clpbrd_timeout"] = "30";
        if (store["prefs_pws_doubleclick_action"].isEmpty())  store["prefs_pws_doubleclick_action"] = "1";
        if (store["prefs_pws_enter_action"].isEmpty())        store["prefs_pws_enter_action"] = "1";
        if (store["prefs_pws_space_action"].isEmpty())        store["prefs_pws_space_action"] = "1";
        if (store["prefs_pws_enable_idle_timeout"].isEmpty()) store["prefs_pws_enable_idle_timeout"] = "1";
        if (store["prefs_pws_date_format"].isEmpty())         store["prefs_pws_date_format"] = "yyyy-MM-dd hh:mm:ss";
        if (store["prefs_pws_scroll_hint"].isEmpty())         store["prefs_pws_scroll_hint"] = "0";

        if (store["prefs_pws_cur_dir"].isEmpty()) 
        {
            if (getenv("HOMESHARE") && getenv ("HOMEPATH"))
                { store["prefs_pws_cur_dir"] = QString (getenv("HOMESHARE")) + QString(getenv("HOMEPATH")); }
            else if (getenv("HOMEDRIVE") && getenv ("HOMEPATH"))
                { store["prefs_pws_cur_dir"] = QString (getenv("HOMEDRIVE")) + QString(getenv("HOMEPATH")); }
            else if (getenv ("HOME"))
                { store["prefs_pws_cur_dir"] = QString (getenv("HOME")); }
        }
    }
}

/**
 * Save the configuration into store["configfile"].
 */
void ConfigStore::saveConfig ()
{
    QFile file(store["configfile"]);
    if (!file.open (QIODevice::WriteOnly|QIODevice::Truncate))
    {
        std::cerr << QObject::tr("Problem writing config file (").toAscii().data() 
                  << store["configfile"].toAscii().data() << ")." << std::endl;
        return;
    }

    QTextStream stream (&file);

    QHashIterator<QString, QString> param(store);
    while (param.hasNext()) 
    {
        param.next();
        if (validParam(param.key()))
        {   
            if (param.value() == "" && validParam (param.key()))
                { stream << param.key() << endl; }
            else if (validParam (param.key()))
                { stream << param.key() << " = " << param.value() << endl; }
        }
    }

    file.close();
}

/**
 * Returns true if param is a application specific preference which
 * should be saved inside the configuration file.
 */
bool ConfigStore::validParam (const QString & param)
{
    for (unsigned int i = 0; i < sizeof(params)/sizeof(QString); i++)
    {
        if (ConfigStore::params[i] == param)
            { return true; }
    }
    return false;
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
