/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "mainwindow.h"

/**
 * This class represents the main window widget.
 */
MainWindow::MainWindow()
{
    clpbrd_contents = new QString;
    fileOpen = false;

    // set up the clipboard-, selection- and idle-timer
    c_timer = new QTimer(this);
    s_timer = new QTimer(this);
    i_timer = new QTimer(this);

    // set up the model/view
    view     = new PWSTreeView(this);
    model    = new PWSTreeModel(view);
    delegate = new EntryItemDelegate(view);

    view->setEditTriggers (QAbstractItemView::EditKeyPressed|QAbstractItemView::SelectedClicked);
    view->setModel (model);
    view->setItemDelegate (delegate);
    view->setSelectionBehavior (QAbstractItemView::SelectRows);
    view->setAllColumnsShowFocus (true);

    // enable drag and drop
    view->setDragEnabled(true);
    view->setAcceptDrops(true);
    view->setDropIndicatorShown(false);

    pwsHandler = new PWSHandler (model);

    // recentFileActs will hold the last recent files
    recentFileActs = (QAction**)malloc(ConfigStore::store["prefs_pws_maxrecentfiles"].toInt()*sizeof(QAction*));
    recentFileActsLen = ConfigStore::store["prefs_pws_maxrecentfiles"].toInt();

    if (ConfigStore::store["prefs_pws_show_grid"] == "1")
        { view->setAlternatingRowColors (true); }

    setCentralWidget(view);

    // setup all actions, menus, toolbars and the statusbar
    createActions();
    createMenus();
    createToolBars();
    createStatusBar();

    // connect the model/view objects
    connect(view,  SIGNAL(expanded(const QModelIndex&)),          this,  SLOT(slot_resizeColumn(const QModelIndex&)));
    connect(view,  SIGNAL(doubleClicked(const QModelIndex &)),    this,  SLOT(slot_doubleClicked(const QModelIndex &)));
    connect(view,  SIGNAL(expanded(const QModelIndex &)),         model, SLOT(expanded(const QModelIndex &)));
    connect(view,  SIGNAL(collapsed(const QModelIndex &)),        model, SLOT(collapsed(const QModelIndex &)));
    connect(view,  SIGNAL(expanded(const QModelIndex &)),         this,  SLOT(slot_expanded(const QModelIndex &)));
    connect(view,  SIGNAL(clicked(const QModelIndex &)),          this,  SLOT(slot_clicked(const QModelIndex &)));
    connect(view,  SIGNAL(dropSuccessfull()),                     this,  SLOT(slot_fileModified()));

    connect(view->selectionModel(), SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)),  this, SLOT(slot_currentChanged(const QModelIndex &)));

    connect(model, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this,  SLOT(slot_fileModified()));
    connect(model, SIGNAL(sorted(EntryItem*)),                    this,  SLOT(slot_sorted(EntryItem*)));

    // connect the idle-timer and the event-watchdog
    connect(i_timer,            SIGNAL(timeout()),  this,  SLOT(slot_lock()));
    connect(WatchDog::watchdog, SIGNAL(hit()),      this,  SLOT(slot_resetTimer()));

    // customizable space key action
    spaceShortcut = new QShortcut(QKeySequence(tr("Space", "")), this);
    connect(spaceShortcut,  SIGNAL(activated()), this,  SLOT(slot_spacePressed()));

    // these ones are needed to open a folder with the enter key
    enterShortcut  = new QShortcut(Qt::Key_Enter, this, SLOT(slot_enterPressed()));
    returnShortcut = new QShortcut(Qt::Key_Return, this, SLOT(slot_enterPressed()));

    connect(delegate, SIGNAL(closeEditor(QWidget*,QAbstractItemDelegate::EndEditHint)), this,  SLOT(slot_enableEnterShortcut()));
    connect(delegate, SIGNAL(openEditor()), this,  SLOT(slot_disableEnterShortcut()));

    // and finally setup the window flags
    setWindowTitle ("pws");
    setWindowFlags (Qt::Window);
    setStatus(MainWindow::FileClosed);
}

/**
 * Delete clpbrd_contents.
 */
MainWindow::~MainWindow()
{
    delete clpbrd_contents;
    delete pwsHandler;
    free (recentFileActs);
}

/**
 * Returns the PWSHandler object.
 *
 * @returns The PWSHandler object.
 */
PWSHandler * MainWindow::handler()
{
    return pwsHandler;
}

/**
 * This method will be called whenever the user requests the contextmenu.
 */
void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu* menu = new QMenu(this);

    menu->addAction(importAct);
    menu->addAction(exportAct);
    menu->addSeparator();
    menu->addAction(addPwdAct);
    menu->addAction(addGroupAct);
    menu->addAction(delEntryAct);
    menu->addAction(renameAct);
    menu->addAction(propAct);
    menu->addSeparator();
    menu->addAction(fetchPwdAct);
    menu->addAction(fetchUsrAct);
    menu->addAction(fetchUrlAct);
    menu->addAction(openUrlAct);
    menu->addSeparator();
    menu->addAction(clearClipboardAction);

    // set the current index to the index at the current mouse position
    QModelIndex index = view->indexAt(view->viewport()->mapFromGlobal(event->globalPos()));
    view->setCurrentIndex (model->index (index.row(), 0, index.parent()));
    menu->exec(event->globalPos());
}

/**
 * Sets the current window/database status to on of the
 * following values:
 *
 * MainWindow::FileModified - The database was modified.
 * MainWindow::FileSaved    - The database was saved.
 * MainWindow::FileNew      - A new database was created.
 * MainWindow::FileOpened   - A existing database was opened.
 * MainWindow::FileClosed   - The database was closed.
 *
 * @param status - The new database status.
 */
void MainWindow::setStatus (MainWindow::PWSFileStatus status)
{
    switch (status)
    {
        // database was modified - enable save-actions
        case MainWindow::FileModified:
            if (ConfigStore::store["readonly"] != "1")
            {
                if (ConfigStore::store["prefs_bool_saveimmediately"] == "1" && ConfigStore::store["curfile"] != "")
                {
                    modified = true;
                    slot_save();
                }
                else
                {
                    modified = true;
                    saveAct->setEnabled(true);

                    if (ConfigStore::store["pws_hdr_db_name"] != "")
                    {
                        setWindowTitle ("pws - " + ConfigStore::store["curfile"] + " [" + ConfigStore::store["pws_hdr_db_name"] + "]" + " [+]");
                    }
                    else if (ConfigStore::store["curfile"] != "")
                    {
                        setWindowTitle ("pws - " + ConfigStore::store["curfile"] + " [+]");
                    }
                    else
                    {
                        setWindowTitle ("pws - New File [+]");
                    }
                    emit fileModified();
                }
            }
            break;
        // database was saved - disable save-actions
        case MainWindow::FileSaved:
            modified = false;
            saveAct->setEnabled(false);
            if (ConfigStore::store["pws_hdr_db_name"] != "")
            {
                setWindowTitle ("pws - " + ConfigStore::store["curfile"] + " [" + ConfigStore::store["pws_hdr_db_name"] + "]");
            }
            else
            {
                setWindowTitle ("pws - " + ConfigStore::store["curfile"]);
            }
            emit fileSaved();
            break;
        // new database - enable default-actions
        case MainWindow::FileNew:
            saveAct->setEnabled(true);
            importAct->setEnabled(true);
            exportAct->setEnabled(true);
            closeAct->setEnabled(true);
            saveAsAct->setEnabled(true);
            chPwdAct->setEnabled(true);
            addPwdAct->setEnabled(true);
            addGroupAct->setEnabled(true);
            delEntryAct->setEnabled(true);
            renameAct->setEnabled(true);
            infoAct->setEnabled(true);
            propAct->setEnabled(true);
            fetchPwdAct->setEnabled(true);
            fetchUsrAct->setEnabled(true);
            fetchUrlAct->setEnabled(true);
            lockAct->setEnabled(true);
            openUrlAct->setEnabled(true);
            clearClipboardAction->setEnabled(true);

            fileOpen = true;
            modified = true;

            if (ConfigStore::store["prefs_pws_enable_idle_timeout"] != "0")
                { i_timer->start (ConfigStore::store["prefs_int_idletimeout"].toInt()*60*1000); }

            setWindowTitle ("pws - New File [+]");
            emit fileNew();
            break;
        // existing database was opened - enable default-actions but disable save-action
        case MainWindow::FileOpened:
            if (ConfigStore::store["readonly"] != "1")
            {
                importAct->setEnabled(true);
                chPwdAct->setEnabled(true);
                addPwdAct->setEnabled(true);
                addGroupAct->setEnabled(true);
                delEntryAct->setEnabled(true);
                renameAct->setEnabled(true);
            }
            else
            {
                importAct->setEnabled(false);
                chPwdAct->setEnabled(false);
                addPwdAct->setEnabled(false);
                addGroupAct->setEnabled(false);
                delEntryAct->setEnabled(false);
                renameAct->setEnabled(false);
            }
            exportAct->setEnabled(true);
            saveAsAct->setEnabled(true);
            closeAct->setEnabled(true);
            infoAct->setEnabled(true);
            propAct->setEnabled(true);
            fetchPwdAct->setEnabled(true);
            fetchUsrAct->setEnabled(true);
            fetchUrlAct->setEnabled(true);
            openUrlAct->setEnabled(true);
            clearClipboardAction->setEnabled(true);
            saveAct->setEnabled(false);
            lockAct->setEnabled(true);

            fileOpen = true;
            modified = false;

            if (ConfigStore::store["pws_hdr_db_name"] != "")
            {
                if (ConfigStore::store["readonly"] == "1")
                    { setWindowTitle ("pws - " + ConfigStore::store["curfile"] + " [" + ConfigStore::store["pws_hdr_db_name"] + "]" + " [ro]"); }
                else
                    { setWindowTitle ("pws - " + ConfigStore::store["curfile"] + " [" + ConfigStore::store["pws_hdr_db_name"] + "]"); }
            }
            else
            {
                if (ConfigStore::store["readonly"] == "1")
                   { setWindowTitle ("pws - " + ConfigStore::store["curfile"] + " [ro]"); }
                else
                   { setWindowTitle ("pws - " + ConfigStore::store["curfile"]); }
            }

            ConfigStore::addRecentFile (ConfigStore::store["curfile"]);
            updateRecentFileActions();

            if (ConfigStore::store["prefs_pws_enable_idle_timeout"] != "0")
                { i_timer->start (ConfigStore::store["prefs_int_idletimeout"].toInt()*60*1000); }

            emit fileOpened();
            break;
        // database closed - disable default actions
        case MainWindow::FileClosed:
            importAct->setEnabled(false);
            exportAct->setEnabled(false);
            closeAct->setEnabled(false);
            saveAct->setEnabled(false);
            saveAsAct->setEnabled(false);
            chPwdAct->setEnabled(false);
            infoAct->setEnabled(false);
            addPwdAct->setEnabled(false);
            addGroupAct->setEnabled(false);
            delEntryAct->setEnabled(false);
            renameAct->setEnabled(false);
            propAct->setEnabled(false);
            fetchPwdAct->setEnabled(false);
            fetchUsrAct->setEnabled(false);
            fetchUrlAct->setEnabled(false);
            openUrlAct->setEnabled(false);
            clearClipboardAction->setEnabled(false);
            lockAct->setEnabled(false);

            fileOpen = false;
            modified = false;

            i_timer->stop();

            updateRecentFileActions();
            setWindowTitle ("pws");

            emit fileClosed();
            break;
    }
}

/**
 * Creates all actions.
 */
void MainWindow::createActions()
{
    // File
    newAct = new QAction(QIcon(":/images/new.png"), tr("&New"), this);
    newAct->setShortcut(tr("Ctrl+N"));
    newAct->setStatusTip(tr("Create a new passwordsafe"));
    connect(newAct, SIGNAL(triggered()), this, SLOT(slot_newFile()));

    openAct = new QAction(QIcon(":/images/open.png"), tr("&Open"), this);
    openAct->setShortcut(tr("Ctrl+O"));
    openAct->setStatusTip(tr("Open an existing passwordsafe"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(slot_open()));

    importAct = new QAction(QIcon(":/images/import.png"), tr("&Import database"), this);
    importAct->setShortcut(tr("Ctrl+I"));
    importAct->setStatusTip(tr("Import an existing passwordsafe"));
    connect(importAct, SIGNAL(triggered()), this, SLOT(slot_import()));

    exportAct = new QAction(QIcon(":/images/export.png"), tr("&Export subtree"), this);
    exportAct->setShortcut(tr("Ctrl+E"));
    exportAct->setStatusTip(tr("Export a subtree"));
    connect(exportAct, SIGNAL(triggered()), this, SLOT(slot_export()));

    saveAct = new QAction(QIcon(":/images/save.png"), tr("&Save"), this);
    saveAct->setShortcut(tr("Ctrl+S"));
    saveAct->setStatusTip(tr("Save the passwordsafe"));
    saveAct->setEnabled(false);
    connect(saveAct, SIGNAL(triggered()), this, SLOT(slot_save()));

    saveAsAct = new QAction(QIcon(":/images/saveas.png"), tr("Save &As"), this);
    saveAsAct->setShortcut(tr("Ctrl+Shift+S"));
    saveAsAct->setStatusTip(tr("Save the passwordsafe under a new name"));
    saveAsAct->setEnabled(false);
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(slot_saveAs()));

    closeAct = new QAction(tr("&Close"), this);
    closeAct->setShortcut(tr("Ctrl+C"));
    closeAct->setStatusTip(tr("Close the passwordsafe"));
    closeAct->setEnabled(false);
    connect(closeAct, SIGNAL(triggered()), this, SLOT(slot_closeFile()));

    chPwdAct = new QAction(tr("Change &passphrase"), this);
    chPwdAct->setStatusTip(tr("Change the safe's passphrase"));
    chPwdAct->setEnabled(false);
    connect(chPwdAct, SIGNAL(triggered()), this, SLOT(slot_changePassphrase()));

    infoAct = new QAction(QIcon(":/images/info.png"), tr("&Information"), this);
    infoAct->setStatusTip(tr("Information about the passwordsafe file"));
    infoAct->setEnabled(false);
    connect(infoAct, SIGNAL(triggered()), this, SLOT(slot_information()));

    lockAct = new QAction(QIcon(":/images/lock.png"), tr("&Lock"), this);
    lockAct->setStatusTip(tr("Lock the passwordsafe"));
    lockAct->setEnabled(false);
    connect(lockAct, SIGNAL(triggered()), this, SLOT(slot_lock()));

    exitAct = new QAction(QIcon(":/images/exit.png"), tr("E&xit"), this);
    exitAct->setShortcut(tr("Ctrl+Q"));
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(slot_exit()));

    for (int i = 0; i < ConfigStore::store["prefs_pws_maxrecentfiles"].toInt(); i++) 
    {
        recentFileActs[i] = new QAction(this);
        recentFileActs[i]->setVisible(false);
        connect(recentFileActs[i], SIGNAL(triggered()), this, SLOT(slot_openRecent()));
    }

    // Edit
    addPwdAct = new QAction(QIcon(":/images/entryadd.png"), tr("&Add entry"), this);
    addPwdAct->setShortcut(tr("Ins")); 
    addPwdAct->setStatusTip(tr("Creates a new password"));
    addPwdAct->setEnabled(false);
    connect(addPwdAct, SIGNAL(triggered()), this, SLOT(slot_addPassword()));

    addGroupAct = new QAction(QIcon(":/images/foldernew.png"), tr("Create &group"), this);
    addGroupAct->setShortcut(tr("Ctrl+G"));
    addGroupAct->setStatusTip(tr("Creates a new group"));
    addGroupAct->setEnabled(false);
    connect(addGroupAct, SIGNAL(triggered()), this, SLOT(slot_createGroup()));

    renameAct = new QAction(QIcon(":/images/edit.png"), tr("&Rename entry"), this);
    renameAct->setShortcut(tr("Ctrl+R"));
    renameAct->setStatusTip(tr("Renames the selected item"));
    renameAct->setEnabled(false);
    connect(renameAct, SIGNAL(triggered()), this, SLOT(slot_renameEntry()));

    delEntryAct = new QAction(QIcon(":/images/entrydel.png"), tr("&Delete entry"), this);
    delEntryAct->setShortcut(tr("Del"));
    delEntryAct->setStatusTip(tr("Deletes the selected item"));
    delEntryAct->setEnabled(false);
    connect(delEntryAct, SIGNAL(triggered()), this, SLOT(slot_deleteEntry()));

    propAct = new QAction(QIcon(":/images/entryprops.png"), tr("&View entry"), this);
    propAct->setShortcut(tr("Alt+Enter"));
    propAct->setStatusTip(tr("Opens the selected item"));
    propAct->setEnabled(false);
    connect(propAct, SIGNAL(triggered()), this, SLOT(slot_properties()));

    fetchPwdAct = new QAction(QIcon(":/images/copypwd.png"), tr("Fetch pass&word"), this);
    fetchPwdAct->setShortcut(tr("CTRL+F"));
    fetchPwdAct->setStatusTip(tr("Copy the selected password to the clipboard"));
    fetchPwdAct->setEnabled(false);
    connect(fetchPwdAct, SIGNAL(triggered()), this, SLOT(slot_fetchPassword()));

    fetchUsrAct = new QAction(QIcon(":/images/copyusr.png"), tr("Fetch &user"), this);
    fetchUsrAct->setShortcut(tr("CTRL+U"));
    fetchUsrAct->setStatusTip(tr("Copy the selected user to the clipboard"));
    fetchUsrAct->setEnabled(false);
    connect(fetchUsrAct, SIGNAL(triggered()), this, SLOT(slot_fetchUser()));

    fetchUrlAct = new QAction(QIcon(":/images/copyurl.png"), tr("&Fetch url"), this);
    fetchUrlAct->setShortcut(tr("CTRL+L"));
    fetchUrlAct->setStatusTip(tr("Copy the selected url to the clipboard"));
    fetchUrlAct->setEnabled(false);
    connect(fetchUrlAct, SIGNAL(triggered()), this, SLOT(slot_fetchUrl()));

    openUrlAct = new QAction(QIcon(":/images/openurl.png"), tr("&Open url"), this);
    openUrlAct->setShortcut(tr("CTRL+W"));
    openUrlAct->setStatusTip(tr("Open the selected url"));
    openUrlAct->setEnabled(false);
    connect(openUrlAct, SIGNAL(triggered()), this, SLOT(slot_openUrl()));

    clearClipboardAction = new QAction(QIcon(":/images/clear.png"), tr("&Clear clipboard"), this);
    clearClipboardAction->setStatusTip(tr("Clear the clipboard"));
    clearClipboardAction->setEnabled(false);
    connect(clearClipboardAction, SIGNAL(triggered()), this, SLOT(slot_clearClipboard()));

    prefAct = new QAction(QIcon(":/images/prefs.png"), tr("&Preferences"), this);
    prefAct->setShortcut(tr("CTRL+P"));
    prefAct->setStatusTip(tr("Edit your preferences"));
    connect(prefAct, SIGNAL(triggered()), this, SLOT(slot_preferences()));

    // Help
    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(slot_about()));
}

/**
 * Creates all menus (except the context menu).
 */
void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(newAct);
    fileMenu->addAction(openAct);
    fileMenu->addAction(importAct);
    fileMenu->addAction(exportAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(saveAsAct);
    fileMenu->addAction(closeAct);
    separatorAct = fileMenu->addSeparator();
    for (int i = 0; i < ConfigStore::store["prefs_pws_maxrecentfiles"].toInt(); i++)
    {
        fileMenu->addAction(recentFileActs[i]);
    }
    fileMenu->addSeparator();
    fileMenu->addAction(chPwdAct);
    fileMenu->addAction(infoAct);
    fileMenu->addAction(lockAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);

    editMenu = menuBar()->addMenu(tr("&Edit"));
    editMenu->addAction(addPwdAct);
    editMenu->addAction(addGroupAct);
    editMenu->addAction(delEntryAct);
    editMenu->addAction(renameAct);
    editMenu->addAction(propAct);
    editMenu->addSeparator();
    editMenu->addAction(fetchPwdAct);
    editMenu->addAction(fetchUsrAct);
    editMenu->addAction(fetchUrlAct);
    editMenu->addAction(openUrlAct);
    editMenu->addAction(clearClipboardAction);
    editMenu->addSeparator();
    editMenu->addAction(prefAct);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
}

/**
 * Updates the recent files menu.
 */
void MainWindow::updateRecentFileActions()
{   
    QStringList files = ConfigStore::store["prefs_pws_recentfiles"].split("$",QString::SkipEmptyParts);

    int i, pos = 0;
    for (i = 0; (i < files.size()) && (i < ConfigStore::store["prefs_pws_maxrecentfiles"].toInt()); i++) 
    {
        if (files.at(i) != ConfigStore::store["curfile"])
        {
            if (ConfigStore::store["prefs_pws_showownlyrecentname"] == "1")
            {
                QStringList lst = files.at(i).split("/");
                recentFileActs[pos]->setText(lst.last());
            }
            else
            {
                recentFileActs[pos]->setText(files.at(i));
            }
            recentFileActs[pos]->setToolTip(files.at(i));
            recentFileActs[pos]->setStatusTip(files.at(i));
            recentFileActs[pos]->setData(files.at(i));
            recentFileActs[pos]->setVisible(true);
            pos++;
        }
    }

    for (i = pos; i < recentFileActsLen; i++)
    {
        recentFileActs[i]->setVisible(false);
    }

    separatorAct->setVisible(files.size() > 0);
}

/**
 * Create the toolbar.
 */
void MainWindow::createToolBars()
{
    toolbar = addToolBar(tr("File"));
    toolbar->addAction(newAct);
    toolbar->addAction(openAct);
    toolbar->addAction(saveAct);
    toolbar->addAction(saveAsAct);
    toolbar->addSeparator();
    toolbar->addAction(addPwdAct);
    toolbar->addAction(delEntryAct);
    toolbar->addAction(addGroupAct);
    toolbar->addAction(renameAct);
    toolbar->addAction(propAct);
    toolbar->addSeparator();
    toolbar->addAction(fetchUsrAct);
    toolbar->addAction(fetchPwdAct);
    toolbar->addAction(fetchUrlAct);
    toolbar->addAction(openUrlAct);
    toolbar->addAction(clearClipboardAction);
    toolbar->addSeparator();
    toolbar->addAction(lockAct);
    toolbar->addAction(prefAct);

    QSize siz (18,18);
    toolbar->setIconSize (siz);
    toolbar->setFloatable (false);
    toolbar->setMovable (false);
}

/**
 * Create the statusbar.
 */
void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

/**
 * If the database was modified, but not saved at this time, ask the user how to
 * proceed (save|discard|cancel).
 *
 * @returns true If the user clicked save/discard and fals if the user clicked cancel.
 */
bool MainWindow::maybeSave()
{
    int ret = QMessageBox::question(this, tr("Passwordsafe has been modified"), 
                                    tr("The passwordsafe has been modified.\nDo you want to save your changes?"),
                                    QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);

    switch (ret)
    {
        case QMessageBox::Save:
            slot_save();
            if (ConfigStore::store["curfile"] == "")
                { return (maybeSave()); }
            else
                { return true; }
            break;
        case QMessageBox::Discard:
            return true;
            break;
        case QMessageBox::Cancel:
            return false;
            break;
    }
    return false;
}

/**
 * When the main window is closed, check whether the database is unsaved and if yes,
 * ask the user how to proceed. Just before closing the main window, the preferences
 * will be saved.
 */
void MainWindow::closeEvent(QCloseEvent * event)
{
    if (modified)
    { 
        if (!maybeSave()) 
        { 
            event->ignore(); 
            return; 
        } 
    }
    
    // save geometry
    QRect geom = geometry();
    ConfigStore::store["prefs_pws_pos_x"] = QString::number (geom.x());
    ConfigStore::store["prefs_pws_pos_y"] = QString::number (geom.y());
    ConfigStore::store["prefs_pws_width"] = QString::number (geom.width());
    ConfigStore::store["prefs_pws_height"] = QString::number (geom.height());

    ConfigStore::saveConfig ();
}

/**
 * This slot is called whenever a tree item is expanded. This slot ensures,
 * that the selected item is always on the top, scrolling the whole tree if necessary.
 *
 * @param index - The expanded tree item.
 */
void MainWindow::slot_expanded(const QModelIndex & index)
{
    if (index.isValid())
    {
        if (ConfigStore::store["prefs_pws_scroll_hint"] == "0")
            { view->scrollTo (index, QAbstractItemView::EnsureVisible); }
        else if (ConfigStore::store["prefs_pws_scroll_hint"] == "1")
            { view->scrollTo (index, QAbstractItemView::PositionAtTop); }

        view->setCurrentIndex (index);
    }
}

/**
 * This slot is called, when the current selected item changes.
 *
 * @param index - The new index.
 */
void MainWindow::slot_currentChanged(const QModelIndex & index)
{
    if (index.isValid())
    {
        // if the index is valid, enable default actions ...
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (childItem->isGroup())
        {
            if (ConfigStore::store["readonly"] != "1")
            {
                delEntryAct->setEnabled(true);
                renameAct->setEnabled(true);
            }
            propAct->setEnabled(false);
            fetchPwdAct->setEnabled(false);
            fetchUsrAct->setEnabled(false);
            fetchUrlAct->setEnabled(false);
            openUrlAct->setEnabled(false);
            exportAct->setEnabled (true);
        }
        else
        {
            if (ConfigStore::store["readonly"] != "1")
            {
                delEntryAct->setEnabled(true);
                renameAct->setEnabled(true);
            }
            propAct->setEnabled(true);
            fetchPwdAct->setEnabled(true);
            fetchUsrAct->setEnabled(true);
            fetchUrlAct->setEnabled(true);
            openUrlAct->setEnabled(true);
            exportAct->setEnabled (true);
        }
    }
    else 
    {
        // otherwise disable default actions
        exportAct->setEnabled (false);
        delEntryAct->setEnabled(false);
        renameAct->setEnabled(false);
        propAct->setEnabled(false);
        fetchPwdAct->setEnabled(false);
        fetchUsrAct->setEnabled(false);
        fetchUrlAct->setEnabled(false);
        openUrlAct->setEnabled(false);
    }
}

/**
 * This lsot is called whenever the tree was sorted. After sorting,
 * the columns are resized to show the whole centents in the cells.
 *
 * @param item - The item which subtree was sorted.
 */
void MainWindow::slot_sorted(EntryItem *item)
{
    QModelIndex index;

    if (item)
    {
        index = model->index(item->row(), 0, view->currentIndex().parent());
        view->setCurrentIndex (index); 
    }
    slot_resizeColumn (index);
}

/**
 * This slot is called, whenever the user clicks the import-menu-entry.
 */
void MainWindow::slot_import()
{
    QString filename = QFileDialog::getOpenFileName(this, tr("Import passwordsafe"), ConfigStore::store["prefs_pws_cur_dir"],
      tr("Passwordsafe v[23] format (*.psafe3 *.dat);;All files (*)"));

    if (filename.isEmpty())
        { return; }

    ConfigStore::store["prefs_pws_cur_dir"] = QFileInfo (filename).absolutePath();

    // we will need that later in slot_importTree()
    ConfigStore::store["importfilename"] = filename;

    // get the passphrase for the database (the one to be imported)
    PassphraseDialog *pwd_dlg = new PassphraseDialog(false, this);
    pwd_dlg->setWindowModality(Qt::ApplicationModal);

    // connect the passphrase dialog - on submit, import the file,
    // when the file was successfully opened, close the passphrase dialog
    connect (pwd_dlg, SIGNAL(submit(const QString &, bool)), this, SLOT(slot_importTree(const QString &)));
    connect (pwsHandler, SIGNAL(wrongPassphrase()), pwd_dlg, SLOT(slot_wrongPassphrase()));
    connect (this, SIGNAL(fileOpened()), pwd_dlg, SLOT(close()));

    pwd_dlg->show();
}

/**
 * This slot imports another database below the current selected item.
 *
 * @param passphrase - The passphrase.
 */
void MainWindow::slot_importTree(const QString & passphrase)
{
    pwsHandler->setPassphrase(passphrase);
    QModelIndex index = view->currentIndex();

    if (pwsHandler->import (ConfigStore::store["importfilename"], index))
    {
        // restore the passphrase of the main database after
        // the import is finished (ConfigStore::pws[23] is used
        // both for importing and opening a database)
        pwsHandler->restorePassphrase();

        setStatus (MainWindow::FileModified); 
        emit fileOpened();
    }
    else
    {
        pwsHandler->restorePassphrase(); 
    }
}

/**
 * This slot is called when the user selects one of the files
 * in the recent files menu.
 */
void MainWindow::slot_openRecent()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action)
    {
        ConfigStore::store["curfile"] = action->data().toString();
    }

    // ask for the passphrase
    PassphraseDialog *pwd_dlg = new PassphraseDialog(true, this);
    pwd_dlg->setWindowModality(Qt::ApplicationModal);

    // and connect the passphrase dialog
    connect (pwd_dlg, SIGNAL(submit(const QString &, bool)), this, SLOT(slot_setupTree(const QString &, bool)));
    connect (pwsHandler, SIGNAL(wrongPassphrase()), pwd_dlg, SLOT(slot_wrongPassphrase()));
    connect (this, SIGNAL(fileOpened()), pwd_dlg, SLOT(close()));

    pwd_dlg->show();
}

/**
 * This slot is called when the user selects the "open" menu entry.
 */
void MainWindow::slot_open()
{
    // before opening a new database check whether the
    // old database is modified
    if (modified)
        { if (!maybeSave()) { return; } }

    QString filename = QFileDialog::getOpenFileName(this, tr("Open passwordsafe"), ConfigStore::store["prefs_pws_cur_dir"],
      tr("Passwordsafe v[23] format (*.psafe3 *.dat);;All files (*)"));

    if (filename.isEmpty())
        { return; }

    ConfigStore::store["prefs_pws_cur_dir"] = QFileInfo (filename).absolutePath();
    ConfigStore::store["curfile"] = filename;

    // ask for the passphrase
    PassphraseDialog *pwd_dlg = new PassphraseDialog(true, this);
    pwd_dlg->setWindowModality(Qt::ApplicationModal);

    // and connect the passphrase dialog
    connect (pwd_dlg, SIGNAL(submit(const QString &, bool)), this, SLOT(slot_setupTree(const QString &, bool)));
    connect (pwsHandler, SIGNAL(wrongPassphrase()), pwd_dlg, SLOT(slot_wrongPassphrase()));
    connect (this, SIGNAL(fileOpened()), pwd_dlg, SLOT(close()));

    pwd_dlg->show();
}

/**
 * This slot is called when the user selects the "add entry" menu entry.
 */
void MainWindow::slot_addPassword()
{
    EntryItem *newItem; 

    // get some defaults
    QString user = "";
    if (ConfigStore::store["prefs_bool_usedefuser"] == "1")
        { user = ConfigStore::store["prefs_str_defaultusername"]; }

    int max_hist_size = -1;
    if (ConfigStore::store["prefs_bool_savepasswordhistory"] == "1")
        { max_hist_size = ConfigStore::store["prefs_int_numpwhistorydefault"].toInt(); }

    int save_pwd_history = -1;
    if (ConfigStore::store["prefs_bool_savepasswordhistory"] == "1")
        { save_pwd_history = 1; }

    newItem = new EntryItem ("", QUuid::createUuid(), user,  "", "", QUrl(""), "", save_pwd_history, max_hist_size,
                             QDateTime::currentDateTime(), QDateTime::currentDateTime(), QDateTime::currentDateTime(), 
                             QDateTime::currentDateTime(), QDateTime::fromTime_t(0),
                             NULL, 0);

    // construct the EditEntryDialog object
    EditEntryDialog *entry = new EditEntryDialog(newItem, this);
    entry->setWindowModality(Qt::ApplicationModal);
    entry->show();

    // connect the EditEntryObject
    connect (entry, SIGNAL(dataChanged(EntryItem*)), this, SLOT(slot_addEntry(EntryItem*)));
    connect (entry, SIGNAL(dataChanged(EntryItem*)), this, SLOT(slot_fileModified()));
    connect (entry, SIGNAL(canceled(EntryItem*)),    this, SLOT(slot_newEntryCanceled(EntryItem*)));
    connect (entry, SIGNAL(fetchPwd()),              this, SLOT(slot_fetchPassword()));
    connect (entry, SIGNAL(fetchHistory()),          this, SLOT(slot_fetchHistory()));
}

/**
 * This slot is called, when the user selects "cancel" in the EditEntryDialog and the EditEntryDialog
 * was used to add a new, non-existing entry (see MainWindow::slot_addPassword())
 */
void MainWindow::slot_newEntryCanceled(EntryItem *item)
{
    // delete the unused item
    delete item;
}

/**
 * This slot is called, when the user selects "ok" in the EditEntryDialog and the EditEntryDialog
 * was used to add a new, non-existing entry (see MainWindow::slot_addPassword())
 */
void MainWindow::slot_addEntry(EntryItem *item)
{
    QModelIndex parent, index = view->currentIndex();
    EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());

    if (!index.isValid())
    {
        // if current index is invalid, add the new item below the root index
        parent = view->rootIndex();
    }
    else if (!childItem->isGroup())
    {
        // if current index is not a group, add the new entry to the parent group
        parent = index.parent();
    }
    else
    {
        // otherwise add the new entry below the current selected group
        parent = index;
    }

    model->addEntry (item, parent);

    // select the new item in the tree view
    if (ConfigStore::store["prefs_pws_scroll_hint"] == "0")
        { view->scrollTo (model->index (item->row(), 0, parent), QAbstractItemView::EnsureVisible); }
    else if (ConfigStore::store["prefs_pws_scroll_hint"] == "1")
        { view->scrollTo (model->index (item->row(), 0, parent), QAbstractItemView::PositionAtTop); }

    view->setCurrentIndex (model->index (item->row(), 0, parent)); 
}

/**
 * This slot is called when the user selects the "rename" menu entry.
 */
void MainWindow::slot_renameEntry()
{
    QModelIndex index = view->currentIndex();

    if (index.isValid())
        { view->edit(index); }
}

/**
 * This slot is called when the user selects the "exit" menu entry.
 */
void MainWindow::slot_exit()
{
    // before closing the application check whether the
    // old database is modified
    if (modified)
        { if (!maybeSave()) { return; } }

    close();
}

/**
 * This slot is called when the user selects the "close" menu entry.
 */
void MainWindow::slot_closeFile()
{
    // before closing the database check whether the
    // old database is modified
    if (modified)
        { if (!maybeSave()) { return; } }

    slot_reset ();
    setStatus (MainWindow::FileClosed);
}

/**
 * This slot is called when the user changes any preferences (see slot_preferences()).
 */
void MainWindow::slot_prefsChanged()
{
    // if the prefs_pws_show_grid pref was changed show, resp. do not show alternating row colors.
    if (ConfigStore::store["prefs_pws_show_grid"] == "1")
        { view->setAlternatingRowColors (true); }
    else
        { view->setAlternatingRowColors (false); }

    // if the prefs_pws_maxrecentfiles changed, adjust the recent files menu accordingly
    int old_siz = recentFileActsLen;
    if (ConfigStore::store["prefs_pws_maxrecentfiles"].toInt() > old_siz)
    {
        recentFileActsLen = ConfigStore::store["prefs_pws_maxrecentfiles"].toInt();
        recentFileActs = (QAction**)realloc (recentFileActs, ConfigStore::store["prefs_pws_maxrecentfiles"].toInt()*sizeof(QAction*));

        for (int i = old_siz; i < ConfigStore::store["prefs_pws_maxrecentfiles"].toInt(); i++)
        {
            recentFileActs[i] = new QAction(this);
            recentFileActs[i]->setVisible(false);
            connect(recentFileActs[i], SIGNAL(triggered()), this, SLOT(slot_openRecent()));
        }
    }

    QStringList files = ConfigStore::store["prefs_pws_recentfiles"].split("$",QString::SkipEmptyParts);
    ConfigStore::store["prefs_pws_recentfiles"] = "";
    for (int i = 0; (i < files.size()) && (i < ConfigStore::store["prefs_pws_maxrecentfiles"].toInt()); i++)
    {
        if (ConfigStore::store["prefs_pws_recentfiles"] == "")
        {
            ConfigStore::store["prefs_pws_recentfiles"] = files.at(i);
        }
        else
        {
            ConfigStore::store["prefs_pws_recentfiles"] += "$" + files.at(i);
        }
    }

    updateRecentFileActions();

    // if the idle timeout changed, reset the idle timer
    if (ConfigStore::store["prefs_pws_enable_idle_timeout"] != "0")
    {
        if (ConfigStore::store["prefs_int_idletimeout"].toInt()*60*1000 != i_timer->interval())
        {
            i_timer->setInterval (ConfigStore::store["prefs_int_idletimeout"].toInt()*60*1000);
        }
    }
    else
    {
        i_timer->stop();
    }

    // update recent files menu
    updateRecentFileActions();
}

/**
 * This slot is called whenever the database was modified.
 */
void MainWindow::slot_fileModified()
{
    setStatus (MainWindow::FileModified);
}

/**
 * This slot is called when the user selects the "new" menu entry.
 */
void MainWindow::slot_newFile()
{
    // before opening a new database check whether the
    // old database is modified
    if (modified)
        { if (!maybeSave()) { return; } }

    // ask for a new passphrase
    NewPassphraseDialog *pwd_dlg = new NewPassphraseDialog(this);
    pwd_dlg->setWindowModality(Qt::ApplicationModal);

    // connect the NewPassphraseDialog object
    connect (pwd_dlg, SIGNAL(submit(const QString &)), this, SLOT(slot_newFileSubmit(const QString &)));

    pwd_dlg->show();
}

/**
 * This slot is called when the user selects "ok" in the NewPassphraseDialog object
 * (see slot_newFile()).
 *
 * @param passphrase - The passphrase.
 */
void MainWindow::slot_newFileSubmit(const QString & passphrase)
{
    slot_reset();
    setStatus(MainWindow::FileNew);
    pwsHandler->setPassphrase (passphrase);
}

/**
 * Resets the database to default values after closing a database, opening or
 * creating a new databas.
 */
void MainWindow::slot_reset()
{
    ConfigStore::defaults(ConfigStore::PWS);
    model->clear();
}

/**
 * Resize the columns to show complete cells.
 */
void MainWindow::slot_resizeColumn (const QModelIndex &index)
{
    index.parent(); // just to avoid the gcc warning
    view->resizeColumnToContents (0);
    view->resizeColumnToContents (1);
    view->resizeColumnToContents (2);
    view->resizeColumnToContents (3);
    view->resizeColumnToContents (4);
    view->resizeColumnToContents (5);
}

/**
 * This slot is called whe the user selects the "save" menu entry.
 */
void MainWindow::slot_save()
{
    if (modified)
    {
        if (ConfigStore::store["curfile"] != "")
        {
            if (ConfigStore::store["prefs_pws_save_backups"] == "1")
            {
                // save a backup
                saveBackup(ConfigStore::store["curfile"]);
            }

            // and save the database
            if (ConfigStore::store["pws_hdr_version"].startsWith ("2"))
            {
                if (pwsHandler->savePwsV2(ConfigStore::store["curfile"]))
                    { setStatus (MainWindow::FileSaved); }
            }
            else if (ConfigStore::store["pws_hdr_version"].startsWith ("3"))
            {
                if (pwsHandler->savePwsV3(ConfigStore::store["curfile"]))
                    { setStatus (MainWindow::FileSaved); }
            }
            else 
            {
                if (pwsHandler->savePwsV3(ConfigStore::store["curfile"]))
                    { setStatus (MainWindow::FileSaved); }
            }
        }
        else
        {
            // if the current file was never saved before ask for a database filename
            slot_saveAs();
        }
    }
}

/**
 * This slot is called when the user selects the "save as" menu entry.
 */
void MainWindow::slot_saveAs()
{
    QPWSFileDialog *dialog = new QPWSFileDialog (this, tr("Save passwordsafe"), ConfigStore::store["prefs_pws_cur_dir"],
      tr("Passwordsafe v3 format (*.psafe3);;Passwordsafe v2 format (*.dat);;All files (*)"));

    dialog->setWindowModality (Qt::ApplicationModal);
    dialog->setAcceptMode(QFileDialog::AcceptSave);
    if (dialog->exec())
    {
        QStringList filenames = dialog->selectedFiles(); 

        if (filenames.isEmpty())
            { delete dialog; return; }

        QString filter = dialog->selectedFilter();

        if (filter.endsWith ("(*.psafe3)"))
            { ConfigStore::store["pws_hdr_version"] = "3.2"; }
        else if (filter.endsWith ("(*.dat)"))
            { ConfigStore::store["pws_hdr_version"] = "2.0"; }
        else if (filter.endsWith ("(*)"))
            { ConfigStore::store["pws_hdr_version"] = "3.2"; }

        // set the current filename to the above string and create a new uuid for
        // the database
        ConfigStore::store["prefs_pws_cur_dir"] = QFileInfo (filenames.at(0)).absolutePath();
        ConfigStore::store["curfile"] = filenames.at(0);
        ConfigStore::store["pws_hdr_uuid"] = QUuid::createUuid().toString();

        // add a recent files entry
        ConfigStore::addRecentFile (filenames.at(0));
        updateRecentFileActions();

        // "save as" is of course possible, when the original database is opened readonly.
        // the new database is no longer readonly, the majority of users will expect this
        // behaviour
        if (ConfigStore::store["readonly"] == "1")
            { ConfigStore::store["readonly"] = "0"; }

        modified = true;
        slot_save();
        setStatus (MainWindow::FileSaved);
    }

    delete dialog;
}

/**
 * This slot is called, when the user selects the "information" menu entry.
 */
void MainWindow::slot_information()
{
    InformationDialog *info_dlg = new InformationDialog(model, this);
    info_dlg->setWindowModality(Qt::ApplicationModal);
    info_dlg->show();

    connect (info_dlg, SIGNAL(submit()), this, SLOT(slot_fileModified()));
}

/**
 * This slot is called, when the user selects the "change passphrase" menu entry.
 */
void MainWindow::slot_changePassphrase()
{
    NewPassphraseDialog *pwd_dlg = new NewPassphraseDialog(this);
    pwd_dlg->setWindowModality(Qt::ApplicationModal);

    pwd_dlg->show();
    connect (pwd_dlg, SIGNAL(submit(const QString &)), this, SLOT(slot_passphraseChanged(const QString &)));
}

/**
 * This slot is called, when the "ok" button was clicked inside the
 * NewPassphraseDialog object (see slot_changePassphrase()).
 *
 * @param passphrase - The passphrase.
 */
void MainWindow::slot_passphraseChanged(const QString & passphrase)
{
    pwsHandler->setPassphrase (passphrase);
    setStatus (MainWindow::FileModified);
}

/**
 * This slot is called, when the user selects the "create" menu entry.
 */
void MainWindow::slot_createGroup()
{
    QModelIndex index = view->currentIndex();

    // if the current index is invalid, use the root index.
    if (!index.isValid())
    {
        index = view->rootIndex();
    }

    // create a new group just below index
    index = model->index (index.row(), 0, index.parent());
    QModelIndex newGroup = model->createGroup (index);

    // select the newly created group
    if (ConfigStore::store["prefs_pws_scroll_hint"] == "0")
        { view->scrollTo (newGroup, QAbstractItemView::EnsureVisible); }
    else if (ConfigStore::store["prefs_pws_scroll_hint"] == "1")
        { view->scrollTo (newGroup, QAbstractItemView::PositionAtTop); }

    view->setCurrentIndex(newGroup);
    view->edit(newGroup);

    setStatus (MainWindow::FileModified);
}

/**
 * This slot is called, when the user selects the "delete" menu entry.
 */
void MainWindow::slot_deleteEntry()
{
    QModelIndex index = view->currentIndex();
    if (index.isValid())
    {
        int ret = QMessageBox::Ok;
        if (ConfigStore::store["prefs_pws_confirm_delete"] == "1")
        {
            // if the prefs_pws_confirm_delete preference is set, ask for confirmation
            ret = QMessageBox::question(this, tr("Confirmation needed"), tr("Really delete the selected item?"),
                                        QMessageBox::Ok|QMessageBox::Cancel);
        }

        if (ret == QMessageBox::Ok)
        {
            // delete the current index
            model->delEntry (index);
            setStatus (MainWindow::FileModified);
        }
    }
}

/**
 * This slot is called, when the user presses enter.
 */
void MainWindow::slot_enterPressed()
{
    switch (ConfigStore::store["prefs_pws_enter_action"].toInt())
    {
        case 0:
            slot_properties();
            break;
        case 1:
            slot_fetchPassword();
            break;
        case 2:
            slot_fetchUser();
            break;
        case 3:
            slot_fetchUrl();
            break;
        case 4:
            slot_openUrl();
            break;
        default:
            slot_properties();
            break;
    }
}

/**
 * This slot is called, when the user presses space.
 */
void MainWindow::slot_spacePressed()
{
    switch (ConfigStore::store["prefs_pws_space_action"].toInt())
    {
        case 0:
            slot_properties();
            break;
        case 1:
            slot_fetchPassword();
            break;
        case 2:
            slot_fetchUser();
            break;
        case 3:
            slot_fetchUrl();
            break;
        case 4:
            slot_openUrl();
            break;
        default:
            slot_properties();
            break;
    }
}

/**
 * This slot is called, when the user double clicks an item.
 *
 * @param index - The doubleclicked index.
 */
void MainWindow::slot_doubleClicked(const QModelIndex & index)
{
    // this one is dirty, i know ...
    // on doubleclick, currentItem is collapsed/expanded
    // two times, first in QTreeView, then in MainWindow,
    // resulting in no visible change. as a workaround,
    // currentItem is collapsed/expanded a third time
    if (index.isValid() && index.column() == 0)
    {
        if (view->isExpanded (index))
            view->setExpanded (index, false);
        else
            view->setExpanded (index, true);
    }

    switch (ConfigStore::store["prefs_pws_doubleclick_action"].toInt())
    {
        case 0:
            slot_properties();
            break;
        case 1:
            slot_fetchPassword();
            break;
        case 2:
            slot_fetchUser();
            break;
        case 3:
            slot_fetchUrl();
            break;
        case 4:
            slot_openUrl();
            break;
        default:
            slot_properties();
            break;
    }
}

/**
 * This slot is called, when the user selects the "properties" menu entry.
 */
void MainWindow::slot_properties()
{
    QModelIndex index = view->currentIndex();

    // proceed only on valid indices
    if (index.isValid())
    { 
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());

        if (!childItem->isGroup())
        {
            // otherwise open the properties
            EditEntryDialog *entry = new EditEntryDialog(childItem, this);
            entry->setWindowModality(Qt::ApplicationModal);
            entry->show();

            if (ConfigStore::store["prefs_bool_maintaindatetimestamps"] == "1" && ConfigStore::store["readonly"] != "1")
            {
                // update access timestamp if prefs_bool_maintaindatetimestamps preference is set
                childItem->setATime (QDateTime::currentDateTime()); 
                setStatus (MainWindow::FileModified);
            }

            // and connect the EditEntryDialog widget
            connect (entry, SIGNAL(dataChanged(EntryItem*)), model, SLOT(sort(EntryItem*)));
            connect (entry, SIGNAL(dataChanged(EntryItem*)), this,  SLOT(slot_fileModified()));
            connect (entry, SIGNAL(fetchPwd()),              this,  SLOT(slot_fetchPassword()));
            connect (entry, SIGNAL(fetchHistory()),          this,  SLOT(slot_fetchHistory()));
        }
        else
        {
            // if the current index is a group entry inverse the expanded/collapsed state
            if (view->isExpanded (index))
                view->setExpanded (index, false);
            else
                view->setExpanded (index, true);
        }
    }
}

/**
 * This slot is called when the user selects the "fetch history" button
 * in the EditEntryDialog widget (see slot_properties()).
 */
void MainWindow::slot_fetchHistory()
{
    QModelIndex index = view->currentIndex();

    if (index.isValid())
    {
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (!childItem->isGroup())
        { 
            // if the current index is valid and not a group entry ...
            int ret = QMessageBox::Ok;
            // ... and "prefs_pws_confirm_clpbrd" preference is set ask the user for confirmation ...
            if (ConfigStore::store["prefs_pws_confirm_clpbrd"] == "1")
            {
                ret = QMessageBox::question(this, tr("Confirmation needed"), tr("Really copy the password history to\nthe system clipboard?"),
                                            QMessageBox::Ok|QMessageBox::Cancel);
            }

            if (ret == QMessageBox::Ok)
            {
                QString *history = new QString;

                // ... copy the whole history to the clipboard
                QListIterator<HistoryEntry> pwd_history(childItem->getPwdHistory());
                while (pwd_history.hasNext()) 
                {
                    *history  += *(pwd_history.next().password) + "\n";
                }

                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(*history, QClipboard::Clipboard);
                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(*history, QClipboard::Selection);

                if (!ConfigStore::store["prefs_pws_clpbrd_timeout"].isEmpty() && ConfigStore::store["prefs_pws_clpbrd_timeout"] != "0")
                {
                    // set up the clipboard timeout if prefs_pws_clpbrd_timeout preference is > 0
                    c_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    s_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    
                    // and connect the clipboard timeout timers
                    connect (c_timer, SIGNAL(timeout()), this, SLOT(slot_clearCClipboard()));
                    connect (s_timer, SIGNAL(timeout()), this, SLOT(slot_clearSClipboard()));
                }

                // and save the history contents for later use (see slot_clearCClipboar() and slot_clearSClipboardd ())
                *clpbrd_contents = *history;

                if (ConfigStore::store["prefs_bool_maintaindatetimestamps"] == "1" && ConfigStore::store["readonly"] != "1")
                {
                    // if prefs_bool_maintaindatetimestamps is set update the access time of this entry
                    childItem->setATime (QDateTime::currentDateTime()); 
                    setStatus (MainWindow::FileModified);
                }

                // and finally inform the user that the action was performed
                statusBar()->showMessage(tr("Password history copied to clipboard"));

                delete history;
            }
        }
    }
}

/**
 * This slot is called when the user selects the "fetch password" button
 * in the EditEntryDialog widget (see slot_properties()) or the corresponding
 * menu entry.
 */
void MainWindow::slot_fetchPassword()
{
    QModelIndex index = view->currentIndex();

    if (index.isValid())
    {
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (!childItem->isGroup())
        { 
            // if the current index is valid and not a group entry ...
            int ret = QMessageBox::Ok;
            if (ConfigStore::store["prefs_pws_confirm_clpbrd"] == "1")
            {
                // ... and "prefs_pws_confirm_clpbrd" preference is set ask the user for confirmation ...
                ret = QMessageBox::question(this, tr("Confirmation needed"), tr("Really copy the selected password to\nthe system clipboard?"),
                                            QMessageBox::Ok|QMessageBox::Cancel);
            }

            if (ret == QMessageBox::Ok)
            {
                // ... copy the password to the clipboard
                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(childItem->getPwd(), QClipboard::Clipboard);
                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(childItem->getPwd(), QClipboard::Selection);

                if (!ConfigStore::store["prefs_pws_clpbrd_timeout"].isEmpty() && ConfigStore::store["prefs_pws_clpbrd_timeout"] != "0")
                {
                    // set up the clipboard timeout if prefs_pws_clpbrd_timeout preference is > 0
                    c_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    s_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    
                    // and connect the clipboard timeout timers
                    connect (c_timer, SIGNAL(timeout()), this, SLOT(slot_clearCClipboard()));
                    connect (s_timer, SIGNAL(timeout()), this, SLOT(slot_clearSClipboard()));
                }

                // and save the history contents for later use (see slot_clearCClipboar() and slot_clearSClipboardd ())
                *clpbrd_contents = childItem->getPwd();

                if (ConfigStore::store["prefs_bool_maintaindatetimestamps"] == "1" && ConfigStore::store["readonly"] != "1")
                {
                    // if prefs_bool_maintaindatetimestamps is set update the access time of this entry
                    childItem->setATime (QDateTime::currentDateTime()); 
                    setStatus (MainWindow::FileModified);
                }

                // and finally inform the user that the action was performed
                statusBar()->showMessage(tr("Password copied to clipboard"));
            }
        }
        else
        { 
            // if the current index is a group entry inverse the expanded/collapsed state
            if (view->isExpanded (index))
                view->setExpanded (index, false);
            else
                view->setExpanded (index, true);
        }
    }
}

/**
 * This slot is called when the user selects the "fetch user" button
 * in the EditEntryDialog widget (see slot_properties()) or the corresponding
 * menu entry.
 */
void MainWindow::slot_fetchUser()
{
    QModelIndex index = view->currentIndex();

    if (index.isValid() )
    {
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (!childItem->isGroup())
        { 
            // if the current index is valid and not a group entry ...
            int ret = QMessageBox::Ok;
            if (ConfigStore::store["prefs_pws_confirm_clpbrd"] == "1")
            {
                // ... and "prefs_pws_confirm_clpbrd" preference is set ask the user for confirmation ...
                ret = QMessageBox::question(this, tr("Confirmation needed"), tr("Really copy the selected username to\nthe system clipboard?"),
                                            QMessageBox::Ok|QMessageBox::Cancel);
            }

            if (ret == QMessageBox::Ok)
            {
                // ... copy the username to the clipboard
                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(childItem->getUser(), QClipboard::Clipboard);
                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(childItem->getUser(), QClipboard::Selection);

                if (!ConfigStore::store["prefs_pws_clpbrd_timeout"].isEmpty() && ConfigStore::store["prefs_pws_clpbrd_timeout"] != "0")
                {
                    // set up the clipboard timeout if prefs_pws_clpbrd_timeout preference is > 0
                    c_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    s_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    
                    // and connect the clipboard timeout timers
                    connect (c_timer, SIGNAL(timeout()), this, SLOT(slot_clearCClipboard()));
                    connect (s_timer, SIGNAL(timeout()), this, SLOT(slot_clearSClipboard()));
                }

                // and save the history contents for later use (see slot_clearCClipboar() and slot_clearSClipboardd ())
                *clpbrd_contents = childItem->getUser();

                if (ConfigStore::store["prefs_bool_maintaindatetimestamps"] == "1" && ConfigStore::store["readonly"] != "1")
                {
                    // if prefs_bool_maintaindatetimestamps is set update the access time of this entry
                    childItem->setATime (QDateTime::currentDateTime()); 
                    setStatus (MainWindow::FileModified);
                }

                // and finally inform the user that the action was performed
                statusBar()->showMessage(tr("Username copied to clipboard"));
            }
        }
        else
        {
            // if the current index is a group entry inverse the expanded/collapsed state
            if (view->isExpanded (index))
                view->setExpanded (index, false);
            else
                view->setExpanded (index, true);
        }
    }
}

/**
 * This slot is called when the user selects the "fetch url" button
 * in the EditEntryDialog widget (see slot_properties()) or the corresponding
 * menu entry.
 */
void MainWindow::slot_fetchUrl()
{
    QModelIndex index = view->currentIndex();

    if (index.isValid() )
    {
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (!childItem->isGroup())
        { 
            // if the current index is valid and not a group entry ...
            int ret = QMessageBox::Ok;
            if (ConfigStore::store["prefs_pws_confirm_clpbrd"] == "1")
            {
                // ... and "prefs_pws_confirm_clpbrd" preference is set ask the user for confirmation ...
                ret = QMessageBox::question(this, tr("Confirmation needed"), tr("Really copy the selected url to\nthe system clipboard?"),
                                            QMessageBox::Ok|QMessageBox::Cancel);
            }

            if (ret == QMessageBox::Ok)
            {
                // ... copy the url to the clipboard
                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(childItem->getUrl().toString(), QClipboard::Clipboard);
                ((QApplication*)QCoreApplication::instance())->clipboard()->setText(childItem->getUrl().toString(), QClipboard::Selection);

                if (!ConfigStore::store["prefs_pws_clpbrd_timeout"].isEmpty() && ConfigStore::store["prefs_pws_clpbrd_timeout"] != "0")
                {
                    // set up the clipboard timeout if prefs_pws_clpbrd_timeout preference is > 0
                    c_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    s_timer->start (ConfigStore::store["prefs_pws_clpbrd_timeout"].toInt()*1000);
                    
                    // and connect the clipboard timeout timers
                    connect (c_timer, SIGNAL(timeout()), this, SLOT(slot_clearCClipboard()));
                    connect (s_timer, SIGNAL(timeout()), this, SLOT(slot_clearSClipboard()));
                }

                // and save the history contents for later use (see slot_clearCClipboar() and slot_clearSClipboardd ())
                *clpbrd_contents = childItem->getUrl().toString();

                if (ConfigStore::store["prefs_bool_maintaindatetimestamps"] == "1")
                {
                    // if prefs_bool_maintaindatetimestamps is set update the access time of this entry
                    childItem->setATime (QDateTime::currentDateTime()); 
                    setStatus (MainWindow::FileModified);
                }

                // and finally inform the user that the action was performed
                statusBar()->showMessage(tr("Url copied to clipboard"));
            }
        }
        else
        {
            // if the current index is a group entry inverse the expanded/collapsed state
            if (view->isExpanded (index))
                view->setExpanded (index, false);
            else
                view->setExpanded (index, true);
        }
    }
}

/**
 * This slot is called when the user selects the "open url" menu entry.
 */
void MainWindow::slot_openUrl()
{
    QModelIndex index = view->currentIndex();

    if (index.isValid() )
    {
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (!childItem->isGroup())
        { 
            // if the current index is valid and not a group entry ...
            if (childItem->getUrl().toString() != "")
            {
                // open the url in your favourite browser
                QDesktopServices::openUrl (childItem->getUrl());

                if (ConfigStore::store["prefs_bool_maintaindatetimestamps"] == "1" && ConfigStore::store["readonly"] != "1")
                {
                    // if prefs_bool_maintaindatetimestamps is set update the access time of this entry
                    childItem->setATime (QDateTime::currentDateTime()); 
                    setStatus (MainWindow::FileModified);
                }

                // and finally inform the user that the action was performed
                statusBar()->showMessage(childItem->getUrl().toString() + tr(" opened"));
            }
        }
        else
        {
            // if the current index is a group entry inverse the expanded/collapsed state
            if (view->isExpanded (index))
                view->setExpanded (index, false);
            else
                view->setExpanded (index, true);
        }
    }
}

/**
 * Enables the Enter-/Return-Key-Shortcut.
 */
void MainWindow::slot_enableEnterShortcut()
{
    enterShortcut->setEnabled (true);
    returnShortcut->setEnabled (true);
}

/**
 * Disables the Enter-/Return-Key-Shortcut.
 */
void MainWindow::slot_disableEnterShortcut()
{
    enterShortcut->setEnabled (false);
    returnShortcut->setEnabled (false);
}

/**
 * This slot is called whenever the user left click inside the tree view.
 */
void MainWindow::slot_clicked(const QModelIndex & index)
{
    if (index.isValid())
    {
        QModelIndex idx = model->index (index.row(), 0, index.parent());
        view->setCurrentIndex (idx);
    }
}

/**
 * This slot is called when the user selects the "clear clipboard" menu entry.
 */
void MainWindow::slot_clearClipboard()
{
    ((QApplication*)QCoreApplication::instance())->clipboard()->clear(QClipboard::Clipboard);
    ((QApplication*)QCoreApplication::instance())->clipboard()->clear(QClipboard::Clipboard);

    statusBar()->showMessage(tr("Clipboard cleared"));
}

/**
 * This slot is called when the c_timer times out.
 */
void MainWindow::slot_clearCClipboard()
{
    if (*clpbrd_contents == ((QApplication*)QCoreApplication::instance())->clipboard()->text(QClipboard::Clipboard))
    {
        ((QApplication*)QCoreApplication::instance())->clipboard()->clear(QClipboard::Clipboard);

        delete clpbrd_contents;
        clpbrd_contents = new QString;

        statusBar()->showMessage(tr("Clipboard cleared"));
    }
}

/**
 * This slot is called when the s_timer times out.
 */
void MainWindow::slot_clearSClipboard()
{
    if (*clpbrd_contents == ((QApplication*)QCoreApplication::instance())->clipboard()->text(QClipboard::Selection))
    {
        ((QApplication*)QCoreApplication::instance())->clipboard()->clear(QClipboard::Selection);

        delete clpbrd_contents;
        clpbrd_contents = new QString;

        statusBar()->showMessage(tr("Clipboard cleared"));
    }
}

/**
 * This slot is called when the user selects the "preferences" menu entry.
 */
void MainWindow::slot_preferences()
{
    PreferencesDialog *pref_dlg = new PreferencesDialog(this);
    pref_dlg->setWindowModality(Qt::ApplicationModal);
    pref_dlg->show();

    connect (pref_dlg, SIGNAL(dataChanged()), this, SLOT(slot_fileModified()));
    connect (pref_dlg, SIGNAL(prefsChanged()), this, SLOT(slot_prefsChanged()));
}

/**
 * This slot is called when the user selects the "about" menu entry.
 */
void MainWindow::slot_about()
{
    AboutDialog *about = new AboutDialog(this);
    about->show();
}

/**
 * This slot is called when the user selects the "lock" menu entry or the
 * i_timer times out.
 */
void MainWindow::slot_lock ()
{
    QWidgetList widgets = QApplication::allWidgets();
    for (int i = 0; i < widgets.size(); i++)
    {
        // save all current visible main widgets in this->hidden
        // and hide them
        if (widgets.at(i)->isWindow() && widgets.at(i)->isVisible())
        {
            widgets.at(i)->hide();
            hidden.append(widgets.at(i));
        }
    }

    // show the UnlockDialog
    UnlockDialog *unlock_dlg = new UnlockDialog(pwsHandler, this);
    unlock_dlg->show();

    // and stop the idle timer until the UnlockDialog id dismissed
    i_timer->stop();

    // connect the UnlockDialog object
    connect (unlock_dlg, SIGNAL(submit()),     this, SLOT(slot_unlock()));
    connect (unlock_dlg, SIGNAL(toBeClosed()), this, SLOT(close()));
}

/**
 * This slot is called when a valid passphrase was set inside the UnlockDialog
 * object (see slot_lock()):
 */
void MainWindow::slot_unlock ()
{
    for (int i = 0; i < hidden.size(); i++)
    {
        // show all hidden windows and ...
        hidden.at(i)->show();
    }
    // ... clear the hidden window list
    hidden.clear();

    // restart the idle timer again
    if (ConfigStore::store["prefs_pws_enable_idle_timeout"] != "0")
        { i_timer->start(); }
}

/**
 * This slot is called whenever the WatchDog recognize any event.
 */
void MainWindow::slot_resetTimer ()
{
    if (hidden.isEmpty() && fileOpen && ConfigStore::store["prefs_pws_enable_idle_timeout"] != "0")
    {
        // just restart the timer if the UnlockDialog is not visible
        i_timer->start();
    }
}

/**
 * This slot is called whenever a new database is set up.
 *
 * @param passphrase - The passphrase.
 * @param readonly   - Whether or not the database was opened readonly.
 */
void MainWindow::slot_setupTree (const QString & passphrase, bool readonly)
{
    pwsHandler->setPassphrase (passphrase);
    ConfigStore::store["readonly"] = readonly ? "1" : "0";

    if (pwsHandler->open (ConfigStore::store["curfile"]))
        { setStatus (MainWindow::FileOpened); }
    else
        { pwsHandler->restorePassphrase(); }

    slot_resizeColumn(QModelIndex());
}

/**
 * Whenever a database is saved, a backup of the original database will
 * be made. This behaviour is configurable by changing the "prefs_pws_save_backups_dir",
 * "prefs_pws_save_num_backups" and "prefs_pws_save_backups" preferences.
 */
void MainWindow::saveBackup(QString filename)
{
    QFileInfo info(filename);
    QFile backup(filename);

    // do not overwrite any existing file
    if (!backup.exists())
    {
        return;
    }

    if (ConfigStore::store["prefs_pws_save_backups_dir"] != "")
    {
        // if the backup should be saved inside some other directory ...
        QDir directory (ConfigStore::store["prefs_pws_save_backups_dir"]);
        if (!directory.exists())
        {
            if (!directory.mkpath (ConfigStore::store["prefs_pws_save_backups_dir"]))
            {
                // if the path does not exist already, create it
                QMessageBox::critical(this, tr("Error"), tr("Cannot create backup directory (") 
                  + ConfigStore::store["prefs_pws_save_backups_dir"] + ")");
            }
            else
            {
                // ... change the pathname
                filename = ConfigStore::store["prefs_pws_save_backups_dir"] + QDir::separator() + info.fileName();
            }
        }
        else
        {
            // ... change the pathname
            filename = ConfigStore::store["prefs_pws_save_backups_dir"] + QDir::separator() + info.fileName();
        }
    }

    // append a timestamp and finally rename it
    QString backup_name = filename + "." + QDateTime::currentDateTime().toString("yyyyMMddhhmmsszzz");
    if (!backup.rename (backup_name))
    {
        QMessageBox::critical(this, tr("Error"), tr("Cannot save backup (") + backup_name+ ")");
    }
    backup.close();

    if (ConfigStore::store["prefs_pws_save_num_backups"] != "")
    {
        // if "prefs_pws_save_num_backups" is set, ensure that no more backups, resp.
        // the oldest backup will be deleted if necessary

        QDir backup_dir = info.absoluteDir();
        if (ConfigStore::store["prefs_pws_save_backups_dir"] != "")
            { backup_dir = ConfigStore::store["prefs_pws_save_backups_dir"]; }

        backup_dir.setSorting(QDir::Time);
        backup_dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);

        QFileInfoList backups = backup_dir.entryInfoList(QStringList(info.fileName()+".*"));
        if (backups.size() > ConfigStore::store["prefs_pws_save_num_backups"].toInt())
        {
            for (int x = ConfigStore::store["prefs_pws_save_num_backups"].toInt(); x < backups.size(); x++)
            {
                QFile to_be_deleted (backups.at(x).absoluteFilePath());
                to_be_deleted.remove();
            }
        }
    }
}

/**
 * Export the current selected subtree.
 */
void MainWindow::slot_export()
{
    QModelIndex index = view->currentIndex();
    if (!index.isValid()) { return; }

    QPWSFileDialog *dialog = new QPWSFileDialog (this, tr("Export subtree"), ConfigStore::store["prefs_pws_cur_dir"],
      tr("Passwordsafe v3 format (*.psafe3);;Passwordsafe v2 format (*.dat);;All files (*)"));

    dialog->setWindowModality (Qt::ApplicationModal);
    dialog->setAcceptMode(QFileDialog::AcceptSave);
    if (dialog->exec())
    {
        QStringList filenames = dialog->selectedFiles(); 

        if (filenames.isEmpty())
            { delete dialog; return; }

        ConfigStore::store["prefs_pws_cur_dir"] = QFileInfo (filenames.at(0)).absolutePath();

        // we will need that later in slot_exportTree()
        ConfigStore::store["exportfilename"] = filenames.at(0);

        QString filter = dialog->selectedFilter();
        if (filter.endsWith ("(*.psafe3)"))
            { ConfigStore::store["exportformat"] = "3"; }
        else if (filter.endsWith ("(*.dat)"))
            { ConfigStore::store["exportformat"] = "2"; }
        else if (filter.endsWith ("(*)"))
            { ConfigStore::store["exportformat"] = "3"; }

        // get the passphrase for the new database (the one to be exported)
        NewPassphraseDialog *pwd_dlg = new NewPassphraseDialog(this);
        pwd_dlg->setWindowModality(Qt::ApplicationModal);

        // connect the passphrase dialog - on submit, export the file
        connect (pwd_dlg, SIGNAL(submit(const QString &)), this, SLOT(slot_exportTree(const QString &)));

        pwd_dlg->show();
    }

    delete dialog;
}

/**
 * Export the current selected subtree.
 *
 * @param passphrase - The passphrase of the new database.
 */
void MainWindow::slot_exportTree(const QString & passphrase)
{
    QModelIndex index = view->currentIndex();
    if (!index.isValid()) { return; }

    pwsHandler->setPassphrase (passphrase);

    if (ConfigStore::store["exportformat"] == "2")
        { pwsHandler->exportPwsV2 (ConfigStore::store["exportfilename"], index); }
    else if (ConfigStore::store["exportformat"] == "3")
        { pwsHandler->exportPwsV3 (ConfigStore::store["exportfilename"], index); }

    pwsHandler->restorePassphrase ();
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
