/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui>
#include <QFileDialog>
#include <QAction>
#include <QMenu>
#include <QTextEdit>
#include <QMainWindow>
#include <QErrorMessage>
#include <QMessageBox>
#include <QClipboard>
#include <QTimer>
#include <QPersistentModelIndex>
#include <QUrl>
#include <QFileInfo>
#include <QDir>
#include <QStringList>
#include <iostream>
#include <sys/types.h>
#include <unistd.h>
#include "pwstreemodel.h"
#include "passphrasedialog.h"
#include "configstore.h"
#include "informationdialog.h"
#include "preferencesdialog.h"
#include "newpassphrasedialog.h"
#include "aboutdialog.h"
#include "unlockdialog.h"
#include "watchdog.h"
#include "startdialog.h"
#include "pwshandler.h"
#include "pwstreeview.h"
#include "libpws/pws2.h"
#include "libpws/pws3.h"

class MainWindow : public QMainWindow
{
    Q_OBJECT

    public:
        /**
         * This class represents the main window widget.
         */
        MainWindow();

        /**
         * Delete clpbrd_contents.
         */
        ~MainWindow();

        /**
         * Returns the PWSHandler object.
         *
         * @returns The PWSHandler object.
         */
        PWSHandler * handler();

        /**
         * This enumeration describes the current database status.
         */
        enum PWSFileStatus { FileModified, FileNew, FileOpened, FileClosed, FileSaved };

    protected:
        /**
         * This method will be called whenever the user requests the contextmenu.
         */
        void contextMenuEvent(QContextMenuEvent *event);

    signals:
        /**
         * This signal will be emitted whenever a database was opened.
         */
        void fileOpened();

        /**
         * This signal will be emitted whenever a database was saved.
         */
        void fileSaved();

        /**
         * This signal will be emitted whenever a new database was created.
         */
        void fileNew();

        /**
         * This signal will be emitted whenever a database was closed
         */
        void fileClosed();

        /**
         * This signal will be emitted whenever a database was modified
         */
        void fileModified();

    private slots:

        /**
         * Enables the Enter-/Return-Key-Shortcut.
         */
        void slot_enableEnterShortcut();

        /**
         * Disables the Enter-/Return-Key-Shortcut.
         */
        void slot_disableEnterShortcut();

        /**
         * This slot is called, when the current selected item changes.
         *
         * @param index - The new index.
         */
        void slot_currentChanged(const QModelIndex & index);

        /**
         * This slot is called, when the user double clicks an item.
         *
         * @param index - The doubleclicked index.
         */
        void slot_doubleClicked(const QModelIndex & index = QModelIndex());

        /**
         * This slot is called, when the user presses enter.
         */
        void slot_enterPressed();

        /**
         * This slot is called, when the user presses space.
         */
        void slot_spacePressed();

        /**
         * This slot is called when the user selects the "new" menu entry.
         */
        void slot_newFile();

        /**
         * This slot is called when the user selects the "open" menu entry.
         */
        void slot_open();

        /**
         * This slot is called, whenever the user clicks the import-menu-entry.
         */
        void slot_import();

        /**
         * This slot is called whe the user selects the "save" menu entry.
         */
        void slot_save();

        /**
         * This slot is called when the user selects the "save as" menu entry.
         */
        void slot_saveAs();

        /**
         * This slot is called, when the user selects the "change passphrase" menu entry.
         */
        void slot_changePassphrase();

        /**
         * This slot is called, when the user selects the "information" menu entry.
         */
        void slot_information();

        /**
         * This slot is called when the user selects the "add entry" menu entry.
         */
        void slot_addPassword ();

        /**
         * This slot is called, when the user selects the "create" menu entry.
         */
        void slot_createGroup ();

        /**
         * This slot is called, when the user selects the "delete" menu entry.
         */
        void slot_deleteEntry ();

        /**
         * This slot is called, when the user selects the "rename" menu entry.
         */
        void slot_renameEntry();

        /**
         * This slot is called, when the user selects the "properties" menu entry.
         */
        void slot_properties ();

        /**
         * This slot is called when the user selects the "fetch password" button
         * in the EditEntryDialog widget (see slot_properties()) or the corresponding
         * menu entry.
         */
        void slot_fetchPassword ();

        /**
         * This slot is called when the user selects the "fetch user" button
         * in the EditEntryDialog widget (see slot_properties()) or the corresponding
         * menu entry.
         */
        void slot_fetchUser ();

        /**
         * This slot is called when the user selects the "fetch url" button
         * in the EditEntryDialog widget (see slot_properties()) or the corresponding
         * menu entry.
         */
        void slot_fetchUrl ();

        /**
         * This slot is called when the user selects the "open url" menu entry.
         */
        void slot_openUrl ();

        /**
         * This slot is called when the user selects the "clear clipboard" menu entry.
         */
        void slot_clearClipboard ();

        /**
         * This slot is called when the user selects the "preferences" menu entry.
         */
        void slot_preferences ();

        /**
         * This slot is called when the user selects the "about" menu entry.
         */
        void slot_about();

        /**
         * Resize the columns to show complete cells.
         */
        void slot_resizeColumn (const QModelIndex &index);

        /**
         * This slot is called when the c_timer times out.
         */
        void slot_clearCClipboard();

        /**
         * This slot is called when the s_timer times out.
         */
        void slot_clearSClipboard();

        /**
         * This slot is called whenever the WatchDog recognize any event.
         */
        void slot_reset();

        /**
         * This slot is called when the user selects the "close" menu entry.
         */
        void slot_closeFile();

        /**
         * This slot is called when the user selects the "exit" menu entry.
         */
        void slot_exit();

        /**
         * This slot is called when the user selects "ok" in the NewPassphraseDialog object
         * (see slot_newFile()).
         *
         * @param passphrase - The passphrase.
         */
        void slot_newFileSubmit(const QString & passphrase);

        /**
         * This slot is called whenever a new database is set up.
         *
         * @param passphrase - The passphrase.
         * @param readonly   - Whether or not the database was opened readonly.
         */
        void slot_setupTree (const QString & passphrase, bool readonly);

        /**
         * This slot is called whenever the database was modified.
         */
        void slot_fileModified();

        /**
         * This slot is called when the user selects the "fetch history" button
         * in the EditEntryDialog widget (see slot_properties()).
         */
        void slot_fetchHistory();

        /**
         * This lsot is called whenever the tree was sorted. After sorting,
         * the columns are resized to show the whole centents in the cells.
         *
         * @param item - The item which subtree was sorted.
         */
        void slot_sorted(EntryItem *item);

        /**
         * This slot is called, when the user selects "ok" in the EditEntryDialog and the EditEntryDialog
         * was used to add a new, non-existing entry (see MainWindow::slot_addPassword())
         */
        void slot_addEntry(EntryItem *item);

        /**
         * This slot is called whenever the user left click inside the tree view.
         */
        void slot_clicked(const QModelIndex & index);

        /**
         * This slot imports another database below the current selected item.
         *
         * @param passphrase - The passphrase.
         */
        void slot_importTree(const QString & passphrase);

        /**
         * This slot is called, when the "ok" button was clicked inside the
         * NewPassphraseDialog object (see slot_changePassphrase()).
         *
         * @param passphrase - The passphrase.
         */
        void slot_passphraseChanged(const QString & passphrase);

        /**
         * This slot is called whenever a tree item is expanded. This slot ensures,
         * that the selected item is always on the top, scrolling the whole tree if necessary.
         *
         * @param index - The expanded tree item.
         */
        void slot_expanded(const QModelIndex & index);

        /**
         * This slot is called when the user changes any preferences (see slot_preferences()).
         */
        void slot_prefsChanged();

        /**
         * This slot is called, when the user selects "cancel" in the EditEntryDialog and the EditEntryDialog
         * was used to add a new, non-existing entry (see MainWindow::slot_addPassword())
         */
        void slot_newEntryCanceled(EntryItem *item);

        /**
         * This slot is called when the user selects one of the files
         * in the recent files menu.
         */
        void slot_openRecent();

        /**
         * This slot is called when the user selects the "lock" menu entry or the
         * i_timer times out.
         */
        void slot_lock ();

        /**
         * This slot is called when a valid passphrase was set inside the UnlockDialog
         * object (see slot_lock()):
         */
        void slot_unlock ();

        /**
         * This slot is called whenever the WatchDog recognize any event.
         */
        void slot_resetTimer ();

        /**
         * When the main window is closed, check whether the database is unsaved and if yes,
         * ask the user how to proceed. Just before closing the main window, the preferences
         * will be saved.
         */
        void closeEvent(QCloseEvent * event);

        /**
         * Export the current selected subtree.
         */
        void slot_export();

        /**
         * Export the current selected subtree.
         *
         * @param passphrase - The passphrase of the new database.
         */
        void slot_exportTree(const QString & passphrase);

    private:
        /**
         * Creates all actions.
         */
        void createActions();

        /**
         * Creates all menus (except the context menu).
         */
        void createMenus();

        /**
         * Create the toolbar.
         */
        void createToolBars();

        /**
         * Create the statusbar.
         */
        void createStatusBar();

        /**
         * Whenever a database is saved, a backup of the original database will
         * be made. This behaviour is configurable by changing the "prefs_pws_save_backups_dir",
         * "prefs_pws_save_num_backups" and "prefs_pws_save_backups" preferences.
         */
        void saveBackup(QString filename);

        /**
         * Updates the recent files menu.
         */
        void updateRecentFileActions();

        /**
         * If the database was modified, but not saved at this time, ask the user how to
         * proceed (save|discard|cancel).
         *
         * @returns true If the user clicked save/discard and fals if the user clicked cancel.
         */
        bool maybeSave();

        /**
         * Sets the current window/database status to on of the
         * following values:
         *
         * MainWindow::FileModified - The database was modified.
         * MainWindow::FileSaved    - The database was saved.
         * MainWindow::FileNew      - A new database was created.
         * MainWindow::FileOpened   - A existing database was opened.
         * MainWindow::FileClosed   - The database was closed.
         *
         * @param status - The new database status.
         */
        void setStatus (MainWindow::PWSFileStatus status);

        /**
         * This string contains the clipboard data after any
         * clipboard action was performed. Used to verify that 
         * the clipboard data belongs to pws before the contents is cleared.
         */
        QString *clpbrd_contents;

        /**
         * This WidgetList contains all windows that were hidden after
         * locking pws. Contains only any data when the UnlockDialog is
         * shown (when pws is locked).
         */
        QWidgetList hidden;

        /**
         * Menus.
         */
        QMenu *fileMenu;
        QMenu *editMenu;
        QMenu *helpMenu;

        /**
         * Toolbar.
         */
        QToolBar *toolbar;

        /**
         * This is the default clipboard idle timer.
         */
        QTimer *c_timer;

        /**
         * This is the selection clipboard idle timer.
         */
        QTimer *s_timer;

        /**
         * This is the pws idle timer.
         */
        QTimer *i_timer;

        /**
         * Actions.
         */
        QAction *newAct;
        QAction *openAct;
        QAction *importAct;
        QAction *exportAct;
        QAction *saveAct;
        QAction *saveAsAct;
        QAction *closeAct;
        QAction *chPwdAct;
        QAction *infoAct;
        QAction *lockAct;
        QAction *exitAct;

        QAction *addPwdAct;
        QAction *addGroupAct;
        QAction *delEntryAct;
        QAction *renameAct;
        QAction *propAct;
        QAction *fetchPwdAct;
        QAction *fetchUsrAct;
        QAction *fetchUrlAct;
        QAction *openUrlAct;
        QAction *clearClipboardAction;
        QAction *prefAct;
        QAction **recentFileActs;
        QAction *separatorAct;

        QAction *aboutAct;

        QShortcut *spaceShortcut;
        QShortcut *enterShortcut;
        QShortcut *returnShortcut;

        /**
         * The model/view objects.
         */
        QTreeView *view;
        PWSTreeModel *model;
        EntryItemDelegate *delegate;

        /**
         * This will be set to true, whenever the database was modified (until it is saved).
         */
        bool modified;

        /**
         * This will be set to true if a database is currently open.
         */
        bool fileOpen;

        /**
         * Holds the curren number of allocated QActions inside
         * recentFileActs.
         */
        int recentFileActsLen;

        /**
         * The PWSHandler object.
         */
        PWSHandler * pwsHandler;
};

/**
 * Make QFileDialog somewhat smarter.
 */
class QPWSFileDialog : public QFileDialog
{
    Q_OBJECT

    public:
        QPWSFileDialog (QWidget * parent = 0, const QString & caption = QString(), 
                                      const QString & directory = QString(), 
                                      const QString & filter = QString()) : QFileDialog (parent, caption, directory, filter) 
        { 
            if (selectedFilter().endsWith ("(*.psafe3)"))
                { setDefaultSuffix ("psafe3"); }
            else if (selectedFilter().endsWith ("(*.dat)"))
                { setDefaultSuffix ("dat"); }
            else if (selectedFilter().endsWith ("(*)"))
                { setDefaultSuffix (""); }
            connect (this, SIGNAL(filterSelected(const QString &)), this, SLOT(slot_filterSelected(const QString &))); 
        }

    protected slots:
        void slot_filterSelected (const QString & filter)
        {
            if (filter.endsWith ("(*.psafe3)"))
                { setDefaultSuffix ("psafe3"); }
            else if (filter.endsWith ("(*.dat)"))
                { setDefaultSuffix ("dat"); }
            else if (filter.endsWith ("(*)"))
                { setDefaultSuffix (""); }
        }
};

class QPWSTreeView : public QTreeView
{
    Q_OBJECT

    public:
        QPWSTreeView (QWidget * parent = 0) : QTreeView (parent) { } 

    signals:
        void dropSuccessfull();

    protected:
        QPoint dragStartPosition;

        void mousePressEvent(QMouseEvent *event)
        {
            QTreeView::mousePressEvent(event);

            if (event->button() == Qt::LeftButton)
            {
                dragStartPosition = event->pos(); 
                setCurrentIndex (indexAt(dragStartPosition));
            }
        }

        void mouseMoveEvent(QMouseEvent *event)
        {
            if (!(event->buttons() & Qt::LeftButton))
                { return; }

            if ((event->pos() - dragStartPosition).manhattanLength() < QApplication::startDragDistance())
                { return; }

            if (indexAt (dragStartPosition).isValid())
            {
                QPersistentModelIndex index (indexAt(dragStartPosition));

                QDrag *drag = new QDrag(this);
                EntryItem *item = static_cast<EntryItem*>(indexAt(dragStartPosition).internalPointer());

                if (item->isGroup())
                    { drag->setPixmap(QIcon(":/images/folder.png").pixmap(15,15)); }
                else
                    { drag->setPixmap(QIcon(":/images/entry.png").pixmap(15,15)); }

                drag->setMimeData(item->toMimeData());
                drag->exec(Qt::CopyAction | Qt::MoveAction);

                if (drag->target())
                    { model()->removeRow (index.row(), index.parent()); }
            }
        }

        void dragEnterEvent(QDragEnterEvent *event)
        {
            if (event->mimeData()->hasFormat("application/pws.dragdrop.entry"))
                { event->acceptProposedAction(); }
        }

        void dropEvent(QDropEvent *event)
        {
            QPersistentModelIndex index (indexAt(event->pos()));
            if (index == indexAt(dragStartPosition))
                { event->ignore(); return; }

            EntryItem * parent = ((PWSTreeModel*)model())->root();
            if (index.isValid())
                { parent = static_cast<EntryItem*>(index.internalPointer()); }

            event->acceptProposedAction();

            if (parent != ((PWSTreeModel*)model())->root())
                { setExpanded (index, true); }

            if (parent->isGroup())
            {
                QList<EntryItem*> entries = EntryItem::fromMimeData (event->mimeData());
                QListIterator<EntryItem*> iter(entries);
                while (iter.hasNext())
                    { parent->appendEntry (iter.next()); }
            }
            else
            {
                QList<EntryItem*> entries = EntryItem::fromMimeData (event->mimeData());
                QListIterator<EntryItem*> iter(entries);
                while (iter.hasNext())
                    { parent->getParent()->appendEntry (iter.next()); }
            }

            ((PWSTreeModel*)model())->sort(NULL);

            emit dropSuccessfull();
        }

        void dragMoveEvent (QDragMoveEvent * event) 
        {
            EntryItem * item    = static_cast<EntryItem*>(indexAt(dragStartPosition).internalPointer());
            EntryItem * curItem = static_cast<EntryItem*>(indexAt(event->pos()).internalPointer());

            if (curItem == item || curItem == item->getParent() || 
                (!indexAt(event->pos()).isValid() && item->getParent() == ((PWSTreeModel*)model())->root()))
                { event->ignore(); return; }

            while (curItem && curItem->getParent())
            {
                if (curItem->getParent() == item)
                    { event->ignore(); return; }
                curItem = curItem->getParent();
            }

            if (event->mimeData()->hasFormat("application/pws.dragdrop.entry"))
                { event->acceptProposedAction(); }
            else
                { event->ignore(); }
        }
};

#endif

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
