/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "entryitem.h"

/**
 * Creates a new EntryItem object. This object will
 * represant a group entry.
 *
 * @param name   - The name of the group.
 */
EntryItem::EntryItem(const QString & name)
{
    m_parent = NULL;
    m_name   = name;
    m_pwd    = NULL;
    m_is_group = true;
    m_expanded = false;

    m_unknown = NULL;
    m_unknown_len = 0;
}

/**
 * Creates a new EntryItem object. This object will
 * represant a password entry.
 *
 * @param name        - The name of this entry.
 * @param uuid        - The uuid of this entry.
 * @param pwd         - The password.
 * @param url         - The URL for this entry.
 * @param autotype    - The autotype for this entry.
 * @param ctime       - Creation Time.
 * @param pwd_mtime   - Password modification time.
 * @param mtime       - Modification time.
 * @param atime       - Access time.
 * @param ltime       - Password lifetime.
 * @param parent      - The parent item or NULL if this is the root entry-
 */
EntryItem::EntryItem(const QString & name, const QUuid & uuid, 
                     const QString & user, const QString & notes, 
                     const QString & pwd, 
                     const QUrl & url,  const QString & autotype,
                     int   save_pwd_history, int max_hist_size,
                     const QDateTime & ctime,  const QDateTime & pwd_mtime, 
                     const QDateTime & mtime,  const QDateTime & atime, 
                     const QDateTime & ltime,  char *unknown, unsigned int unknown_len)
{
    m_name        = name;
    m_uuid        = uuid,
    m_user        = user;
    m_pwd         = new QString (pwd);
    m_url         = url;
    m_autotype    = autotype;
    m_ctime       = ctime;
    m_pwd_mtime   = pwd_mtime;
    m_mtime       = mtime;
    m_atime       = atime,
    m_ltime       = ltime;
    m_parent      = NULL;
    m_save_pwd_history = save_pwd_history;
    m_max_hist_size    = max_hist_size;

    m_unknown = NULL;
    m_unknown_len = 0;
    if (unknown_len)
    {
        m_unknown_len = unknown_len;
        m_unknown = (char*)malloc (m_unknown_len*sizeof(char));
        memcpy (m_unknown, unknown, m_unknown_len);
    }

    QStringList notesList = notes.split ('\n');
    m_notes = notesList[0];
    m_notes_long = notes;

    m_is_group = false;
}

/**
 * Deletes all child items on destruction.
 */
EntryItem::~EntryItem()
{
    qDeleteAll(m_childs);
    clearPwdHistory();

    if (m_unknown) free (m_unknown);

    delete m_pwd;
}

/**
 * Clears the password history.
 */
void EntryItem::clearPwdHistory()
{
    QListIterator<HistoryEntry> pwd_history(m_pwd_history);
    while (pwd_history.hasNext()) 
        { delete (pwd_history.next().password); }

    m_pwd_history.clear();
}

/**
 * Adds a new password history entry.
 *
 * @param timestamp - The creation time of the password.
 * @param pwd       - The password to be added.
 */
void EntryItem::addPwdHistoryEntry(const QDateTime & timestamp, const QString & pwd)
{
    if ((m_save_pwd_history == -1 && ConfigStore::store["prefs_bool_savepasswordhistory"] == "0") ||
         m_save_pwd_history == 0)
        { return; }

    int max = 0;
    if (m_max_hist_size == -1)
        { max = ConfigStore::store["prefs_int_numpwhistorydefault"].toInt(); }
    else
        { max = m_max_hist_size; }

    HistoryEntry entry;
    entry.timestamp = timestamp.toTime_t();
    entry.password  = new QString (pwd);

    m_pwd_history.append (entry);

    if (m_pwd_history.size() > max && m_pwd_history.size())
    {
        delete m_pwd_history.at(0).password; 
        m_pwd_history.removeAt(0);
    }
}

/**
 * Returns the password history.
 *
 * @returns the password history.
 */
const QList<HistoryEntry> & EntryItem::getPwdHistory()
{
    return m_pwd_history;
}

/**
 * Returns m_childs.
 */
QList<EntryItem*> EntryItem::childs()
{
    return m_childs;
}

/**
 * Returns m_parent.
 */
EntryItem * EntryItem::getParent()
{
    return m_parent;
}

/**
 * Sets m_parent.
 */
void EntryItem::setParent(EntryItem * parent)
{
    m_parent = parent;
}

/**
 * Sets m_uuid.
 */
void EntryItem::setUuid(const QUuid & uuid)
{
    m_uuid = uuid;
}

/**
 * Sets m_save_pwd_history.
 */
void EntryItem::setSavePwdHistory(int use_pwd_history)
{
    m_save_pwd_history = use_pwd_history;
}

/**
 * Sets m_max_hist_size.
 */
void EntryItem::setMaxHistSize(int max_hist_size)
{
    m_max_hist_size = max_hist_size;
}

/**
 * Sets m_user.
 */
void EntryItem::setUser(const QString & user)
{
    m_user = user;
}

/**
 * Sets m_pwd.
 */
void EntryItem::setPwd(const QString & pwd)
{
    *m_pwd = pwd;
}

/**
 * Sets m_expanded.
 */
void EntryItem::setExpanded(bool expanded)
{
    if (m_is_group)
        m_expanded = expanded;
}

/**
 * Sets m_pwd_url.
 */
void EntryItem::setUrl(const QUrl & url)
{
    m_url = url;

}

/**
 * Sets m_autotype.
 */
void EntryItem::setAutoType(const QString & autotype)
{
    m_autotype = autotype;
}

/**
 * Sets m_notes_long.
 */
void EntryItem::setNotes(const QString & notes)
{
    QStringList notesList = notes.split ('\n');
    m_notes = notesList[0];
    m_notes_long = notes;
}

/**
 * Sets m_ctime.
 */
void EntryItem::setCTime(const QDateTime & ctime)
{
    m_ctime = ctime;
}

/**
 * Sets m_mtime.
 */
void EntryItem::setMTime(const QDateTime & mtime)
{
    m_mtime = mtime;
}

/**
 * Sets m_pwd_mtime.
 */
void EntryItem::setPwdMTime(const QDateTime & pwd_mtime)
{
    m_pwd_mtime = pwd_mtime;
}

/**
 * Sets m_atime.
 */
void EntryItem::setATime(const QDateTime & atime)
{
    m_atime = atime;
}

/**
 * Sets m_ltime.
 */
void EntryItem::setLTime(const QDateTime & atime)
{
    m_atime = atime;
}

/**
 * Clears the whole subtree from this entry on
 * (except this node).
 */
void EntryItem::clear()
{
    qDeleteAll(m_childs);
    m_childs.clear();
}

/**
 * Returns m_expanded.
 */
bool EntryItem::isExpanded()
{
    return m_expanded;
}

/**
 * Returns m_name.
 */
const QString & EntryItem::getName()
{
    return m_name;
}

/**
 * Returns m_uuid.
 */
const QUuid & EntryItem::getUuid()
{
    return m_uuid;
}

/**
 * Returns m_save_pwd_history.
 */
int EntryItem::getSavePwdHistory()
{
    return m_save_pwd_history;
}

/**
 * Returns m_max_hist_size.
 */
int EntryItem::getMaxHistSize()
{
    return m_max_hist_size;
}

/**
 * Returns m_user.
 */
const QString & EntryItem::getUser()
{
    return m_user;
}

/**
 * Returns m_notes_long.
 */
const QString & EntryItem::getNotes()
{
    return m_notes_long;
}

/**
 * Returns m_pwd.
 */
const QString & EntryItem::getPwd()
{
    return (*m_pwd);
}

/**
 * Returns m_url.
 */
const QUrl & EntryItem::getUrl()
{
    return m_url;
}

/**
 * Returns m_autotype.
 */
const QString & EntryItem::getAutoType()
{
    return m_autotype;
}

/**
 * Returns m_ctime.
 */
const QDateTime & EntryItem::getCTime()
{
    return m_ctime;
}

/**
 * Returns m_mtime.
 */
const QDateTime & EntryItem::getMTime()
{
    return m_mtime;
}

/**
 * Returns m_pwd_mtime.
 */
const QDateTime & EntryItem::getPwdMTime()
{
    return m_pwd_mtime;
}

/**
 * Returns m_atime.
 */
const QDateTime & EntryItem::getATime()
{
    return m_atime;
}

/**
 * Returns m_ltime.
 */
const QDateTime & EntryItem::getLTime()
{
    return m_ltime;
}

/**
 * Adds a new child to this entry. All child entries will be deleted 
 * automatically on destruction.
 *
 * @param child - The child to be added.
 */
void EntryItem::appendEntry(EntryItem *child)
{
    QString tmp_name = child->getName();

    if (child->isGroup())
    {
        int i = 1;
        while (findGroup(child->getName()))
            { child->rename (tmp_name + " (" + QString::number(i++) + ")");}
    }

    m_childs.append(child);
    child->setParent (this);
}

/**
 * Sorts the whole (sub)tree.
 */
void EntryItem::sort()
{
    bool sorted = false;
    while (!sorted)
    {
        sorted = true;
        for (int z = 0; z < m_childs.size()-1; z++) 
        {
            if (m_childs[z]->m_is_group && m_childs[z+1]->m_is_group)
            {
                if (m_childs[z]->m_name.toLower() > m_childs[z+1]->m_name.toLower())
                { 
                    EntryItem *tmp = m_childs[z];
                    m_childs[z] = m_childs[z+1];
                    m_childs[z+1] = tmp;
                    sorted = false;
                }
            }
            else if (!m_childs[z]->m_is_group && m_childs[z+1]->m_is_group)
            {
                EntryItem *tmp = m_childs[z];
                m_childs[z] = m_childs[z+1];
                m_childs[z+1] = tmp;
                sorted = false;
            }
            else if (!m_childs[z]->m_is_group && !m_childs[z+1]->m_is_group)
            {
                if (m_childs[z]->m_name.toLower() > m_childs[z+1]->m_name.toLower())
                { 
                    EntryItem *tmp = m_childs[z];
                    m_childs[z] = m_childs[z+1];
                    m_childs[z+1] = tmp;
                    sorted = false;
                }
            }
        }
    }

    for (int y = 0; y < m_childs.size(); y++)
        { m_childs[y]->sort (); }
}

/**
 * Find a entry.
 *
 * @param name - The name of the entry that should be returned.
 *
 * @returns The EntryItem object or NULL if no suitable entry could be found.
 */
EntryItem *EntryItem::findEntry(const QString & name)
{
    for (int i = 0; i < m_childs.size(); i++) 
    {
        if (m_childs.at(i)->m_name == name && !m_childs.at(i)->m_is_group)
            { return m_childs.at(i); }
    }

    return (NULL);
}

/**
 * Find a group.
 *
 * @param group - The name of the group that should be returned.
 *
 * @returns The EntryItem object or NULL if no suitable entry could be found.
 */
EntryItem *EntryItem::findGroup(const QString & name)
{
    for (int i = 0; i < m_childs.size(); i++) 
    {
        if (m_childs.at(i)->m_name == name && m_childs.at(i)->m_is_group)
            { return m_childs.at(i); }
    }

    return (NULL);
}

/**
 * Returns the entry at index row.
 *
 * @param row - The index of the entry that should be returned.
 *
 * @returns The child at index row.
 */
EntryItem *EntryItem::child(int row)
{
    return m_childs.value(row);
}

/**
 * Returns the number of childs in this group.
 *
 * @returns The number of childs.
 */
int EntryItem::childCount() const
{
    return m_childs.count();
}

/**
 * Returns the data of this entry at column col.
 *
 * @returns the data of this entry at column col.
 */
QVariant EntryItem::data(int col) const
{
    if (m_is_group)
    {
        switch (col)
        {
            // column data
            case 0: 
                return m_name;
                break;
            // tooltips
            case 100: 
                return m_name;
                break;
        }
    }
    else
    {
        switch (col)
        {
            // column data
            case 0: 
                return m_name;
                break;
            case 1: 
                return m_user;
                break;
            case 2: 
                return m_notes;
                break;
            case 3: 
                if (m_mtime >  m_pwd_mtime)
                { 
                    if (m_mtime != QDateTime::fromTime_t(0))
                        { return m_mtime.toString(ConfigStore::store["prefs_pws_date_format"]);  }
                }
                else
                { 
                    if (m_pwd_mtime != QDateTime::fromTime_t(0))
                        { return m_pwd_mtime.toString(ConfigStore::store["prefs_pws_date_format"]); } 
                }
                break;
            case 4: 
                if (m_atime != QDateTime::fromTime_t(0))
                    { return m_atime.toString(ConfigStore::store["prefs_pws_date_format"]); }
                break;
            case 5: 
                if (m_ctime != QDateTime::fromTime_t(0))
                    { return m_ctime.toString(ConfigStore::store["prefs_pws_date_format"]); }
                break;
            // tooltips
            case 100: 
                return m_name;
                break;
            case 101: 
                return m_user;
                break;
            case 102: 
                return m_notes_long;
                break;
            case 103: 
                if (m_mtime >  m_pwd_mtime)
                { 
                    if (m_mtime != QDateTime::fromTime_t(0))
                        { return m_mtime.toString(ConfigStore::store["prefs_pws_date_format"]);  }
                }
                else
                { 
                    if (m_pwd_mtime != QDateTime::fromTime_t(0))
                        { return m_pwd_mtime.toString(ConfigStore::store["prefs_pws_date_format"]); } 
                }
                break;
            case 104: 
                if (m_atime != QDateTime::fromTime_t(0))
                    { return m_atime.toString(ConfigStore::store["prefs_pws_date_format"]); }
                break;
            case 105: 
                if (m_ctime != QDateTime::fromTime_t(0))
                    { return m_ctime.toString(ConfigStore::store["prefs_pws_date_format"]); }
                break;
        }
    }

    return ("");
}

/**
 * Returns the parent item.
 *
 * @returns the parent item.
 */
EntryItem *EntryItem::parent()
{
    return m_parent;
}

/**
 * Returns the index of this entry inside m_parent.
 *
 * @returns the index of this entry inside m_parent.
 */
int EntryItem::row() const
{
    if (m_parent)
        return m_parent->m_childs.indexOf(const_cast<EntryItem*>(this));

    return 0;
}

/**
 * Returns true, if this entry is a group.
 *
 * @returns true, if this entry is a group.
 */
bool EntryItem::isGroup()
{
    return m_is_group;
}

/**
 * Renames the current item.
 *
 * @param name - The new name for this entry.
 */
void EntryItem::rename(const QString & name)
{
    QString tmp_name = name;
    if (m_parent && m_is_group)
    {
        int i = 1;
        while (m_parent->findGroup(tmp_name))
            { tmp_name = name + " (" + QString::number(i++) + ")";}
    }

    m_name = tmp_name;
}

/**
 * Deletes one child item (incl. childs).
 *
 * @param item - The item to be removed
 * @param del  - If set to true, all child items will be deleted,
 *               otherwise they will just be removed.
 */
void EntryItem::removeEntry(EntryItem * item, bool del)
{
    for (int z = 0; z < m_childs.size(); z++) 
    {
        if (m_childs.at(z) == item)
        {
            if (del) delete m_childs.at(z);
            else m_childs.at(z)->setParent (NULL);

            m_childs.removeAt(z);
            return;
        }
    }
}

/**
 * Deletes all child items (incl. childs).
 *
 * @param del  - If set to true, all child items will be deleted,
 *               otherwise they will just be removed.
 */
void EntryItem::removeAllEntries (bool del)
{
    while (!m_childs.isEmpty())
        { removeEntry (m_childs.at(0), del); }
}

/**
 * Returns the number of child entries, excluding groups.
 */
int EntryItem::numEntries()
{
    int num = 0;
    for (int z = 0; z < m_childs.size(); z++) 
    {
        if (m_childs.at(z)->isGroup())
            { num += m_childs.at(z)->numEntries(); }
        else
            { num++; }
    }

    return num;
}

/**
 * Returns the number of groups, excluding password entries.
 */
int EntryItem::numGroups()
{
    int num = 0;
    for (int z = 0; z < m_childs.size(); z++) 
    {
        if (m_childs.at(z)->isGroup())
            { num += m_childs.at(z)->numGroups() + 1; }
    }

    return num;
}

/**
 * Convenience method that calls toAscii(m_name);
 */
QByteArray EntryItem::toAscii()
{
    return toAscii(m_name, true);
}

/**
 * Returns a "application/pws.dragdrop.entry" Ascii representation of this entry and all sub-entries.
 */
QByteArray EntryItem::toAscii(QString group, bool firstrun)
{
    QByteArray data ("");

    if (!firstrun && m_is_group)
        { group = group + "." + m_name; }

    if (!firstrun || m_is_group)
        { data += "group=" + QString (group).toUtf8().toBase64() + ";"; }

    if (!m_is_group)
    {
        data +=  "name="             + QString (m_name).toUtf8().toBase64()                        + ";";
        data +=  "uuid="             + m_uuid.toString().toUtf8().toBase64()                       + ";";
        data +=  "user="             + m_user.toUtf8().toBase64()                                  + ";";
        data +=  "notes="            + m_notes_long.toUtf8().toBase64()                            + ";";
        data +=  "password="         + m_pwd->toUtf8().toBase64()                                  + ";";
        data +=  "url="              + m_url.toString().toUtf8().toBase64()                        + ";";
        data +=  "autotype="         + m_autotype.toUtf8().toBase64()                              + ";";
        data +=  "ctime="            + QString::number(m_ctime.toTime_t()).toUtf8().toBase64()     + ";";
        data +=  "pwd_mtime="        + QString::number(m_pwd_mtime.toTime_t()).toUtf8().toBase64() + ";";
        data +=  "mtime="            + QString::number(m_mtime.toTime_t()).toUtf8().toBase64()     + ";";
        data +=  "atime="            + QString::number(m_atime.toTime_t()).toUtf8().toBase64()     + ";";
        data +=  "ltime="            + QString::number(m_ltime.toTime_t()).toUtf8().toBase64()     + ";";
        data +=  "save_pwd_history=" + QString::number(m_save_pwd_history).toUtf8().toBase64()     + ";";
        data +=  "max_hist_size="    + QString::number(m_max_hist_size).toUtf8().toBase64()        + ";";
        data +=  "is_expanded="      + QString::number(m_expanded).toUtf8().toBase64()             + ";";
        data +=  "unknown="          + QByteArray(m_unknown, m_unknown_len).toBase64()             + ";";

        for (int i = 0; i < m_pwd_history.size(); i++)
        {
            QByteArray hst_entry (QString::number(m_pwd_history.at(i).timestamp).toUtf8());
            hst_entry.append (QString(":").toUtf8());
            hst_entry.append (m_pwd_history.at(i).password->toUtf8());

            data += "pwd_history_entry=" + hst_entry.toBase64() + ";";
        }
    }

    data += "\n";

    for (int z = 0; z < m_childs.size(); z++) 
        { data += m_childs.at(z)->toAscii (group, false); }

    return data;
}

/**
 * Parses a "application/pws.dragdrop.entry" Ascii String and returns
 * all decoded entries.
 */
QList<EntryItem*> EntryItem::fromAscii(QString data)
{
    EntryItem root("root");

    QStringList entry_list = data.split ("\n", QString::SkipEmptyParts);
    for (int i = 0; i < entry_list.size(); i++)
    {
        QString name       = "";
        QString user       = "";
        QString password   = "";
        QString notes      = "";
        QString autotype   = "";

        int save_pwd_history = 0;
        int max_hist_size = 0;
        int is_expanded = 0;

        char * unknown = NULL;
        int unknown_len = 0;
        QUrl url;

        QUuid uuid;

        QDateTime mtime;
        QDateTime pwd_mtime;
        QDateTime ctime;
        QDateTime atime;
        QDateTime ltime;

        EntryItem * pos = &root;

        QList<HistoryEntry> pwd_history;

        QStringList field_list = entry_list.at(i).split (";", QString::SkipEmptyParts);
        for (int z = 0; z < field_list.size(); z++)
        {
            if (field_list.at(z).startsWith ("group="))
            {
                field_list[z].replace ("group=", "");
                QString group = QString::fromUtf8((QByteArray::fromBase64(field_list.at(z).toAscii())));
                
                QStringList group_list = group.split (".", QString::SkipEmptyParts);
                for (int y = 0; y < group_list.size(); y++)
                {
                    EntryItem * tmp_pos = pos->findGroup (group_list.at(y));
                    if (!tmp_pos)
                    {
                        pos->appendEntry (new EntryItem (group_list.at(y)));
                        pos = pos->findGroup (group_list.at(y));
                    }
                    else
                    {
                        pos = tmp_pos;
                    }
                }
            }
            else if (field_list.at(z).startsWith("name="))
            {
                field_list[z].replace ("name=", "");
                name = QString::fromUtf8(QByteArray::fromBase64(field_list.at(z).toAscii()));
            }
            else if (field_list.at(z).startsWith("uuid="))
            {
                field_list[z].replace ("uuid=", "");
                uuid = QString::fromUtf8 ((QByteArray::fromBase64(field_list.at(z).toAscii())));
            }
            else if (field_list.at(z).startsWith("user="))
            {
                field_list[z].replace ("user=", "");
                user = QString::fromUtf8 (QByteArray::fromBase64(field_list.at(z).toAscii()));
            }
            else if (field_list.at(z).startsWith("notes="))
            {
                field_list[z].replace ("notes=", "");
                notes = QString::fromUtf8 (QByteArray::fromBase64(field_list.at(z).toAscii()));
            }
            else if (field_list.at(z).startsWith("password="))
            {
                field_list[z].replace ("password=", "");
                password = QString::fromUtf8 (QByteArray::fromBase64(field_list.at(z).toAscii()));
            }
            else if (field_list.at(z).startsWith("url="))
            {
                field_list[z].replace ("url=", "");
                url = QString::fromUtf8 ((QByteArray::fromBase64(field_list.at(z).toAscii())));
            }
            else if (field_list.at(z).startsWith("autotype="))
            {
                field_list[z].replace ("autotype=", "");
                autotype = QString::fromUtf8 (QByteArray::fromBase64(field_list.at(z).toAscii()));
            }
            else if (field_list.at(z).startsWith("ctime="))
            {
                field_list[z].replace ("ctime=", "");
                ctime = QDateTime::fromTime_t (QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt());
            }
            else if (field_list.at(z).startsWith("pwd_mtime="))
            {
                field_list[z].replace ("pwd_mtime=", "");
                pwd_mtime = QDateTime::fromTime_t (QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt());
            }
            else if (field_list.at(z).startsWith("mtime="))
            {
                field_list[z].replace ("mtime=", "");
                mtime = QDateTime::fromTime_t (QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt());
            }
            else if (field_list.at(z).startsWith("atime="))
            {
                field_list[z].replace ("atime=", "");
                atime = QDateTime::fromTime_t (QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt());
            }
            else if (field_list.at(z).startsWith("ltime="))
            {
                field_list[z].replace ("ltime=", "");
                ltime = QDateTime::fromTime_t (QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt());
            }
            else if (field_list.at(z).startsWith("save_pwd_history="))
            {
                field_list[z].replace ("save_pwd_history=", "");
                save_pwd_history = QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt();
            }
            else if (field_list.at(z).startsWith("max_hist_size="))
            {
                field_list[z].replace ("max_hist_size=", "");
                max_hist_size = QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt();
            }
            else if (field_list.at(z).startsWith("is_expanded="))
            {
                field_list[z].replace ("is_expanded=", "");
                is_expanded = QString (QByteArray::fromBase64(field_list.at(z).toAscii())).toInt();
            }
            else if (field_list.at(z).startsWith("unknown="))
            {
                field_list[z].replace ("unknown=", "");
                unknown = QByteArray::fromBase64(field_list.at(z).toAscii()).data();
                unknown_len = QByteArray::fromBase64(field_list.at(z).toAscii()).size();
            }
            else if (field_list.at(z).startsWith("pwd_history_entry="))
            {
                field_list[z].replace ("pwd_history_entry=", "");
                QString pwd_history_entry = QString::fromUtf8 (QByteArray::fromBase64(field_list.at(z).toAscii()).data());

                QStringList hst_entry_lst = pwd_history_entry.split (":");

                HistoryEntry entry;
                entry.timestamp = hst_entry_lst.at(0).toInt();
                entry.password  = new QString (hst_entry_lst.at(1));

                pwd_history.append (entry);
            }
        }

        if (!uuid.isNull())
        {
            EntryItem * newItem = new EntryItem (name, uuid, user, notes, password, url,
                                                 autotype, save_pwd_history, max_hist_size, ctime,
                                                 pwd_mtime, mtime, atime, ltime, unknown, unknown_len);

            newItem->setExpanded(is_expanded);

            QListIterator<HistoryEntry> iter (pwd_history);
            while (iter.hasNext()) 
            {
                HistoryEntry entry = iter.next();
                newItem->addPwdHistoryEntry (QDateTime::fromTime_t(entry.timestamp), *entry.password); 
                delete entry.password;
            }

            pos->appendEntry (newItem);
        }
    }

    QList<EntryItem*> entries = root.childs();
    root.removeAllEntries(false);

    return entries;
}

/**
 * Returns a "application/pws.dragdrop.entry" QMimeData representation of this entry and all sub-entries.
 */
QMimeData * EntryItem::toMimeData()
{
    QMimeData *mimeData = new QMimeData();
    mimeData->setData("application/pws.dragdrop.entry", toAscii());

    return mimeData;
}

/**
 * Parses a "application/pws.dragdrop.entry" QMimeData object and returns
 * all decoded entries.
 */
QList<EntryItem*> EntryItem::fromMimeData (const QMimeData *mimeData)
{
    if (mimeData->hasFormat ("application/pws.dragdrop.entry"))
        { return EntryItem::fromAscii(QString (mimeData->data("application/pws.dragdrop.entry"))); }

    QList<EntryItem*> empty;
    return empty;
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
