/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef ENTRYITEM_H
#define ENTRYITEM_H

#include <QList>
#include <QUuid>
#include <QDate>
#include <QString>
#include <QVariant>
#include <QStringList>
#include <QList>
#include <QUrl>
#include <QMimeData>
#include "configstore.h"
#include "libpws/util.h"
#include "libpws/pws2.h"
#include "libpws/pws3.h"

class HistoryEntry
{
    public:
        time_t   timestamp;
        QString *password;
};

class EntryItem
{
    friend class PWSHandler;

    public:
        /**
         * Creates a new EntryItem object. This object will
         * represant a group entry.
         *
         * @param name   - The name of the group.
         */
        EntryItem(const QString & name);

        /**
         * Creates a new EntryItem object. This object will
         * represant a password entry.
         *
         * @param name        - The name of this entry.
         * @param uuid        - The uuid of this entry.
         * @param pwd         - The password.
         * @param url         - The URL for this entry.
         * @param autotype    - The autotype for this entry.
         * @param ctime       - Creation Time.
         * @param pwd_mtime   - Password modification time.
         * @param mtime       - Modification time.
         * @param atime       - Access time.
         * @param ltime       - Password lifetime.
         */
        EntryItem(const QString & name, const QUuid & uuid, 
                  const QString & user, const QString & notes, 
                  const QString & pwd,
                  const QUrl & url,  const QString & autotype,
                  int save_pwd_history, int max_hist_size,
                  const QDateTime & ctime,  const QDateTime & pwd_mtime, 
                  const QDateTime & mtime,  const QDateTime & atime, 
                  const QDateTime & ltime,  char *unknown, unsigned int unknown_len);

        /**
         * Deletes all child items on destruction.
         */
        ~EntryItem();

        /**
         * Sets m_parent.
         */
        void setParent(EntryItem * parent);

        /**
         * Sets m_expanded.
         */
        void setExpanded(bool expanded);

        /**
         * Sets m_uuid.
         */
        void setUuid(const QUuid & uuid);

        /**
         * Sets m_user.
         */
        void setUser(const QString & user);

        /**
         * Sets m_pwd.
         */
        void setPwd(const QString & pwd);

        /**
         * Sets m_pwd_url.
         */
        void setUrl(const QUrl & url);

        /**
         * Sets m_autotype.
         */
        void setAutoType(const QString & autotype);

        /**
         * Sets m_notes_long.
         */
        void setNotes(const QString & notes);

        /**
         * Sets m_ctime.
         */
        void setCTime(const QDateTime & ctime);

        /**
         * Sets m_mtime.
         */
        void setMTime(const QDateTime & mtime);

        /**
         * Sets m_pwd_mtime.
         */
        void setPwdMTime(const QDateTime & pwd_mtime);

        /**
         * Sets m_atime.
         */
        void setATime(const QDateTime & atime);

        /**
         * Sets m_ltime.
         */
        void setLTime(const QDateTime & atime);

        /**
         * Returns m_parent.
         */
        EntryItem * getParent();

        /**
         * Returns m_expanded.
         */
        bool isExpanded();

        /**
         * Returns m_childs.
         */
        QList<EntryItem*> childs();

        /**
         * Returns m_name;
         */
        const QString & getName();

        /**
         * Returns m_uuid.
         */
        const QUuid & getUuid();

        /**
         * Returns m_user.
         */
        const QString & getUser();

        /**
         * Returns m_pwd.
         */
        const QString & getPwd();

        /**
         * Returns m_pwd_url.
         */
        const QUrl & getUrl();

        /**
         * Returns m_autotype.
         */
        const QString & getAutoType();

        /**
         * Returns m_notes_long.
         */
        const QString & getNotes();

        /**
         * Returns m_ctime.
         */
        const QDateTime & getCTime();

        /**
         * Returns m_mtime.
         */
        const QDateTime & getMTime();

        /**
         * Returns m_pwd_mtime.
         */
        const QDateTime & getPwdMTime();

        /**
         * Returns m_atime.
         */
        const QDateTime & getATime();

        /**
         * Returns m_ltime.
         */
        const QDateTime & getLTime();

        /**
         * Clears the whole subtree from this entry on
         * (except this node).
         */
        void clear();

        /**
         * Sorts the whole (sub)tree.
         */
        void sort();

        /**
         * Adds a new child to this entry. All child entries will be deleted 
         * automatically on destruction.
         *
         * @param child - The child to be added.
         */
        void appendEntry (EntryItem *child);

        /**
         * Deletes one child item (incl. childs).
         *
         * @param item - The item to be removed
         * @param del  - If set to true, the child item will be deleted,
         *               otherwise it will just be removed.
         */
        void removeEntry (EntryItem * item, bool del = true);

        /**
         * Deletes all child items (incl. childs).
         *
         * @param del  - If set to true, all child items will be deleted,
         *               otherwise they will just be removed.
         */
        void removeAllEntries (bool del = true);

        /**
         * Find a entry.
         *
         * @param name - The name of the entry that should be returned.
         *
         * @returns The EntryItem object or NULL if no suitable entry could be found.
         */
        EntryItem *findEntry (const QString & name);

        /**
         * Find a group.
         *
         * @param group - The name of the group that should be returned.
         *
         * @returns The EntryItem object or NULL if no suitable entry could be found.
         */
        EntryItem *findGroup (const QString & name);

        /**
         * Returns the entry at index row.
         *
         * @param row - The index of the entry that should be returned.
         *
         * @returns The child at index row.
         */
        EntryItem *child(int row);

        /**
         * Returns the parent item.
         *
         * @returns the parent item.
         */
        EntryItem *parent();

        /**
         * Returns the number of childs in this group.
         *
         * @returns The number of childs.
         */
        int childCount() const;

        /**
         * Returns the data of this entry at column sol.
         *
         * @returns the data of this entry at column sol.
         */
        QVariant data(int col) const;

        /**
         * Returns the index of this entry inside m_parent.
         *
         * @returns the index of this entry inside m_parent.
         */
        int row() const;

        /**
         * Renames the current item.
         *
         * @param name - The new name for this entry.
         */
        void rename(const QString & name);

        /**
         * Returns true, if this entry is a group.
         *
         * @returns true, if this entry is a group.
         */
        bool isGroup();

        /**
         * Clears the password history.
         */
        void clearPwdHistory();

        /**
         * Adds a new password history entry.
         *
         * @param timestamp - The creation time of the password.
         * @param pwd       - The password to be added.
         */
        void addPwdHistoryEntry(const QDateTime & timestamp,const QString & pwd);

        /**
         * Returns the password history.
         *
         * @returns the password history.
         */
        const QList<HistoryEntry> & getPwdHistory();

        /**
         * Sets m_save_pwd_history.
         */
        void setSavePwdHistory(int use_pwd_history);

        /**
         * Returns m_save_pwd_history.
         */
        int getSavePwdHistory();

        /**
         * Sets m_max_hist_size.
         */
        void setMaxHistSize(int use_pwd_history);

        /**
         * Returns m_max_hist_size.
         */
        int getMaxHistSize();

        /**
         * Returns the number of child entries, excluding groups.
         */
        int numEntries();

        /**
         * Returns the number of groups, excluding password entries.
         */
        int numGroups();

        /**
         * Returns a QMimeData object suitable for Drag and Drop actions.
         */
        QMimeData * toMimeData();

        /**
         * Returns a "application/pws.dragdrop.entry" Ascii representation of this entry and all sub-entries.
         */
        QByteArray toAscii(QString group, bool firstrun);

        /**
         * Convenience method that calls toAscii(m_name);
         */
        QByteArray toAscii();

        /**
         * Parses a "application/pws.dragdrop.entry" Ascii String and returns
         * all decoded entries.
         */
        static QList<EntryItem*> fromAscii(QString data);

        /**
         * Parses a "application/pws.dragdrop.entry" QMimeData object and returns
         * all decoded entries.
         */
        static QList<EntryItem*> fromMimeData (const QMimeData *mimeData);

    private:

        QUuid     m_uuid;           /* The uuid                   */
        QString   m_name;           /* The title                  */
        QString   m_user;           /* The username               */
        QString   m_notes;          /* The first line of notes    */
        QString   m_notes_long;     /* The complete notes         */
        QString  *m_pwd;            /* The password               */
        QUrl      m_url;            /* The url for this entry     */
        QString   m_autotype;       /* The autotype               */
        QDateTime m_ctime;          /* Creation time.             */
        QDateTime m_pwd_mtime;      /* Password modification time */
        QDateTime m_mtime;          /* Modification time          */
        QDateTime m_atime;          /* Access time                */
        QDateTime m_ltime;          /* Password lifetime          */
        bool      m_is_group;       /* set to true, if this entry represents a group */
        bool      m_expanded;       /* Whether this group is currently expanded or not */
        int       m_save_pwd_history; /* whether to save the pwd history */
        int       m_max_hist_size;    /* maximum number of password history entries */
        char     *m_unknown;          /* unknown fields */
        unsigned int m_unknown_len;   /* unknown fields length*/
        QList<HistoryEntry> m_pwd_history; /* password history */

        /**
         * All Child entries.
         */
        QList<EntryItem*> m_childs;

        /**
         * The parent object.
         */
        EntryItem        *m_parent;
};

#endif

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
