/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "pwstreemodel.h"

/**
 * Creates a new PWSTreeModel object.
 *
 * @param parent - The parent object.
 */
PWSTreeModel::PWSTreeModel(QObject *parent) : QAbstractItemModel(parent)
{
    rootItem = new EntryItem("");
}

/**
 * Delets the EntryItem tree.
 */
PWSTreeModel::~PWSTreeModel()
{
    delete rootItem;
}

/**
 * Overloaded: Returns the actual data for a specific EntryItem object.
 */
QVariant PWSTreeModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        { return QVariant(); }

    static const QIcon entry_icon(":/images/entry.png");
    static const QIcon group_icon(":/images/folder.png");
    static const QIcon group_open_icon(":/images/folderopen.png");

    EntryItem *item = static_cast<EntryItem*>(index.internalPointer());
    if (role == Qt::DisplayRole)
    {
        return item->data(index.column()); 
    }
    else if (role == Qt::DecorationRole && index.column() == 0)
    {
        if (item->isGroup() && item->isExpanded())
            return group_open_icon;
        else if (item->isGroup())
            return group_icon;
        else
            return entry_icon;
    }
    else if (role == Qt::ToolTipRole)
    {
        return item->data(index.column()+100); 
    }

    return QVariant();
}

/**
 * Overloaded: Returns the header data.
 */
QVariant PWSTreeModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole || orientation != Qt::Horizontal)
        { return QVariant(); }

    if (section == 0)
        { return ("Name"); }
    else if (section == 1)
        { return ("User"); }
    else if (section == 2)
        { return ("Notes"); }
    else if (section == 3)
        { return ("Last Modified"); }
    else if (section == 4)
        { return ("Accessed on"); }
    else if (section == 5)
        { return ("Created on"); }

    return QVariant();
}

/**
 * Overloaded: Returns a EntryItem object.
 */
QModelIndex PWSTreeModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        { return QModelIndex(); }

    EntryItem *parentItem;
    if (!parent.isValid())
        { parentItem = rootItem; }
    else
        { parentItem = static_cast<EntryItem*>(parent.internalPointer()); }

    EntryItem *childItem = parentItem->child(row);
    if (childItem)
        { return createIndex(row, column, childItem); }
    else
        { return QModelIndex(); }
}

/**
 * Overloaded: Returns the parent EntryItem object.
 */
QModelIndex PWSTreeModel::parent(const QModelIndex &index) const
{
    if (!index.isValid())
        { return QModelIndex(); }

    EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
    EntryItem *parentItem = childItem->parent();
    if (parentItem == rootItem)
        { return QModelIndex(); }

    return createIndex(parentItem->row(), 0, parentItem);
}

/**
 * Overloaded: Returns the column count for a specific EntryItem object.
 */
int PWSTreeModel::columnCount(const QModelIndex &parent) const
{
    parent.parent(); // just to avoid the gcc warning
    return 6;
}

/**
 * Overloaded: Returns the rowcount for the parent EntryItem object.
 */
int PWSTreeModel::rowCount(const QModelIndex &parent) const
{
    EntryItem *parentItem;
    if (parent.column() > 0)
        { return 0; }

    if (!parent.isValid())
        { parentItem = rootItem; }
    else
        { parentItem = static_cast<EntryItem*>(parent.internalPointer()); }

    return parentItem->childCount();
}

/**
 * Overloaded: Returns the flags for a specific EntryItem object.
 */
Qt::ItemFlags PWSTreeModel::flags ( const QModelIndex & index ) const
{
    if (!index.isValid())
        { return Qt::ItemIsDropEnabled; }

    if (index.column() == 0)
    {
        return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
    }

    return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
};

/**
 * Overloaded: Called by the delegator to set the new value inside the model.
 */
bool PWSTreeModel::setData (const QModelIndex & index, const QVariant & value, int role)
{
    if (role == Qt::EditRole)
    {
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (value != childItem->getName())
        {
            childItem->rename (value.toString());
            sort(childItem);

            emit dataChanged(index, index);
        }
        return true;
    }
    
    return false;
}

/**
 * Clears the whole model.
 */
void PWSTreeModel::clear()
{
    rootItem->clear();
    reset();
    emit layoutChanged ();
}

/**
 * Sorts the model.
 */
void PWSTreeModel::sort(EntryItem * item)
{
    emit layoutAboutToBeChanged ();

    if (item)
    {
        item->parent()->sort();
    }
    else
    {
        rootItem->sort();
    }

    emit layoutChanged ();
    emit sorted(item);
}

/**
 * Creates a new group with empty name.
 */
QModelIndex PWSTreeModel::createGroup (const QModelIndex & index)
{
    QModelIndex newItem;

    if (!index.isValid())
    {
        EntryItem * newGroup = new EntryItem ("NA"); 
        rootItem->appendEntry (newGroup);
        sort(newGroup);
        newItem = this->index (newGroup->row(), 0);
    }
    else
    {
        EntryItem *childItem  = static_cast<EntryItem*>(index.internalPointer());
        if (childItem->isGroup())
        { 
            EntryItem * newGroup = new EntryItem ("NA"); 
            childItem->appendEntry (newGroup);
            sort(newGroup);
            newItem = index.child (newGroup->row(), 0);
        }
        else
        { 
            EntryItem * newGroup = new EntryItem ("NA"); 
            childItem->parent()->appendEntry (newGroup);
            sort(newGroup);
            newItem = index.parent().child (newGroup->row(), 0);
        }
    }
    return newItem;
}

/**
 * Saves item and updates the tree view.
 *
 * @param item - The item to be added.
 */
void PWSTreeModel::addEntry (EntryItem * item, const QModelIndex & parent)
{
    EntryItem *parent_item = static_cast<EntryItem*>(parent.internalPointer());
    if (!parent.isValid())
        { parent_item = rootItem; }

    parent_item->appendEntry (item);


    if (parent_item == rootItem)
        { sort(NULL); }
    else
        { sort (parent_item); }

    beginInsertRows (parent, item->row(), item->row());
    endInsertRows ();
}

/**
 * Delets item and updates the tree view.
 *
 * @param item - The item to be added.
 */
void PWSTreeModel::delEntry (const QModelIndex & index)
{
    if (!index.isValid())
        { return; }

    EntryItem *item = static_cast<EntryItem*>(index.internalPointer());
    if (item->parent())
    { 
        beginRemoveRows (index.parent(), index.row(), index.row());
        item->parent()->removeEntry (item);
        endRemoveRows();
        sort(NULL);
    }
}

/**
 * Returns the root item.
 */
EntryItem *PWSTreeModel::root()
{
    return rootItem;
}

/**
 * Set the item at index in expanded state.
 */
void PWSTreeModel::expanded(const QModelIndex & index)
{
    EntryItem *item = static_cast<EntryItem*>(index.internalPointer());
    item->setExpanded (true);
    emit layoutChanged();
}

/**
 * Set the item at index in collapsed state.
 */
void PWSTreeModel::collapsed(const QModelIndex & index)
{
    EntryItem *item = static_cast<EntryItem*>(index.internalPointer());
    item->setExpanded (false);
    emit layoutChanged();
}

/**
 * Returns the number of entries, excluding groups.
 */
int PWSTreeModel::numEntries()
{
    return rootItem->numEntries();
}

/**
 * Returns the number of groups, excluding password entries.
 */
int PWSTreeModel::numGroups()
{
    return rootItem->numGroups();
}

/**
 * Remove one rows from the model.
 */
bool PWSTreeModel::removeRow (int row, const QModelIndex & parent)
{
    removeRows (row, 1, parent);
    return (true);
}

/**
 * Remove multiple rows from the model.
 */
bool PWSTreeModel::removeRows (int row, int count, const QModelIndex & parent)
{
    beginRemoveRows (parent, row, row+(count-1));
    for (int i = 0; i < count; i++)
    {
        EntryItem * parent_item = rootItem;
        if (parent.isValid())
            { parent_item = static_cast<EntryItem*>(parent.internalPointer()); }

        parent_item->removeEntry (parent_item->child (row+i));
    }
    endRemoveRows ();

    return true;
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
