/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWSTREEMODEL_H
#define PWSTREEMODEL_H

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QVariant>
#include <QtGui>
#include <QPushButton>
#include <QMessageBox>
#include "entryitem.h"
#include "entryitemdelegate.h"
#include "editentrydialog.h"
#include "libpws/pws2.h"
#include "libpws/pws3.h"

class PWSTreeModel : public QAbstractItemModel
{
    Q_OBJECT

    public:
        /**
         * Passwordsafe database format.
         */
        enum PWSFormat { PWS_V2, PWS_V3 };

        /**
         * Create a new PWSTreeModel object.
         *
         * @param parent - The parent object.
         */
        PWSTreeModel(QObject *parent = 0);

        /**
         * Delets the EntryItem tree.
         */
        ~PWSTreeModel();

        /**
         * Returns the actual data for a specific EntryItem object.
         */
        QVariant data(const QModelIndex &index, int role) const;

        /**
         * Returns the flags for a specific EntryItem object.
         */
        Qt::ItemFlags flags(const QModelIndex &index) const;

        /**
         * Returns the header data.
         */
        QVariant headerData(int section, Qt::Orientation orientation,
                            int role = Qt::DisplayRole) const;

        /**
         * Returns a EntryItem object.
         */
        QModelIndex index(int row, int column,
                          const QModelIndex &parent = QModelIndex()) const;

        /**
         * Returns the parent EntryItem object.
         */
        QModelIndex parent(const QModelIndex &index) const;

        /**
         * Returns the rowcount for the parent EntryItem object.
         */
        int rowCount(const QModelIndex &parent = QModelIndex()) const;

        /**
         * Returns the column count for a specific EntryItem object.
         */
        int columnCount(const QModelIndex &parent = QModelIndex()) const;

        /**
         * Called by the delegator to set the new value inside the model.
         */
        bool setData (const QModelIndex & index, const QVariant & value, int role  = Qt::EditRole);

        /**
         * Delets item and updates the tree view.
         *
         * @param item - The item to be added.
         */
        void delEntry (const QModelIndex & index);

        /**
         * Creates a new group with empty name.
         */
        QModelIndex createGroup (const QModelIndex & index);

        /**
         * Returns the root item.
         */
        EntryItem *root ();

        /**
         * Returns the number of entries, excluding groups.
         */
        int numEntries();

        /**
         * Returns the number of groups, excluding password entries.
         */
        int numGroups();

    signals:
        /**
         * Will be emitted when the subtree below item was sorted.
         */
        void sorted(EntryItem *item);

    public slots:
        /**
         * Saves item and updates the tree view.
         *
         * @param item - The item to be added.
         */
        void addEntry (EntryItem * item, const QModelIndex & parent);

        /**
         * Sorts the model.
         */
        void sort(EntryItem * item);

        /**
         * Clears the whole model.
         */
        void clear();

        /**
         * Set the item at index in expanded state.
         */
        void expanded(const QModelIndex & index);

        /**
         * Set the item at index in collapsed state.
         */
        void collapsed(const QModelIndex & index);

        /**
         * Remove one row from the model.
         */
        bool removeRow ( int row, const QModelIndex & parent = QModelIndex() );

        /**
         * Remove multiple rows from the model.
         */
        bool removeRows (int row, int count, const QModelIndex & parent = QModelIndex());

    private:
        /**
         * The root EntryItem object.
         */
        EntryItem *rootItem;
};

#endif

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
