/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "pwshandler.h"

/**
 * Creates a new PWSHandler object.
 *
 * @param parent - The parent object.
 */
PWSHandler::PWSHandler(PWSTreeModel * model) 
{ 
    m_model = model;

    pws_prefs_unknown     = NULL;
    pws_prefs_unknown_len = 0;
    pws_hdr_unknown       = NULL;;
    pws_hdr_unknown_len   = 0;

    ctx                   = NULL;
    ctx_backup            = NULL;
}

/**
 * Free unneeded ressources.
 */
PWSHandler::~PWSHandler()
{
    if (ctx)
    {
        pws_release_context (ctx);
        free (ctx);
    }

    if (ctx_backup)
    {
        pws_release_context (ctx_backup);
        free (ctx_backup);
    }

    if (pws_prefs_unknown)
    {
        free (pws_prefs_unknown);
    }

    if (pws_hdr_unknown)
    {
        free (pws_hdr_unknown);
    }
}

/**
 * Save the current database in PWSv2 format.
 */
bool PWSHandler::savePwsV2 (const QString & filename)
{
    pws_data_t dat;
    pws_init_data (&dat);

    FILE *out = fopen (filename.toAscii().data(), "wb+");
    if (!out)
        { QMessageBox::critical(0, tr("Error"), strerror(errno)); return false; }

    prepSave (&dat, PWSHandler::PWS_V2);
    buildPwsArray (m_model->root(), &dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws2_encrypt (&dat, ctx, out))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRITE_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
        }

        pws_release_data (&dat);
        fclose (out);
        return false;
    }
    QApplication::restoreOverrideCursor();
    
    pws_release_data (&dat);
    fclose (out);
    return true;
}

/**
 * Save the current database in PWSv3 format.
 */
bool PWSHandler::savePwsV3 (const QString & filename)
{
    pws_data_t dat;
    pws_init_data (&dat);

    FILE *out = fopen (filename.toAscii().data(), "wb+");
    if (!out)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); return false; }

    prepSave (&dat, PWSHandler::PWS_V3);
    buildPwsArray (m_model->root(), &dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws3_encrypt (&dat, ctx, out))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRITE_ERROR:
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                QApplication::restoreOverrideCursor();
                break;
        }

        pws_release_data (&dat);
        fclose (out);
        return false;
    }
    QApplication::restoreOverrideCursor();
    
    pws_release_data (&dat);
    fclose (out);
    return true;
}

/**
 * Set the preferences for this database.
 *
 * @param dat    - The pws_data_t object where the preferences should be stored.
 * @param format - The pws format for which dat should be prepared.
 * @param createUuid - Whether or not a new Uuid should be generated. If set to yes, only
 *                     a temporarily Uuid will be created which will not be saved inside
 *                     the ConfigStore object.
 */
void PWSHandler::prepSave (pws_data_t *dat, PWSHandler::PWSFormat format, bool createUuid)
{
    // all header information (except the preferences) is only supported in pws v3 files
    if (format == PWSHandler::PWS_V3)
    {
        // Header
        QString uuid;
        if (ConfigStore::store["pws_hdr_uuid"] != "" && !createUuid)
        {
            uuid = ConfigStore::store["pws_hdr_uuid"]; 
        }
        else if (createUuid)
        { 
            uuid = QUuid::createUuid().toString(); 
        }
        else
        {
            uuid = QUuid::createUuid().toString(); 
            ConfigStore::store["pws_hdr_uuid"] = uuid;
        }

        uuid.replace ('-', "");
        uuid.replace ('{', "");
        uuid.replace ('}', "");

        unsigned char char_uuid[16];
        for (int z = 0; z < 16; z++)
        {
            QString hex = uuid.left(2);
            uuid = uuid.remove(0, 2);
            char_uuid[z] = strtol (hex.toAscii().data(), 0, 16);
        }
        
#ifdef MINGW32
        DWORD len = 255;
        WCHAR name [255];

        GetUserName (name, &len);
        ConfigStore::store["pws_hdr_last_uid"] = QString::fromWCharArray (name);
#else
        ConfigStore::store["pws_hdr_last_uid"] = getpwuid (getuid())->pw_name;
#endif

        ConfigStore::store["pws_hdr_last_host"] = QHostInfo::localHostName();
        ConfigStore::store["pws_hdr_last_modified"] = QString::number (QDateTime::currentDateTime().toTime_t());
        ConfigStore::store["pws_hdr_last_app"] = QString("PWS v") + QString(PWS_VERSION);
        pws_set_header (dat, char_uuid, 16, ConfigStore::store["pws_hdr_last_app"].toUtf8().data(), ConfigStore::store["pws_hdr_last_app"].toUtf8().size(),
                          ConfigStore::store["pws_hdr_last_uid"].toUtf8().data(), ConfigStore::store["pws_hdr_last_uid"].toUtf8().size(),
                          ConfigStore::store["pws_hdr_last_host"].toUtf8().data(), ConfigStore::store["pws_hdr_last_host"].toUtf8().size(),
                          ConfigStore::store["pws_hdr_db_name"].toUtf8().data(), ConfigStore::store["pws_hdr_db_name"].toUtf8().size(),
                          ConfigStore::store["pws_hdr_db_desc"].toUtf8().data(), ConfigStore::store["pws_hdr_db_desc"].toUtf8().size(),
                          pws_hdr_unknown, pws_hdr_unknown_len,
                          ConfigStore::store["pws_hdr_last_modified"].toULong());

        // store header information inside configuration store
        if (dat->hdr3.version)   
        { 
            QString v1 = QString::number (dat->hdr3.version[1]);
            QString v2 = QString::number (dat->hdr3.version[0]);
            ConfigStore::store["pws_hdr_version"] = v1 + "." + v2;
        }
    }
    else
    {
        ConfigStore::store["pws_hdr_version"] = "2.0";
    }
    
    // Prefs
    pws_set_prefs (dat, ConfigStore::store["prefs_bool_showpwdefault"].toUInt(), ConfigStore::store["prefs_bool_showpasswordintree"].toUInt(),
                    ConfigStore::store["prefs_bool_sortascending"].toUInt(), ConfigStore::store["prefs_bool_usedefuser"].toUInt(), 
                    ConfigStore::store["prefs_bool_saveimmediately"].toUInt(), ConfigStore::store["prefs_bool_pwuselowercase"].toUInt(),
                    ConfigStore::store["prefs_bool_pwuseuppercase"].toUInt(), ConfigStore::store["prefs_bool_pwusedigits"].toUInt(),
                    ConfigStore::store["prefs_bool_pwusesysmbols"].toUInt(), ConfigStore::store["prefs_bool_pwusehexdigit"].toUInt(),
                    ConfigStore::store["prefs_bool_pweasyvision"].toUInt(), 1,
                    ConfigStore::store["prefs_bool_displayexpandedaddeditdialog"].toUInt(), ConfigStore::store["prefs_bool_maintaindatetimestamps"].toUInt(),
                    ConfigStore::store["prefs_bool_savepasswordhistory"].toUInt(), ConfigStore::store["prefs_bool_shownotesdefault"].toUInt(),
                    ConfigStore::store["prefs_bool_showusernameintree"].toUInt(), ConfigStore::store["prefs_int_pwlendefault"].toUInt(),
                    ConfigStore::store["prefs_int_idletimeout"].toUInt(), ConfigStore::store["prefs_int_treedisplaystatusatopen"].toUInt(),
                    ConfigStore::store["prefs_int_numpwhistorydefault"].toUInt(), 
                    pws_prefs_unknown, pws_prefs_unknown_len,
                    ConfigStore::store["prefs_str_defaultusername"].toUtf8().data(), ConfigStore::store["prefs_str_defaultusername"].toUtf8().size(), 
                    ConfigStore::store["prefs_str_defaultautootypestring"].toUtf8().data(), ConfigStore::store["prefs_str_defaultautootypestring"].toUtf8().size());
}

/**
 * Imports an existing database at file below index.
 *
 * @param file  - The database to import.
 * @param index - The index beow which file should be imported.
 */
bool PWSHandler::import (const QString & filename, QModelIndex & index)
{
    FILE *in = fopen (filename.toAscii().data(), "rb");
    if (!in)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); return false; }

    char buf[4];
    if (fread (buf, 1, 4, in) != 4)
    { 
        QMessageBox::critical(NULL, tr("Error"), ConfigStore::store["curfile"] + tr(" is not a valid PWS file."));
        return false;
    }
    fclose (in);

    if (memcmp (buf, "PWS3", 4) == 0)
        { return importPwsV3 (filename, index); }
    else
        { return importPwsV2 (filename, index); }
}

/**
 * Imports an existing PWSv3 database at file below index.
 *
 * @param file  - The database to import.
 * @param index - The index beow which file should be imported.
 */
bool PWSHandler::importPwsV3 (const QString & file, const QModelIndex & index)
{
    FILE *in = fopen (file.toAscii().data(), "rb");
    if (!in)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); return false; }

    pws_data_t dat;
    pws_init_data (&dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws3_decrypt (&dat, ctx, in))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRONG_FORMAT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), file + tr(" is not a valid PWS file."));
                break;
            case PWFILE_WRONG_KEY:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), tr("Wrong passphrase."));
                emit wrongPassphrase();
                break;
            case PWFILE_CORRUPT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), file + tr(" is either corrupt or truncated."));
                break;
            case PWFILE_WRITE_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
            case PWFILE_GCRYPT_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
        }

        pws_release_data (&dat);
        fclose (in);
        return false;
    }
    QApplication::restoreOverrideCursor();
        
    EntryItem *parent = m_model->root();;
    if (index.isValid())
    {
        parent = static_cast<EntryItem*>(index.internalPointer()); 

        if (!parent->isGroup())
            { parent = parent->parent(); }
    }
    buildEntryTree (parent, &dat);
    m_model->sort(NULL);

    pws_release_data (&dat);
    fclose (in);

    return true;
}

/**
 * Imports an existing PWSv2 database at file below index.
 *
 * @param file  - The database to import.
 * @param index - The index beow which file should be imported.
 */
bool PWSHandler::importPwsV2 (const QString & file, const QModelIndex & index)
{
    FILE *in = fopen (file.toAscii().data(), "rb");
    if (!in)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); return false; }

    pws_data_t dat;
    pws_init_data (&dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws2_decrypt (&dat, ctx, in))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRONG_FORMAT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), file + tr(" is not a valid PWS file."));
                break;
            case PWFILE_WRONG_KEY:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), tr("Wrong passphrase (or not a valid database file)."));
                emit wrongPassphrase();
                break;
            case PWFILE_CORRUPT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), file + tr(" is either corrupt or truncated."));
                break;
            case PWFILE_WRITE_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
            case PWFILE_GCRYPT_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
        }

        pws_release_data (&dat);
        fclose (in);
        return false;
    }
    QApplication::restoreOverrideCursor();

    EntryItem *parent = m_model->root();;
    if (index.isValid())
    {
        parent = static_cast<EntryItem*>(index.internalPointer()); 

        if (!parent->isGroup())
            { parent = parent->parent(); }
    }
    buildEntryTree (parent, &dat);
    m_model->sort(NULL);

    pws_release_data (&dat);
    fclose (in);

    return true;
}

/**
 * Sets up a new database tree from the database filename.
 */
bool PWSHandler::open (const QString & filename)
{
    FILE *in = fopen (filename.toAscii().data(), "rb");
    if (!in)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); return false; }

    char buf[4];
    if (fread (buf, 1, 4, in) != 4)
    { 
        QMessageBox::critical(NULL, tr("Error"), ConfigStore::store["curfile"] + tr(" is not a valid PWS file."));
        fclose (in);
        return false;
    }
    fclose (in);

    if (memcmp (buf, "PWS3", 4) == 0)
        { return openPwsV3 (filename); }
    else
        { return openPwsV2 (filename); }
}

/**
 * Sets up a new database tree from the PWSv3 database filename.
 */
bool PWSHandler::openPwsV3 (const QString & filename)
{
    FILE *in = fopen (filename.toAscii().data(), "rb");
    if (!in)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); return false; }

    pws_data_t dat;
    pws_init_data (&dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws3_decrypt (&dat, ctx, in))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRONG_FORMAT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), ConfigStore::store["curfile"] + tr(" is not a valid PWS file."));
                break;
            case PWFILE_WRONG_KEY:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), tr("Wrong passphrase."));
                emit wrongPassphrase();
                break;
            case PWFILE_CORRUPT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), ConfigStore::store["curfile"] + tr(" is either corrupt or truncated."));
                break;
            case PWFILE_WRITE_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error(ctx));
                break;
            case PWFILE_GCRYPT_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error(ctx));
                break;
        }

        pws_release_data (&dat);

        fclose (in);
        return false;
    }
    QApplication::restoreOverrideCursor();

    ConfigStore::defaults(ConfigStore::PWS); 
    m_model->clear();

    setupPwsPrefs (&dat, PWSHandler::PWS_V3);

    m_model->clear();
    buildEntryTree (m_model->root(), &dat);
    m_model->sort(NULL);

    pws_release_data (&dat);
    fclose (in);

    return true;
}

/**
 * Sets up a new database tree from the PWSv2 database filename.
 */
bool PWSHandler::openPwsV2 (const QString & filename)
{
    FILE *in = fopen (filename.toAscii().data(), "rb");
    if (!in)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); return false; }

    pws_data_t dat;
    pws_init_data (&dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws2_decrypt (&dat, ctx, in))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRONG_FORMAT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), ConfigStore::store["curfile"] + tr(" is not a valid PWS file."));
                break;
            case PWFILE_WRONG_KEY:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), tr("Wrong passphrase (or not a valid database file)."));
                break;
            case PWFILE_CORRUPT:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), ConfigStore::store["curfile"] + tr(" is either corrupt or truncated."));
                break;
            case PWFILE_WRITE_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
            case PWFILE_GCRYPT_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
        }
        
        pws_release_data (&dat);
        
        fclose (in);
        return false;
    }
    QApplication::restoreOverrideCursor();

    ConfigStore::defaults(ConfigStore::PWS); 
    m_model->clear();

    ConfigStore::store["pws_hdr_version"] = "2.0";

    setupPwsPrefs(&dat, PWSHandler::PWS_V2);

    m_model->clear();
    buildEntryTree (m_model->root(), &dat);
    m_model->sort(NULL);

    pws_release_data (&dat);
    fclose (in);

    return true;
}

/**
 * Set the preferences stored inside the pws_data_t object.
 *
 * @param dat    - The pws_data_t object containing the preferences.
 * @param format - The pws format for which the preferences should be set. 
 */
void PWSHandler::setupPwsPrefs (pws_data_t *dat, PWSHandler::PWSFormat format)
{
    ConfigStore::store["prefs_bool_showpwdefault"]                = QString::number (dat->prefs.showpwdefault);
    ConfigStore::store["prefs_bool_showpasswordintree"]           = QString::number (dat->prefs.showpasswordintree);
    ConfigStore::store["prefs_bool_sortascending"]                = QString::number (dat->prefs.sortascending);
    ConfigStore::store["prefs_bool_usedefuser"]                   = QString::number (dat->prefs.usedefuser);
    ConfigStore::store["prefs_bool_saveimmediately"]              = QString::number (dat->prefs.saveimmediately);
    ConfigStore::store["prefs_bool_pwuselowercase"]               = QString::number (dat->prefs.pwuselowercase);
    ConfigStore::store["prefs_bool_pwuseuppercase"]               = QString::number (dat->prefs.pwuseuppercase);
    ConfigStore::store["prefs_bool_pwusedigits"]                  = QString::number (dat->prefs.pwusedigits);
    ConfigStore::store["prefs_bool_pwusesysmbols"]                = QString::number (dat->prefs.pwusesysmbols);
    ConfigStore::store["prefs_bool_pwusehexdigit"]                = QString::number (dat->prefs.pwusehexdigit);
    ConfigStore::store["prefs_bool_pweasyvision"]                 = QString::number (dat->prefs.pweasyvision);
    ConfigStore::store["prefs_bool_displayexpandedaddeditdialog"] = QString::number (dat->prefs.displayexpandedaddeditdialog);
    ConfigStore::store["prefs_bool_maintaindatetimestamps"]       = QString::number (dat->prefs.maintaindatetimestamps);
    ConfigStore::store["prefs_bool_savepasswordhistory"]          = QString::number (dat->prefs.savepasswordhistory);
    ConfigStore::store["prefs_bool_shownotesdefault"]             = QString::number (dat->prefs.shownotesdefault);
    ConfigStore::store["prefs_bool_showusernameintree"]           = QString::number (dat->prefs.showusernameintree);

    ConfigStore::store["prefs_int_pwlendefault"]            = QString::number(dat->prefs.pwlendefault);
    ConfigStore::store["prefs_int_treedisplaystatusatopen"] = QString::number(dat->prefs.treedisplaystatusatopen);
    ConfigStore::store["prefs_int_numpwhistorydefault"]     = QString::number(dat->prefs.numpwhistorydefault);

    ConfigStore::store["prefs_str_defaultusername"] 
      = QString::fromUtf8 (dat->prefs.defaultusername, dat->prefs.defaultusername_len);
    ConfigStore::store["prefs_str_defaultautootypestring"] 
      = QString::fromUtf8 (dat->prefs.defaultautootypestring, dat->prefs.defaultautootypestring_len);

    if (dat->prefs.idletimeout > 0)
        { ConfigStore::store["prefs_int_idletimeout"] = QString::number(dat->prefs.idletimeout); }
    else
        { ConfigStore::store["prefs_int_idletimeout"] = "5"; }

    if (dat->prefs.unknown)
    {
        free (pws_prefs_unknown);
        pws_prefs_unknown = (char*)malloc (dat->prefs.unknown_len*sizeof(char));
        memcpy (pws_prefs_unknown, dat->prefs.unknown, dat->prefs.unknown_len);
        pws_prefs_unknown_len = dat->prefs.unknown_len;
    }

    // header information - only applicable for pws v3 files
    if (format == PWSHandler::PWS_V3)
    {
        QUuid uuid;
        if (dat->hdr3.uuid_len)
        { 
            uint l;
            ushort w1, w2;
            uchar b1, b2, b3, b4, b5, b6, b7, b8;

            memcpy (&l,  dat->hdr3.uuid,    4);
            memcpy (&w1, dat->hdr3.uuid+4,  2);
            memcpy (&w2, dat->hdr3.uuid+6,  2);
            memcpy (&b1, dat->hdr3.uuid+8,  1);
            memcpy (&b2, dat->hdr3.uuid+9,  1);
            memcpy (&b3, dat->hdr3.uuid+10, 1);
            memcpy (&b4, dat->hdr3.uuid+11, 1);
            memcpy (&b5, dat->hdr3.uuid+12, 1);
            memcpy (&b6, dat->hdr3.uuid+13, 1);
            memcpy (&b7, dat->hdr3.uuid+14, 1);
            memcpy (&b8, dat->hdr3.uuid+15, 1);

            ConfigStore::store["pws_hdr_uuid"] = QUuid (l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8).toString(); 
        }

        if (dat->hdr3.version)   
        { 
            QString v1 = QString::number (dat->hdr3.version[1]);
            QString v2 = QString::number (dat->hdr3.version[0]);
            ConfigStore::store["pws_hdr_version"] = v1 + "." + v2;
        }

        if (dat->hdr3.last_app)  { ConfigStore::store["pws_hdr_last_app"]  = QString::fromUtf8 (dat->hdr3.last_app, dat->hdr3.last_app_len);   }
        if (dat->hdr3.last_uid)  { ConfigStore::store["pws_hdr_last_uid"]  = QString::fromUtf8 (dat->hdr3.last_uid, dat->hdr3.last_uid_len);   }
        if (dat->hdr3.last_host) { ConfigStore::store["pws_hdr_last_host"] = QString::fromUtf8 (dat->hdr3.last_host, dat->hdr3.last_host_len); }
        if (dat->hdr3.db_name)   { ConfigStore::store["pws_hdr_db_name"]   = QString::fromUtf8 (dat->hdr3.db_name, dat->hdr3.db_name_len);     }
        if (dat->hdr3.db_desc)   { ConfigStore::store["pws_hdr_db_desc"]   = QString::fromUtf8 (dat->hdr3.db_desc, dat->hdr3.db_desc_len);     }
        if (dat->hdr3.last_modified) { ConfigStore::store["pws_hdr_last_modified"] = QString::number (dat->hdr3.last_modified); }

        if (dat->hdr3.unknown)
        {
            free (pws_hdr_unknown);
            pws_hdr_unknown = (char*)malloc (dat->hdr3.unknown_len*sizeof(char));
            memcpy (pws_hdr_unknown, dat->hdr3.unknown, dat->hdr3.unknown_len);
            pws_hdr_unknown_len = dat->hdr3.unknown_len;
        }

    }
}

/**
 * Export the subtree at index and write a new PWSv2 database to filename.
 *
 * @param filename - The database file to be written.
 * @param index    - The subtree to be written.
 */
void PWSHandler::exportPwsV2 (const QString & filename, QModelIndex & index)
{
    if (!index.isValid()) { return; }

    EntryItem *item = static_cast<EntryItem*>(index.internalPointer()); 

    pws_data_t dat;
    pws_init_data (&dat);

    FILE *out = fopen (filename.toAscii().data(), "wb+");
    if (!out)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); }

    prepSave (&dat, PWSHandler::PWS_V2);
    buildPwsArray (item, &dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws2_encrypt (&dat, ctx, out))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRITE_ERROR:
                QApplication::restoreOverrideCursor();
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                break;
        }

        pws_release_data (&dat);
        fclose (out);
    }
    QApplication::restoreOverrideCursor();
    
    pws_release_data (&dat);
    fclose (out);
}

/**
 * Export the subtree at index and write a new PWSv3 database to filename.
 *
 * @param filename - The database file to be written.
 * @param index    - The subtree to be written.
 */
void PWSHandler::exportPwsV3 (const QString & filename, QModelIndex & index)
{
    if (!index.isValid()) { return; }

    EntryItem *item = static_cast<EntryItem*>(index.internalPointer()); 

    pws_data_t dat;
    pws_init_data (&dat);

    FILE *out = fopen (filename.toAscii().data(), "wb+");
    if (!out)
        { QMessageBox::critical(NULL, tr("Error"), strerror(errno)); }

    prepSave (&dat, PWSHandler::PWS_V3, true);
    buildPwsArray (item, &dat);

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    if(!pws3_encrypt (&dat, ctx, out))
    {
        switch (pws_int_error (ctx))
        {
            case PWFILE_WRITE_ERROR:
                QMessageBox::critical(NULL, tr("Error"), pws_str_error (ctx));
                QApplication::restoreOverrideCursor();
                break;
        }

        pws_release_data (&dat);
        fclose (out);
    }
    QApplication::restoreOverrideCursor();
    
    pws_release_data (&dat);
    fclose (out);
}

/**
 * Sets a new passphrase for encryption.
 */
void PWSHandler::setNewPassphrase (const QString & p)
{
    if (ctx)
    {
        if (ctx_backup) 
        {
            pws_release_context (ctx_backup); 
            free (ctx_backup);
        }

        ctx_backup = ctx;
    }
    
    ctx = (pws_context_t*)malloc (sizeof (pws_context_t));
    
    pws_init_context (ctx, p.toAscii().data(), 1);
}

/**
 * Sets a new passphrase for encryption and decryption.
 */
void PWSHandler::setPassphrase (const QString & p)
{
    if (ctx)
    {
        if (ctx_backup) 
        {
            pws_release_context (ctx_backup); 
            free (ctx_backup);
        }

        ctx_backup = ctx;
    }
    
    ctx = (pws_context_t*)malloc (sizeof (pws_context_t));
    
    pws_init_context (ctx, p.toAscii().data(), 0);
}

/**
 * Restores the previous passphrase.
 */
void PWSHandler::restorePassphrase ()
{
    if (ctx_backup)
    {
        if (ctx) 
        { 
            pws_release_context (ctx); 
            free (ctx);
        }

        ctx = ctx_backup;
        ctx_backup = NULL;
    }
}

/**
 * Verifies the given passphrase.
 */
bool PWSHandler::verifyPassphrase (const QString & pwd)
{
    if (pws_verify_pwd (ctx, pwd.toAscii().data()))
    {
        return true;
    }

    return false;
}

/**
 * Save the whole subtree at root in dat.
 *
 * @param root - The (sub)tree to be saved.
 * @param dat  - The pws_data_t object that should be used for saving.
 */
void PWSHandler::buildPwsArray (EntryItem *root, pws_data_t *dat)
{
    buildPwsArray ("", root, dat);
}

/**
 * Save the whole subtree at root in dat (internal method).
 *
 * @param group - The group prefix for this group.
 * @param root  - The (sub)tree to be saved.
 * @param dat   - The pws_data_t object that should be used for saving.
 */
void PWSHandler::buildPwsArray (QString group, EntryItem *root, pws_data_t *dat)
{
    if (root->isGroup())
    {
        if (root->m_parent && group == "")
            { group = root->m_name; }
        else if (root->m_parent)
            { group += "." + root->m_name; }

        for (int i = 0; i < root->m_childs.size(); i++)
        {
            if (!root->m_childs.at(i)->isGroup())
            {
                addPwsEntry (group, root->m_childs.at(i), dat);
            }
            else
            {
                buildPwsArray (group, root->m_childs.at(i), dat);
            }
        }
    }
    else
    {
        addPwsEntry (group, root, dat);
    }
}

/**
 * Adds entry to the pws_data_t object dat.
 *
 * @param group - The group prefix for this group.
 * @param entry - The entry to be added.
 * @param dat   - The pws_data_t object that should be used for saving.
 */
void PWSHandler::addPwsEntry (QString group, EntryItem *entry, pws_data_t *dat)
{
    QString name     = "";   
    QString user     = ""; 
    QString notes    = "";
    QString autotype = "";
    QString *pwd     = NULL;
    QUrl url("");

    unsigned char char_uuid[16];
    QString uuid;

    QDateTime ctime = QDateTime::fromTime_t(0);
    QDateTime atime = QDateTime::fromTime_t(0);
    QDateTime mtime = QDateTime::fromTime_t(0);
    QDateTime ltime = QDateTime::fromTime_t(0);
    QDateTime pwd_mtime = QDateTime::fromTime_t(0);

    if (entry->m_name != "")        name     = entry->m_name;
    if (entry->m_user!= "")         user     = entry->m_user;
    if (entry->m_notes != "")       notes    = entry->m_notes_long;
    if (entry->m_autotype != "")    autotype = entry->m_autotype;
    if (entry->m_url.toString() != "") url   = entry->m_url;

    pwd = entry->m_pwd;

    notes = notes.replace ('\n', "\r\n");

    int save_pwd_history = entry->m_save_pwd_history;
    int max_hist_size    = entry->m_max_hist_size;

    uuid = entry->m_uuid.toString();
    uuid.replace ('-', "");
    uuid.replace ('{', "");
    uuid.replace ('}', "");
    for (int z = 0; z < 32; z+=2)
    {
        QString hex = uuid.left(2);
        uuid = uuid.remove(0, 2);
        char_uuid[z/2] = strtol (hex.toAscii().data(), 0, 16);
    }

    if(entry->m_ctime.toTime_t() > 0)     ctime     = entry->m_ctime;
    if(entry->m_atime.toTime_t() > 0)     atime     = entry->m_atime;
    if(entry->m_mtime.toTime_t() > 0)     mtime     = entry->m_mtime;
    if(entry->m_ltime.toTime_t() > 0)     ltime     = entry->m_ltime;
    if(entry->m_pwd_mtime.toTime_t() > 0) pwd_mtime = entry->m_pwd_mtime;

    pws_entry_t * pws_entry;
    pws_entry = pws_new_entry (char_uuid, 16, group.toUtf8().data(), group.toUtf8().size(), name.toUtf8().data(), name.toUtf8().size(),
                               user.toUtf8().data(), user.toUtf8().size(), notes.toUtf8().data(), notes.toUtf8().size(),
                               pwd->toUtf8().data(), pwd->toUtf8().size(), "", 0,
                               url.toString().toUtf8().data(), url.toString().toUtf8().size(), autotype.toUtf8().data(), autotype.toUtf8().size(),
                               save_pwd_history, max_hist_size, entry->m_unknown, entry->m_unknown_len,
                               ctime.toTime_t(), pwd_mtime.toTime_t(), mtime.toTime_t(), atime.toTime_t(), ltime.toTime_t());

    QListIterator<HistoryEntry> pwd_history(entry->m_pwd_history);
    while (pwd_history.hasNext()) 
    {
        HistoryEntry entry = pwd_history.next();

        pws_add_hst_entry (pws_entry, entry.password->toUtf8().data(), entry.password->toUtf8().size(), entry.timestamp);
    }

    pws_add_entry (dat, pws_entry);
}

/**
 * Convenient method to build the whole PWS tree at once.
 *
 * @param root - The root item, where all childs will be added.
 * @param dat  - the pws_data_t object which contains ths pws data.
 */
void PWSHandler::buildEntryTree (EntryItem *root, pws_data_t *dat)
{
    for (int idx = 0; idx < dat->num_entries; idx++)
    {
        EntryItem *pos = root;

        QString group    = "";
        QString title    = "";
        QString user     = "";
        QString pwd      = "";
        QString notes    = "";
        QString autotype = "";

        QUrl url;

        if (dat->entries[idx]->group_len       && !dat->entries[idx]->group[dat->entries[idx]->group_len-1])             { dat->entries[idx]->group_len-=1; }
        if (dat->entries[idx]->title_len       && !dat->entries[idx]->title[dat->entries[idx]->title_len-1])             { dat->entries[idx]->title_len-=1; }
        if (dat->entries[idx]->username_len    && !dat->entries[idx]->username[dat->entries[idx]->username_len-1])       { dat->entries[idx]->username_len-=1; }
        if (dat->entries[idx]->pwd_len         && !dat->entries[idx]->pwd[dat->entries[idx]->pwd_len-1])                 { dat->entries[idx]->pwd_len-=1; }
        if (dat->entries[idx]->notes_len       && !dat->entries[idx]->notes[dat->entries[idx]->notes_len-1])             { dat->entries[idx]->notes_len-=1; }
        if (dat->entries[idx]->autotype_len    && !dat->entries[idx]->autotype[dat->entries[idx]->autotype_len-1])       { dat->entries[idx]->autotype_len-=1; }
        if (dat->entries[idx]->url_len         && !dat->entries[idx]->url[dat->entries[idx]->url_len-1])                 { dat->entries[idx]->url_len-=1; }

        group    = QString::fromUtf8 (dat->entries[idx]->group,       dat->entries[idx]->group_len);
        title    = QString::fromUtf8 (dat->entries[idx]->title,       dat->entries[idx]->title_len);
        user     = QString::fromUtf8 (dat->entries[idx]->username,    dat->entries[idx]->username_len);
        pwd      = QString::fromUtf8 (dat->entries[idx]->pwd,         dat->entries[idx]->pwd_len);
        notes    = QString::fromUtf8 (dat->entries[idx]->notes,       dat->entries[idx]->notes_len);
        autotype = QString::fromUtf8 (dat->entries[idx]->autotype,    dat->entries[idx]->autotype_len);
        url      = QString::fromUtf8 (dat->entries[idx]->url,         dat->entries[idx]->url_len);

        notes = notes.replace ('\r', "");

        QUuid uuid;
        if (dat->entries[idx]->uuid_len)
        { 
            uint l;
            ushort w1, w2;
            uchar b1, b2, b3, b4, b5, b6, b7, b8;

            memcpy (&l,  dat->entries[idx]->uuid,    4);
            memcpy (&w1, dat->entries[idx]->uuid+4,  2);
            memcpy (&w2, dat->entries[idx]->uuid+6,  2);
            memcpy (&b1, dat->entries[idx]->uuid+8,  1);
            memcpy (&b2, dat->entries[idx]->uuid+9,  1);
            memcpy (&b3, dat->entries[idx]->uuid+10, 1);
            memcpy (&b4, dat->entries[idx]->uuid+11, 1);
            memcpy (&b5, dat->entries[idx]->uuid+12, 1);
            memcpy (&b6, dat->entries[idx]->uuid+13, 1);
            memcpy (&b7, dat->entries[idx]->uuid+14, 1);
            memcpy (&b8, dat->entries[idx]->uuid+15, 1);

            uuid = QUuid (l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8); 
        }

        QDateTime ctime     = QDateTime::fromTime_t (dat->entries[idx]->ctime);
        QDateTime atime     = QDateTime::fromTime_t (dat->entries[idx]->atime);
        QDateTime mtime     = QDateTime::fromTime_t (dat->entries[idx]->mtime);
        QDateTime ltime     = QDateTime::fromTime_t (dat->entries[idx]->ltime);
        QDateTime pwd_mtime = QDateTime::fromTime_t (dat->entries[idx]->pwd_mtime);

        EntryItem *tmp_pos = root;
        QStringList groups = group.split ('.', QString::SkipEmptyParts);
        for (int i = 0; i < groups.size(); i++)
        {
            tmp_pos = pos->findGroup (groups.at(i));
            if (!tmp_pos)
            {
                pos->appendEntry (new EntryItem (groups.at(i)));
                pos = pos->findGroup (groups.at(i));
            }
            else
            {
                pos = tmp_pos;
            }
        }

        int save_pwd_history = dat->entries[idx]->save_pwd_history;
        int max_hist_size    = dat->entries[idx]->def_pwd_max;

        EntryItem *item = new EntryItem (title, uuid, user, notes, pwd, url, autotype, save_pwd_history, max_hist_size,
                                        ctime, pwd_mtime, mtime, atime, ltime,
                                        dat->entries[idx]->unknown , dat->entries[idx]->unknown_len);

        for (int x = 0; x < dat->entries[idx]->pwd_history_len; x++)
        {
            if (dat->entries[idx]->pwd_history[x].pwd_len)
            {
                if (!dat->entries[idx]->pwd_history[x].pwd[dat->entries[idx]->pwd_history[x].pwd_len-1]) 
                { 
                    dat->entries[idx]->pwd_history[x].pwd_len-=1; 
                }

                QString hst_pwd = QString::fromUtf8 (dat->entries[idx]->pwd_history[x].pwd,  dat->entries[idx]->pwd_history[x].pwd_len);

                item->addPwdHistoryEntry (QDateTime::fromTime_t(dat->entries[idx]->pwd_history[x].timestamp), hst_pwd);
            }
        }

        pos->appendEntry (item);
    }
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
