/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWSHANDLER_H
#define PWSHANDLER_H

#include <QtGui>
#include <QMessageBox>
#include <QtNetwork>
#include "entryitem.h"
#include "pwstreemodel.h"
#include "libpws/pws2.h"
#include "libpws/pws3.h"

#ifndef MINGW32
#include <pwd.h>
#endif

class PWSHandler : public QObject
{
    Q_OBJECT
    
    public:
        /**
         * Passwordsafe database format.
         */
        enum PWSFormat { PWS_V2, PWS_V3 };

        /**
         * Create a new PWSHandler object.
         */
        PWSHandler(PWSTreeModel * model);

        /**
         * Free unneeded ressources.
         */
        ~PWSHandler();

        /**
         * Save the current database in PWSv2 format.
         */
        bool savePwsV2 (const QString & filename);

        /**
         * Save the current database in PWSv3 format.
         */
        bool savePwsV3 (const QString & filename);

        /**
         * Imports an existing PWSv3 database at file below index.
         *
         * @param file  - The database to import.
         * @param index - The index beow which file should be imported.
         */
        bool importPwsV2 (const QString & file, const QModelIndex & index);

        /**
         * Imports an existing PWSv3 database at file below index.
         *
         * @param file  - The database to import.
         * @param index - The index beow which file should be imported.
         */
        bool importPwsV3 (const QString & file, const QModelIndex & index);

        /**
         * Export the subtree at index and write a new PWSv2 database to filename.
         *
         * @param filename - The database file to be written.
         * @param index    - The subtree to be written.
         */
        void exportPwsV2 (const QString & filename, QModelIndex & index);

        /**
         * Export the subtree at index and write a new PWSv3 database to filename.
         *
         * @param filename - The database file to be written.
         * @param index    - The subtree to be written.
         */
        void exportPwsV3 (const QString & filename, QModelIndex & index);

        /**
         * Sets up a new database tree from the PWSv2 database filename.
         */
        bool openPwsV2 (const QString & filename);

        /**
         * Sets up a new database tree from the PWSv3 database filename.
         */
        bool openPwsV3 (const QString & filename);

        /**
         * Sets up a new database tree from the database filename.
         */
        bool open (const QString & filename);

        /**
         * Imports an existing database at file below index.
         *
         * @param file  - The database to import.
         * @param index - The index beow which file should be imported.
         */
        bool import (const QString & filename, QModelIndex & index);

        /**
         * Sets a new passphrase for encryption and decryption.
         */
        void setPassphrase (const QString & p);

        /**
         * Sets a new passphrase for encryption.
         */
        void setNewPassphrase (const QString & p);

        /**
         * Restores the previous passphrase.
         */
        void restorePassphrase ();

        /**
         * Verifies the given passphrase.
         */
        bool verifyPassphrase (const QString & pwd);

    signals:
        /**
         * This signal will be emitted whenever a wrong passphrase
         * was entered.
         */
        void wrongPassphrase();
    
    protected:
        /**
         * Save the whole subtree at root in dat.
         *
         * @param root - The (sub)tree to be saved.
         * @param dat  - The pws_data_t object that should be used for saving.
         */
        void buildPwsArray (EntryItem *root, pws_data_t *dat);

        /**
         * Convenient method to build the whole PWS tree at once.
         *
         * @param root - The root item, where all childs will be added.
         * @param dat  - the pws_data_t object which contains ths pws data.
         */
        void buildEntryTree (EntryItem *root, pws_data_t *dat);

        /**
         * Save the whole subtree at root in dat.
         *
         * @param group - The group prefix for this group.
         * @param root  - The (sub)tree to be saved.
         * @param dat   - The pws_data_t object that should be used for saving.
         */
        void buildPwsArray (QString group, EntryItem *root, pws_data_t *dat);

        /**
         * Adds entry to the pws_data_t object dat.
         *
         * @param group - The group prefix for this group.
         * @param entry - The entry to be added.
         * @param dat   - The pws_data_t object that should be used for saving.
         */
        void addPwsEntry (QString group, EntryItem *entry, pws_data_t *dat);

        /**
         * Set the preferences stored inside the pws_data_t object.
         *
         * @param dat - The pws_data_t object containing the preferences.
         * @param format - The pws format for which the preferences should be set. 
         */
        void setupPwsPrefs (pws_data_t *dat, PWSHandler::PWSFormat format);

        /**
         * Set the preferences for this database.
         *
         * @param dat    - The pws_data_t object where the preferences should be stored.
         * @param format - The pws format for which dat should be prepared.
         * @param createUuid - Whether or not a new Uuid should be generated. If set to yes, only
         *                     a temporarily Uuid will be created which will not be saved inside
         *                     the ConfigStore object.
         */
        void prepSave (pws_data_t *dat, PWSHandler::PWSFormat format, bool createUuid = false);

        /**
         * The item model.
         */
        PWSTreeModel * m_model;

        /**
         * The current pws_context_t context object.
         */
        pws_context_t *ctx;

        /**
         * The previous pws_context_t context object.
         */
        pws_context_t *ctx_backup;

        /**
         * Unknown prefs.
         */
        char *pws_prefs_unknown;

        /**
         * Length of ConfigStore::pws_prefs_unknown.
         */
        unsigned int pws_prefs_unknown_len;

        /**
         * Unknown header fields.
         */
        char *pws_hdr_unknown;

        /**
         * Length of ConfigStore::pws_hdr_unknown_len.
         */
        unsigned int pws_hdr_unknown_len;

};

#endif

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
