/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "editentrydialog.h"

/**
 * This dialog displays all values stored in the EntryItem object item.
 *
 * @param item - The EntryItem object to be displayed.
 */
EditEntryDialog::EditEntryDialog(EntryItem *item, QWidget *parent) : QDialog(parent)
{   
    setAttribute (Qt::WA_DeleteOnClose, true);

    m_item = item;
    m_data_changed = false;
    m_history_cleared = false;

    // create the tabs
    m_tab_general  = new EditEntryWidgetTabGeneral(item); // displays all general values
    m_tab_details  = new EditEntryWidgetTabDetails(item); // displays password policy and timestamps
    m_tab_history  = new EditEntryWidgetTabHistory(item); // displays the password history

    // setup the tabs
    m_tab = new QTabWidget(this);
    m_tab->addTab (m_tab_general, "Ge&neral");
    m_tab->addTab (m_tab_details, "&Details");
    m_tab->addTab (m_tab_history, "&History");

    m_ok_btn     = new QPushButton (tr("&Ok"));
    m_cancel_btn = new QPushButton (tr("&Cancel"));

    // buttons should always hav a fixed size
    m_ok_btn->setMinimumWidth  (100);
    m_ok_btn->setMaximumWidth  (100);
    m_cancel_btn->setMinimumWidth  (100);
    m_cancel_btn->setMaximumWidth  (100);

    // setup the layout
    m_grid = new QGridLayout();
    m_grid->addWidget (m_tab, 0,0,1,3);
    m_grid->addWidget (m_cancel_btn, 1,1);
    m_grid->addWidget (m_ok_btn, 1,2);

    setLayout(m_grid);
    setWindowTitle (tr("Edit Entry"));

    // connect all members
    connect (m_tab_history, SIGNAL(fetchHistory()), this, SIGNAL(fetchHistory()));

    connect (m_tab_general, SIGNAL(dataChanged()),  this, SLOT(slot_dataChanged()));
    connect (m_tab_details, SIGNAL(dataChanged()),  this, SLOT(slot_dataChanged()));
    connect (m_tab_history, SIGNAL(dataChanged()),  this, SLOT(slot_dataChanged()));
    connect (m_tab_history, SIGNAL(clearClicked()), this, SLOT(slot_historyCleared()));
    connect (m_tab_general, SIGNAL(fetchClicked()), this, SLOT(slot_fetchPwd()));

    // connect the buttons
    connect (m_ok_btn,     SIGNAL(clicked(bool)), this, SLOT(slot_okClicked()));
    connect (m_cancel_btn, SIGNAL(clicked(bool)), this, SLOT(slot_cancelClicked()));
}

/**
 * Overridden to set the focus at m_name_edt at
 * startup.
 */
void EditEntryDialog::show ()
{
    QWidget::show ();
    m_tab_general->setNameFocus();
}


/**
 * This slot is called when the user selects the "fetch password" button.
 */
void EditEntryDialog::slot_fetchPwd()
{
    // password fetching is handled inside the main window, to make it aware of a changed but not
    // submitted password, this workaround is put here
    if (*m_tab_general->m_pwd != m_item->getPwd())
    {
        QString *old_pwd = new QString (m_item->getPwd());
        m_item->setPwd(*(m_tab_general->m_pwd));

        emit (fetchPwd());

        m_item->setPwd(*old_pwd);
        delete old_pwd;
    }
    else
    {
        emit (fetchPwd());
    }
}

/**
 * This slot is called whenever the data in any of the tabs was changed by the user.
 */
void EditEntryDialog::slot_dataChanged()
{
    if (m_tab_general->m_name_edt->text()         != m_item->getName()  ||
        m_tab_general->m_user_edt->text()         != m_item->getUser()  ||
        m_tab_general->m_pwd                      != m_item->getPwd()   ||
        QUrl (m_tab_general->m_url_edt->text())   != m_item->getUrl()   ||
        m_tab_general->m_notes_edt->toPlainText() != m_item->getNotes() ||
        m_history_cleared)
    {
        m_data_changed = true; 
        return;
    }

    if (m_item->getMaxHistSize() == -1 && 
        m_tab_history->m_numpwd_spn->value() != ConfigStore::store["prefs_int_numpwhistorydefault"].toInt())
    {
        m_data_changed = true; 
        return;
    }

    if (m_item->getMaxHistSize() != -1    &&
        m_tab_history->m_numpwd_spn->value() != m_item->getMaxHistSize())
    {
        m_data_changed = true; 
        return;
    }

    if (m_item->getMaxHistSize() != -1    &&
        m_tab_history->m_numpwd_spn->value() == ConfigStore::store["prefs_int_numpwhistorydefault"].toInt())
    {
        m_data_changed = true; 
        return;
    }

    // password history handling works the following way:
    //  - if password history is disabled by preferences, no action is performed
    //  - if password history is enabled, but not set inside the EntryItem object,
    //    the historical password will be saved
    //  - if password history is enabled, but disabled inside the EntryItem object,
    //    the historical password will not be saved
    //  - if password history is enabled both inside the EntryItem object and by preferences,
    //    the historical password will be saved
    if (m_item->getSavePwdHistory() == 0  &&
        m_tab_history->m_save_history_chk->checkState() == Qt::Checked)
    {
        m_data_changed = true; 
        return;
    }

    if (m_item->getSavePwdHistory() == 1  &&
        m_tab_history->m_save_history_chk->checkState() == Qt::Unchecked)
    {
        m_data_changed = true; 
        return;
    }

    if (m_item->getSavePwdHistory() == -1                           &&
        ConfigStore::store["prefs_bool_savepasswordhistory"] == "1" &&
        m_tab_history->m_save_history_chk->checkState() == Qt::Unchecked)
    {
        m_data_changed = true; 
        return;
    }

    m_data_changed = false;
}

/**
 * This slot is called when the user selected the "clear password history" button
 * inside the password history tab.
 */
void EditEntryDialog::slot_historyCleared()
{
    m_history_cleared = true;
    m_data_changed    = true;
}

/**
 * This slot is called when the user selects the "ok" button and it
 * emits a dataChanged() signal if some of the values inside the
 * tabs where changed.
 */
void EditEntryDialog::slot_okClicked()
{
    if (m_data_changed)
    {
        if (!saveItem())
            { return; }
        emit (dataChanged(m_item));
    }

    close();
}

/**
 * This slot is called when the user selects the "cancel" button. It emits
 * a canceled().
 */
void EditEntryDialog::slot_cancelClicked()
{
    emit (canceled (m_item));
    close();
}

/**
 * Synchronizes the values stored inside the tabs with the actual EntryItem object
 * that is displayed.
 */
bool EditEntryDialog::saveItem()
{
    // get the current time
    QDateTime timestamp = QDateTime::currentDateTime();

    if (m_tab_general->m_name_edt->text() == "")
    {
        // no username is not possible
        QMessageBox::critical(this, tr("Error"), tr("Entry Name cannot be empty."));
        return false;
    }

    if (ConfigStore::store["prefs_bool_savepasswordhistory"] == "1" || m_item->getSavePwdHistory() == 1)
    {
        // save password history if it is enables by preferences
        if (m_tab_general->m_pwd != m_item->getPwd() && m_item->getPwd() != "")
        {
            m_item->addPwdHistoryEntry(m_item->getPwdMTime(), m_item->getPwd()); 
        }
    }

    if (m_tab_general->m_pwd != m_item->getPwd())
    {
        // if the password was changed, record modification time
        m_item->setPwd (*(m_tab_general->m_pwd));
        m_item->setPwdMTime(timestamp);
    }

    // synchronize remaining values
    m_item->rename      (m_tab_general->m_name_edt->text());
    m_item->setUser     (m_tab_general->m_user_edt->text());
    m_item->setUrl      (m_tab_general->m_url_edt->text());
    m_item->setNotes    (m_tab_general->m_notes_edt->toPlainText());
    m_item->setMTime(timestamp);

    // update the password history according to preferences
    if (m_history_cleared)
        { m_item->clearPwdHistory(); }

    // max_hist_size not set inside item and differs from the global value, then set it
    if (m_item->getMaxHistSize() == -1 && m_tab_history->m_numpwd_spn->value() != ConfigStore::store["prefs_int_numpwhistorydefault"].toInt())
        { m_item->setMaxHistSize  (m_tab_history->m_numpwd_spn->value()); }

    // max_hist_size set inside item and differs from the currently set value, then set it to the new value
    if (m_item->getMaxHistSize() != -1 && m_tab_history->m_numpwd_spn->value() != m_item->getMaxHistSize())
        { m_item->setMaxHistSize  (m_tab_history->m_numpwd_spn->value()); }

    // max_hist_size is equal to the global value, then unset it inside item
    if (m_item->getMaxHistSize() == ConfigStore::store["prefs_int_numpwhistorydefault"].toInt())
        { m_item->setMaxHistSize  (-1); }

    // history disabled inside item, but checked inside the tab, then fall back to the global value
    if (m_item->getSavePwdHistory() == 0 && m_tab_history->m_save_history_chk->checkState() == Qt::Checked)
        { m_item->setSavePwdHistory(-1); }

    // history enabled inside item, but unchecked inside the tab, then disable it for item
    if (m_item->getSavePwdHistory() == 1 && m_tab_history->m_save_history_chk->checkState() == Qt::Unchecked)
        { m_item->setSavePwdHistory(0); }

    // history not set inside item, but globally enabled and unchecked inside the tab, then disable it for item
    if (m_item->getSavePwdHistory() == -1 && ConfigStore::store["prefs_bool_savepasswordhistory"] == "1" &&
        m_tab_history->m_save_history_chk->checkState() == Qt::Unchecked)
    { 
        m_item->setSavePwdHistory(0);
        m_item->setMaxHistSize (-1);
    }

    return true;
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
