/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "editentrywidgettabdetails.h"

/**
 * This widget displays the password policy settings and the
 * timestamps for the EntryItem object item.
 *
 * @param item - The EntryItem object to be displayed.
 */
EditEntryWidgetTabDetails::EditEntryWidgetTabDetails(EntryItem *item, QWidget *parent) : QWidget(parent)
{   
    createUuidGroup (item);
    createDateGroup (item);
    createPwdPolicyGroup ();

    // setup the layout
    m_vbox = new QVBoxLayout();
    m_vbox->addWidget (m_uuid_box);
    m_vbox->addWidget (m_date_box);
    m_vbox->addWidget (m_pwd_policy_box);
    m_vbox->setStretchFactor (m_uuid_box, 0);
    m_vbox->setStretchFactor (m_date_box, 0);
    m_vbox->setStretchFactor (m_pwd_policy_box, 255);

    setLayout (m_vbox);

    // PWD Policy checkboxes
    connect (m_hex_chk, SIGNAL(stateChanged(int)),        this, SLOT(slot_hexChkStateChanged(int)));
    connect (m_lower_case_chk, SIGNAL(stateChanged(int)), this, SLOT(slot_otherChkStateChanged(int)));
    connect (m_upper_case_chk, SIGNAL(stateChanged(int)), this, SLOT(slot_otherChkStateChanged(int)));
    connect (m_digits_chk, SIGNAL(stateChanged(int)),     this, SLOT(slot_otherChkStateChanged(int)));
    connect (m_symbols_chk, SIGNAL(stateChanged(int)),    this, SLOT(slot_otherChkStateChanged(int)));
    connect (m_easy_chk, SIGNAL(stateChanged(int)),       this, SLOT(slot_otherChkStateChanged(int)));

    // data changed
    connect (m_hex_chk, SIGNAL(stateChanged(int)),         this, SIGNAL(dataChanged()));
    connect (m_lower_case_chk, SIGNAL(stateChanged(int)),  this, SIGNAL(dataChanged()));
    connect (m_upper_case_chk, SIGNAL(stateChanged(int)),  this, SIGNAL(dataChanged()));
    connect (m_digits_chk, SIGNAL(stateChanged(int)),      this, SIGNAL(dataChanged()));
    connect (m_symbols_chk, SIGNAL(stateChanged(int)),     this, SIGNAL(dataChanged()));
    connect (m_easy_chk, SIGNAL(stateChanged(int)),        this, SIGNAL(dataChanged()));
    connect (m_pwd_length_spin, SIGNAL(valueChanged(int)), this, SIGNAL(dataChanged()));
}

/**
 * This slot will be called when the user selects the "use only hex characters"
 * checkbox and unchecks all other password policy checkboxes.
 */
void EditEntryWidgetTabDetails::slot_hexChkStateChanged(int state)
{
    if (state == Qt::Checked)
    {
        m_lower_case_chk->setCheckState(Qt::Unchecked);
        m_upper_case_chk->setCheckState(Qt::Unchecked);
        m_digits_chk->setCheckState(Qt::Unchecked);
        m_symbols_chk->setCheckState(Qt::Unchecked);
        m_easy_chk->setCheckState(Qt::Unchecked);
    }
}

/**
 * This slot will be called when the user selects any other than the "use only hex characters"
 * password policy checkbox and unchecks the "use only hex characters" checkbox.
 */
void EditEntryWidgetTabDetails::slot_otherChkStateChanged(int state)
{
    if (state == Qt::Checked)
        { m_hex_chk->setCheckState(Qt::Unchecked); }
}

/**
 * Creates the timestamp groupbox.
 */
void EditEntryWidgetTabDetails::createDateGroup(EntryItem *item)
{
    // create the child widgets
    m_mtime_lbl      = new QLabel(tr("Modification time: "));
    m_pwd_mtime_lbl  = new QLabel(tr("Password modification time: "));
    m_ctime_lbl      = new QLabel(tr("Creation time: "));
    m_atime_lbl      = new QLabel(tr("Access time: "));

    if (item->getMTime() != QDateTime::fromTime_t(0))
        { m_mtime_dat = new QLabel(item->getMTime().toString(ConfigStore::store["prefs_pws_date_format"])); }
    else
        { m_mtime_dat = new QLabel(); }

    if (item->getPwdMTime() != QDateTime::fromTime_t(0))
        { m_pwd_mtime_dat = new QLabel(item->getPwdMTime().toString(ConfigStore::store["prefs_pws_date_format"])); }
    else
        { m_pwd_mtime_dat = new QLabel(); }

    if (item->getCTime() != QDateTime::fromTime_t(0))
        { m_ctime_dat     = new QLabel(item->getCTime().toString(ConfigStore::store["prefs_pws_date_format"])); }
    else
        { m_ctime_dat = new QLabel(); }

    if (item->getATime() != QDateTime::fromTime_t(0))
        { m_atime_dat     = new QLabel(item->getATime().toString(ConfigStore::store["prefs_pws_date_format"])); }
    else
        { m_atime_dat = new QLabel(); }

    // setup the layout
    m_date_grid = new QGridLayout();
    m_date_grid->addWidget (m_mtime_lbl,     0,0, 1,2);
    m_date_grid->addWidget (m_mtime_dat,     0,2);
    m_date_grid->addWidget (m_pwd_mtime_lbl, 1,0, 1,2);
    m_date_grid->addWidget (m_pwd_mtime_dat, 1,2);
    m_date_grid->addWidget (m_ctime_lbl,     2,0, 1,2);
    m_date_grid->addWidget (m_ctime_dat,     2,2);
    m_date_grid->addWidget (m_atime_lbl,     3,0, 1,2);
    m_date_grid->addWidget (m_atime_dat,     3,2);

    m_date_box = new QGroupBox(tr("Date/Time information"));
    m_date_box->setLayout (m_date_grid);

}

/**
 * Creates the uuid groupbox.
 */
void EditEntryWidgetTabDetails::createUuidGroup(EntryItem *item)
{
    m_uuid_dat = new QLabel(item->getUuid().toString());

    m_uuid_vbox = new QVBoxLayout();
    m_uuid_vbox->addWidget (m_uuid_dat);

    m_uuid_box = new QGroupBox(tr("Uuid"));
    m_uuid_box->setLayout (m_uuid_vbox);
}

/**
 * Creates the password policy group box.
 */
void EditEntryWidgetTabDetails::createPwdPolicyGroup()
{
    // create the child widgets
    m_pwd_length_lbl = new QLabel(tr("Password length "));

    m_lower_case_chk = new QCheckBox (tr("Use lower case characters"));
    m_upper_case_chk = new QCheckBox (tr("Use upper case characters"));
    m_digits_chk     = new QCheckBox (tr("Use digits"));
    m_symbols_chk    = new QCheckBox (tr("Use symbols ('%','$','!',...)"));
    m_easy_chk       = new QCheckBox (tr("Use (only) easy to read characters (no 'l','1',...)"));
    m_hex_chk        = new QCheckBox (tr("Use (only) hexadecimal characters"));

    m_pwd_length_spin = new QSpinBox();
    m_pwd_length_spin->setRange (1, 99);

    // setup the layout
    m_pwd_policy_grid = new QGridLayout();
    m_pwd_policy_grid->addWidget (m_pwd_length_lbl,  0,0);
    m_pwd_policy_grid->addWidget (m_pwd_length_spin, 0,1);
    m_pwd_policy_grid->addWidget (m_lower_case_chk,  1,0, 1,2);
    m_pwd_policy_grid->addWidget (m_upper_case_chk,  2,0, 1,2);
    m_pwd_policy_grid->addWidget (m_digits_chk,      3,0, 1,2);
    m_pwd_policy_grid->addWidget (m_symbols_chk,     4,0, 1,2);
    m_pwd_policy_grid->addWidget (m_easy_chk,        5,0, 1,2);
    m_pwd_policy_grid->addWidget (m_hex_chk,         6,0, 1,2);

    m_pwd_policy_box = new QGroupBox(tr("Password policy rules (global)"));
    m_pwd_policy_box->setLayout (m_pwd_policy_grid);

    m_pwd_length_spin->setValue(ConfigStore::store["prefs_int_pwlendefault"].toInt());

    if (ConfigStore::store["prefs_bool_pwuselowercase"] == "1")
        { m_lower_case_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwuseuppercase"] == "1")
        { m_upper_case_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwusedigits"] == "1")
        { m_digits_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwusesysmbols"] == "1")
        { m_symbols_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwusehexdigit"] == "1")
        { m_easy_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pweasyvision"] == "1")
        { m_hex_chk->setCheckState (Qt::Checked); }

    // per entry policy currently not supported (PWSv3.2 limitation)
    m_pwd_policy_box->setDisabled (true);
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
