/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "editentrywidgettabhistory.h"

/**
 * This widget displays the passowrd history stored in the EntryItem object item.
 *
 * @param item - The EntryItem object containing the password history that should be
 *               displayed.
 */
EditEntryWidgetTabHistory::EditEntryWidgetTabHistory(EntryItem *item, QWidget *parent) : QWidget(parent)
{   
    // create all child widgets
    m_table = new QTableWidget(item->getPwdHistory().size(), 2);

    QStringList header;
    header.append(tr("Timestamp"));
    header.append(tr("Password"));

    m_table->setHorizontalHeaderLabels (header);

    // setup the table rows containing the password history
    int x = 0;
    QListIterator<HistoryEntry> iter (item->getPwdHistory());
    iter.toBack();
    while (iter.hasPrevious()) 
    {
        HistoryEntry entry = iter.previous();
        if (entry.timestamp == 0)
        {
            QTableWidgetItem *item = new QTableWidgetItem(tr("N/A")); 
            m_table->setItem(x, 0, item); 
            item->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter); 
        }
        else
        {
            m_table->setItem(x, 0, new QTableWidgetItem(QDateTime::fromTime_t(entry.timestamp).toString("yyyy-MM-dd hh:mm:ss"))); 
        }

        m_table->setItem(x, 1, new QTableWidgetItem(*entry.password));
        x++;
    }

    m_table->horizontalHeader()->setStretchLastSection (true);
    m_table->verticalHeader()->hide();

    m_table->resizeColumnToContents (0);
    m_table->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_table->setSelectionMode(QAbstractItemView::NoSelection);
  
    m_save_history_chk = new QCheckBox (tr("Keep "));
    m_numpwd_lbl       = new QLabel (tr(" last passwords"));
    m_numpwd_spn       = new QSpinBox();
    m_numpwd_spn->setRange (1, 99);

    m_clear_btn = new QPushButton (tr("&Clear"));
    m_copy_btn  = new QPushButton (tr("Copy to cli&pboard"));

    m_options_grid = new QGridLayout ();
    m_options_grid->addWidget (m_save_history_chk, 0,0);
    m_options_grid->addWidget (m_numpwd_spn,       0,1);
    m_options_grid->addWidget (m_numpwd_lbl,       0,2);
    m_options_grid->setColumnStretch (3, 255); 

    m_options_box = new QGroupBox (tr("Password history options"));
    m_options_box->setLayout(m_options_grid);

    // setup the layout
    m_grid = new QGridLayout();
    m_grid->addWidget (m_options_box, 0,0, 1,2);
    m_grid->addWidget (m_table,       1,0, 1,2);
    m_grid->addWidget (m_clear_btn,   2,0);
    m_grid->addWidget (m_copy_btn,    2,1);

    if (ConfigStore::store["prefs_bool_savepasswordhistory"] == "0")
    {
        m_options_box->setTitle (tr("Password history options (globally disabled)"));
        setDisabled (true); 
    }

    if (ConfigStore::store["pws_hdr_version"].startsWith("2"))
    {
        m_options_box->setTitle (tr("Password history options (not supported in PWSv2 files)"));
        setDisabled (true); 
    }

    if (ConfigStore::store["readonly"] == "1")
    {
        m_clear_btn->setDisabled (true); 
        m_options_box->setDisabled (true);
    }

    if (item->getSavePwdHistory() == 1)
        { m_save_history_chk->setCheckState(Qt::Checked); }
    else if (item->getSavePwdHistory() == 0)
        { m_save_history_chk->setCheckState(Qt::Unchecked); }
    else if (item->getSavePwdHistory() == -1 && ConfigStore::store["prefs_bool_savepasswordhistory"] == "1")
        { m_save_history_chk->setCheckState(Qt::Checked); }


    if (item->getMaxHistSize() != -1)
        { m_numpwd_spn->setValue(item->getMaxHistSize()); }
    else 
        { m_numpwd_spn->setValue(ConfigStore::store["prefs_int_numpwhistorydefault"].toInt()); }

    setLayout (m_grid);

    // connect the child widgets
    connect (m_clear_btn,        SIGNAL(clicked(bool)),           this,    SLOT(slot_clearClicked()));
    connect (m_table,            SIGNAL(itemSelectionChanged()),  m_table, SLOT(clearSelection()));

    connect (m_copy_btn,         SIGNAL(clicked(bool)),           this, SIGNAL(fetchHistory()));
    connect (m_save_history_chk, SIGNAL(stateChanged(int)),       this, SIGNAL(dataChanged()));
    connect (m_save_history_chk, SIGNAL(stateChanged(int)),       this, SLOT(slot_saveHistoryClicked()));
    connect (m_numpwd_spn,       SIGNAL(valueChanged(int)),       this, SIGNAL(dataChanged()));
}

/**
 * This slot is called when the user selects the "clear" button.
 * It clears the password history and emitts a clearClicked signal.
 */
void EditEntryWidgetTabHistory::slot_clearClicked()
{
    m_table->clear();
    m_table->setRowCount(0);

    QStringList header;
    header.append(tr("Timestamp"));
    header.append(tr("Password"));

    m_table->setHorizontalHeaderLabels (header);

    emit clearClicked();
}

/**
 * This slot is called when the user selects the "save password history" checkbox and
 * enables, resp. disables the password history feature.
 */
void EditEntryWidgetTabHistory::slot_saveHistoryClicked()
{
    if (m_save_history_chk->checkState() == Qt::Checked)
        { m_numpwd_spn->setEnabled (true); }
    else
        { m_numpwd_spn->setEnabled (false); }
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
