/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "preferenceswidgettabpolicy.h"

/**
 * This widgets represents the password policy and history preferences tab.
 */
PreferencesWidgetTabPolicy::PreferencesWidgetTabPolicy(QWidget *parent) : QWidget(parent)
{
    createPwdPolicyGroup ();
    createPwdHistoryGroup ();

    // setup the layout
    m_vbox = new QVBoxLayout();
    m_vbox->addWidget (m_pwd_policy_box);
    m_vbox->addWidget (m_pwd_history_box);
    m_vbox->setStretchFactor (m_pwd_history_box, 255);

    setLayout (m_vbox);

    // connect the PWD Policy checkboxes
    connect (m_hex_chk,        SIGNAL(stateChanged(int)), this, SLOT(slot_hexChkStateChanged(int)));

    connect (m_easy_chk,       SIGNAL(stateChanged(int)), this, SLOT(slot_easyChkChanged(int)));
    connect (m_lower_case_chk, SIGNAL(stateChanged(int)), this, SLOT(slot_lowerChkChanged(int)));
    connect (m_upper_case_chk, SIGNAL(stateChanged(int)), this, SLOT(slot_upperChkChanged(int)));
    connect (m_digits_chk,     SIGNAL(stateChanged(int)), this, SLOT(slot_digitsChkChanged(int)));
    connect (m_symbols_chk,    SIGNAL(stateChanged(int)), this, SLOT(slot_symbolsChkChanged(int)));
}

/**
 * This slot is called whenever the user selects the "use only easy characters"
 * checkbox.
 */
void PreferencesWidgetTabPolicy::slot_easyChkChanged(int state)
{
    if (state == Qt::Checked)
    {
        m_hex_chk->setCheckState(Qt::Unchecked); 
    }
}

/**
 * This slot is called whenever the user selects the "use lower case characters"
 * checkbox.
 */
void PreferencesWidgetTabPolicy::slot_lowerChkChanged(int state)
{
    if (state == Qt::Checked)
    {
        m_hex_chk->setCheckState(Qt::Unchecked); 
    }
    else if (m_hex_chk->checkState()        != Qt::Checked &&
             m_upper_case_chk->checkState() != Qt::Checked &&
             m_digits_chk->checkState()     != Qt::Checked &&
             m_symbols_chk->checkState()    != Qt::Checked)
    {
        m_lower_case_chk->setCheckState (Qt::Checked);
    }
}

/**
 * This slot is called whenever the user selects the "use upper case characters"
 * checkbox.
 */
void PreferencesWidgetTabPolicy::slot_upperChkChanged(int state)
{
    if (state == Qt::Checked)
    {
        m_hex_chk->setCheckState(Qt::Unchecked); 
    }
    else if (m_hex_chk->checkState()        != Qt::Checked &&
             m_lower_case_chk->checkState() != Qt::Checked &&
             m_digits_chk->checkState()     != Qt::Checked &&
             m_symbols_chk->checkState()    != Qt::Checked)
    {
        m_upper_case_chk->setCheckState (Qt::Checked);
    }
}

/**
 * This slot is called whenever the user selects the "use digits characters"
 * checkbox.
 */
void PreferencesWidgetTabPolicy::slot_digitsChkChanged(int state)
{
    if (state == Qt::Checked)
    {
        m_hex_chk->setCheckState(Qt::Unchecked); 
    }
    else if (m_hex_chk->checkState()        != Qt::Checked &&
             m_lower_case_chk->checkState() != Qt::Checked &&
             m_upper_case_chk->checkState() != Qt::Checked &&
             m_symbols_chk->checkState()    != Qt::Checked)
    {
        m_digits_chk->setCheckState (Qt::Checked);
    }
}

/**
 * This slot is called whenever the user selects the "use symbols characters"
 * checkbox.
 */
void PreferencesWidgetTabPolicy::slot_symbolsChkChanged(int state)
{
    if (state == Qt::Checked)
    {
        m_hex_chk->setCheckState(Qt::Unchecked); 
    }
    else if (m_hex_chk->checkState()        != Qt::Checked &&
             m_lower_case_chk->checkState() != Qt::Checked &&
             m_upper_case_chk->checkState() != Qt::Checked &&
             m_digits_chk->checkState()     != Qt::Checked)
    {
        m_symbols_chk->setCheckState (Qt::Checked);
    }
}

/**
 * This slot is called whenever the user selects the "use only hex characters"
 * checkbox and unchecks all other password policy checkboxes.
 */
void PreferencesWidgetTabPolicy::slot_hexChkStateChanged(int state)
{
    if (state == Qt::Checked)
    {
        m_lower_case_chk->setCheckState(Qt::Unchecked);
        m_upper_case_chk->setCheckState(Qt::Unchecked);
        m_digits_chk->setCheckState(Qt::Unchecked);
        m_symbols_chk->setCheckState(Qt::Unchecked);
        m_easy_chk->setCheckState(Qt::Unchecked);
    }
}

/**
 * Creates the password history group box.
 */
void PreferencesWidgetTabPolicy::createPwdHistoryGroup()
{
    // create the child widgets
    m_save_pwd_chk = new QCheckBox (tr("Save password history and save max. entries: "));

    m_max_hst_entries_spn = new QSpinBox();
    m_max_hst_entries_spn->setRange (1, 99);
    m_max_hst_entries_spn->setValue (ConfigStore::store["prefs_int_numpwhistorydefault"].toInt());

    // setup the layout
    m_pwd_history_grid = new QGridLayout();
    m_pwd_history_grid->addWidget (m_save_pwd_chk,         0,0);
    m_pwd_history_grid->addWidget (m_max_hst_entries_spn,  0,1);

    m_pwd_history_box = new QGroupBox(tr("Password history"));
    m_pwd_history_box->setLayout (m_pwd_history_grid);

    m_pwd_history_grid->setRowStretch (0, 0);
    m_pwd_history_grid->setRowStretch (1, 255);

    // set all values according to the settings in ConfigStore
    if (ConfigStore::store["prefs_bool_savepasswordhistory"] == "1")
    {
        m_save_pwd_chk->setCheckState(Qt::Checked);
    }

    if (ConfigStore::store["pws_hdr_version"].startsWith("2"))
    {
        m_pwd_history_box->setTitle (tr("Password history (not supported in PWSv2 files)"));
        m_pwd_history_box->setDisabled (true); 
    }

}

/**
 * Creates the password policy group box.
 */
void PreferencesWidgetTabPolicy::createPwdPolicyGroup()
{
    // create the child widgets
    m_pwd_length_lbl = new QLabel(tr("Password length "));

    m_lower_case_chk = new QCheckBox (tr("Use lower case characters"));
    m_upper_case_chk = new QCheckBox (tr("Use upper case characters"));
    m_digits_chk     = new QCheckBox (tr("Use digits"));
    m_symbols_chk    = new QCheckBox (tr("Use symbols ('%','$','!',...)"));
    m_easy_chk       = new QCheckBox (tr("Use (only) easy to read characters (no 'l','1',...)"));
    m_hex_chk        = new QCheckBox (tr("Use (only) hexadecimal characters"));

    m_pwd_length_spin = new QSpinBox();
    m_pwd_length_spin->setRange (1, 99);

    // setup the layout
    m_pwd_policy_grid = new QGridLayout();
    m_pwd_policy_grid->addWidget (m_pwd_length_lbl,  0,0);
    m_pwd_policy_grid->addWidget (m_pwd_length_spin, 0,1);
    m_pwd_policy_grid->addWidget (m_lower_case_chk,  1,0, 1,2);
    m_pwd_policy_grid->addWidget (m_upper_case_chk,  2,0, 1,2);
    m_pwd_policy_grid->addWidget (m_digits_chk,      3,0, 1,2);
    m_pwd_policy_grid->addWidget (m_symbols_chk,     4,0, 1,2);
    m_pwd_policy_grid->addWidget (m_easy_chk,        5,0, 1,2);
    m_pwd_policy_grid->addWidget (m_hex_chk,         6,0, 1,2);

    m_pwd_policy_grid->setRowStretch (0, 0);
    m_pwd_policy_grid->setRowStretch (1, 0);
    m_pwd_policy_grid->setRowStretch (3, 0);
    m_pwd_policy_grid->setRowStretch (4, 0);
    m_pwd_policy_grid->setRowStretch (5, 0);
    m_pwd_policy_grid->setRowStretch (6, 0);
    m_pwd_policy_grid->setRowStretch (7, 255);

    m_pwd_policy_box = new QGroupBox(tr("Password policy rules (global)"));
    m_pwd_policy_box->setLayout (m_pwd_policy_grid);

    // set all values according to the settings in ConfigStore
    m_pwd_length_spin->setValue(ConfigStore::store["prefs_int_pwlendefault"].toInt());

    if (ConfigStore::store["prefs_bool_pwuselowercase"] == "1")
        { m_lower_case_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwuseuppercase"] == "1")
        { m_upper_case_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwusedigits"] == "1")
        { m_digits_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwusesysmbols"] == "1")
        { m_symbols_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pweasyvision"] == "1")
        { m_easy_chk->setCheckState (Qt::Checked); }

    if (ConfigStore::store["prefs_bool_pwusehexdigit"] == "1")
        { m_hex_chk->setCheckState (Qt::Checked); }
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
