/***************************************************************************
 *                                                                         *
 *   copyright (C) 2007-2008 by Timo Benk                                  *
 *   email: tbenk@pwsafe.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "startdialog.h"

/**
 * This dialog is shown when pws is started.
 */
StartDialog::StartDialog(QWidget *parent) : QDialog(parent)
{   
    setAttribute (Qt::WA_DeleteOnClose, true);

    // create all child widgets
    m_ok_btn     = new QPushButton (tr("&Ok"));
    m_cancel_btn = new QPushButton (tr("&Cancel"));
    m_new_btn    = new QPushButton (tr("&New"));
    m_dots_btn   = new QPushButton ("...");

    // buttons always should have a fixed size
    m_ok_btn->setMinimumWidth     (120);
    m_ok_btn->setMaximumWidth     (120);
    m_cancel_btn->setMinimumWidth (120);
    m_cancel_btn->setMaximumWidth (120);
    m_new_btn->setMinimumWidth    (120);
    m_new_btn->setMaximumWidth    (120);

    m_dots_btn->setMinimumWidth    (40);
    m_dots_btn->setMaximumWidth    (40);

    m_pwd_lbl = new QLabel (tr("Passphrase"));
    m_recent_lbl = new QLabel (tr("Open passwordsafe"));

    m_pwd_edt = new QLineEdit();
    m_pwd_edt->setMaximumWidth (380);
    m_recent_edt = new QLineEdit();

    m_pwd_edt->setEchoMode (QLineEdit::Password);

    m_recent_cmb = new QComboBox();
    m_recent_cmb->setLineEdit(m_recent_edt);
    m_recent_cmb->setMaximumWidth (380);

    // show the last recent files inside the combo box
    QStringList files = ConfigStore::store["prefs_pws_recentfiles"].split("$");
    for (int x = 0; x < files.size(); x++)
    {
        m_recent_cmb->addItem(files.at(x));
    }

    m_readonly_chk = new QCheckBox (tr("Open readonly"));

    // setup the grid layout
    m_grid = new QGridLayout();
    m_grid->addWidget (m_recent_lbl, 0,0, 1,3);
    m_grid->addWidget (m_recent_cmb, 1,0, 1,3);
    m_grid->addWidget (m_dots_btn,   1,3);
    m_grid->addWidget (m_pwd_lbl,    2,0, 1,3);
    m_grid->addWidget (m_pwd_edt,    3,0, 1,3);
    m_grid->addWidget (m_readonly_chk, 4,0);
    m_grid->addWidget (m_ok_btn,     5,0);
    m_grid->addWidget (m_cancel_btn, 5,1);
    m_grid->addWidget (m_new_btn,    5,2);

    setLayout(m_grid);
    setWindowTitle (tr("Open passwordsafe"));

    // and connect the child widgets
    connect (m_cancel_btn, SIGNAL(clicked(bool)), this, SIGNAL(cancelClicked()));
    connect (m_ok_btn,     SIGNAL(clicked(bool)), this, SLOT(slot_okClicked()));
    connect (m_new_btn,    SIGNAL(clicked(bool)), this, SLOT(slot_newClicked()));
    connect (m_dots_btn,   SIGNAL(clicked(bool)), this, SLOT(slot_dotsClicked()));
    connect (m_dots_btn,   SIGNAL(clicked(bool)), this, SLOT(slot_setEditFocus()));
    connect (m_recent_cmb, SIGNAL(currentIndexChanged(int)), this, SLOT(slot_setEditFocus()));

    // submit on enter
    (void) new QShortcut(Qt::Key_Enter, this, SLOT(slot_okClicked()));
    (void) new QShortcut(Qt::Key_Return, this, SLOT(slot_okClicked()));

    // just for convenience, position the focus inside the passphrase field
    slot_setEditFocus();

    adjustSize();
}

/**
 * This slot is called when the user selects the "new" button.
 */
void StartDialog::slot_newClicked()
{
    NewPassphraseDialog *pwd_dlg = new NewPassphraseDialog(this);
    pwd_dlg->setWindowModality(Qt::ApplicationModal);

    connect (pwd_dlg, SIGNAL(submit(const QString &)), this, SIGNAL(newClicked(const QString &)));
    pwd_dlg->show();
}

/**
 * This slot is called when the user entered a wrong passphrase.
 */
void StartDialog::slot_wrongPassphrase()
{
    m_pwd_edt->selectAll();
}

/**
 * This slot is called when the user selects the "ok" button.
 */
void StartDialog::slot_okClicked()
{
    if (m_pwd_edt->text() == "" && m_recent_cmb->hasFocus())
    {
        // position the focus inside the passphrase field when it is empty and
        // the current focus is held by the combobox
        slot_setEditFocus();
        return;
    }

    if (m_recent_cmb->currentText() == "")
    { 
        // empty filename is not possible
        QMessageBox::critical(this, tr("Error"), tr("No filename given")); 
        return;
    }

    if (m_pwd_edt->text() == "")
    { 
        // empty passphrase is not possible
        QMessageBox::critical(this, tr("Error"), tr("No passphrase given")); 
        return;
    }
        
    ConfigStore::store["curfile"] = m_recent_cmb->currentText();

    // at this point the password safe is not opened. if the passphrase
    // will work is checked inside MainWindow. This dialog will be closed 
    // also from MainWindow.
    if (m_readonly_chk->checkState() == Qt::Checked)
        { emit okClicked(m_pwd_edt->text(), true); }
    else
        { emit okClicked(m_pwd_edt->text(), false); }

    slot_setEditFocus();

}

/**
 * This slot is called when the user selects the dots button.
 */
void StartDialog::slot_dotsClicked()
{
    QString filename = QFileDialog::getOpenFileName(this, tr("Open passwordsafe"), ConfigStore::store["prefs_pws_cur_dir"],
      tr("Passwordsafe v[23] format (*.psafe3 *.dat);;All files (*)"));

    if (!filename.isEmpty())
    {
        ConfigStore::store["prefs_pws_cur_dir"] = QFileInfo (filename).absolutePath();
        m_recent_cmb->setEditText (QFileInfo(filename).absoluteFilePath());
    }
}

/**
 * Set the focus to the passphrase field.
 */
void StartDialog::slot_setEditFocus()
{
    m_pwd_edt->setFocus(Qt::OtherFocusReason);
}

/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap: */
